@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "mTunnel" multicast access service
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // Helper routines to implement 'group sockets'
 // Implementation
 
@@ -39,6 +39,16 @@ static void socketErr(UsageEnvironment& env, char* errorMsg) {
 	env.setResultErrMsg(errorMsg);
 }
 
+static int reuseFlag = 1;
+
+NoReuse::NoReuse() {
+  reuseFlag = 0;
+}
+
+NoReuse::~NoReuse() {
+  reuseFlag = 1;
+}
+
 int setupDatagramSocket(UsageEnvironment& env, Port port,
 #ifdef IP_MULTICAST_LOOP
 			Boolean setLoopback
@@ -57,22 +67,21 @@ int setupDatagramSocket(UsageEnvironment& env, Port port,
     return newSocket;
   }
   
-  const int reuseFlag = 1;
   if (setsockopt(newSocket, SOL_SOCKET, SO_REUSEADDR,
 		 (const char*)&reuseFlag, sizeof reuseFlag) < 0) {
     socketErr(env, "setsockopt(SO_REUSEADDR) error: ");
-    _close(newSocket);
+    closeSocket(newSocket);
     return -1;
   }
   
 #if defined(__WIN32__) || defined(_WIN32)
-  // Windoze doesn't handle SO_REUSEPORT or IP_MULTICAST_LOOP
+  // Windoze doesn't properly handle SO_REUSEPORT or IP_MULTICAST_LOOP
 #else
 #ifdef SO_REUSEPORT
   if (setsockopt(newSocket, SOL_SOCKET, SO_REUSEPORT,
 		 (const char*)&reuseFlag, sizeof reuseFlag) < 0) {
     socketErr(env, "setsockopt(SO_REUSEPORT) error: ");
-    _close(newSocket);
+    closeSocket(newSocket);
     return -1;
   }
 #endif
@@ -82,7 +91,7 @@ int setupDatagramSocket(UsageEnvironment& env, Port port,
   if (setsockopt(newSocket, IPPROTO_IP, IP_MULTICAST_LOOP,
 		 (const char*)&loop, sizeof loop) < 0) {
     socketErr(env, "setsockopt(IP_MULTICAST_LOOP) error: ");
-    _close(newSocket);
+    closeSocket(newSocket);
     return -1;
   }
 #endif
@@ -91,18 +100,15 @@ int setupDatagramSocket(UsageEnvironment& env, Port port,
   // Note: Windoze requires binding, even if the port number is 0
 #if defined(__WIN32__) || defined(_WIN32)
 #else
-  if (port.num() != 0) {
+  if (port.num() != 0 || ReceivingInterfaceAddr != INADDR_ANY) {
 #endif
-    struct sockaddr_in name;
-    name.sin_family = AF_INET;
-    name.sin_port = port.num();
-    name.sin_addr.s_addr = ReceivingInterfaceAddr;
+    MAKE_SOCKADDR_IN(name, ReceivingInterfaceAddr, port.num());
     if (bind(newSocket, (struct sockaddr*)&name, sizeof name) != 0) {
       char tmpBuffer[100];
       sprintf(tmpBuffer, "bind() error (port number: %d): ",
 	      ntohs(port.num()));
       socketErr(env, tmpBuffer);
-      _close(newSocket);
+      closeSocket(newSocket);
       return -1;
     }
 #if defined(__WIN32__) || defined(_WIN32)
@@ -118,7 +124,7 @@ int setupDatagramSocket(UsageEnvironment& env, Port port,
     if (setsockopt(newSocket, IPPROTO_IP, IP_MULTICAST_IF,
 		   (const char*)&addr, sizeof addr) < 0) {
       socketErr(env, "error setting outgoing multicast interface: ");
-      _close(newSocket);
+      closeSocket(newSocket);
       return -1;
     }
   }
@@ -139,11 +145,10 @@ int setupStreamSocket(UsageEnvironment& env,
     return newSocket;
   }
   
-  const int reuseFlag = 1;
   if (setsockopt(newSocket, SOL_SOCKET, SO_REUSEADDR,
 		 (const char*)&reuseFlag, sizeof reuseFlag) < 0) {
     socketErr(env, "setsockopt(SO_REUSEADDR) error: ");
-    _close(newSocket);
+    closeSocket(newSocket);
     return -1;
   }
   
@@ -152,13 +157,13 @@ int setupStreamSocket(UsageEnvironment& env,
   // #define REUSE_FOR_TCP
 #ifdef REUSE_FOR_TCP
 #if defined(__WIN32__) || defined(_WIN32)
-    // Windoze doesn't handle SO_REUSEPORT
+  // Windoze doesn't properly handle SO_REUSEPORT
 #else
 #ifdef SO_REUSEPORT
   if (setsockopt(newSocket, SOL_SOCKET, SO_REUSEPORT,
 		 (const char*)&reuseFlag, sizeof reuseFlag) < 0) {
     socketErr(env, "setsockopt(SO_REUSEPORT) error: ");
-    _close(newSocket);
+    closeSocket(newSocket);
     return -1;
   }
 #endif
@@ -168,18 +173,15 @@ int setupStreamSocket(UsageEnvironment& env,
   // Note: Windoze requires binding, even if the port number is 0
 #if defined(__WIN32__) || defined(_WIN32)
 #else
-  if (port.num() != 0) {
+  if (port.num() != 0 || ReceivingInterfaceAddr != INADDR_ANY) {
 #endif
-    struct sockaddr_in name;
-    name.sin_family = AF_INET;
-    name.sin_port = port.num();
-    name.sin_addr.s_addr = ReceivingInterfaceAddr;
+    MAKE_SOCKADDR_IN(name, ReceivingInterfaceAddr, port.num());
     if (bind(newSocket, (struct sockaddr*)&name, sizeof name) != 0) {
       char tmpBuffer[100];
       sprintf(tmpBuffer, "bind() error (port number: %d): ",
 	      ntohs(port.num()));
       socketErr(env, tmpBuffer);
-      _close(newSocket);
+      closeSocket(newSocket);
       return -1;
     }
 #if defined(__WIN32__) || defined(_WIN32)
@@ -202,7 +204,7 @@ int setupStreamSocket(UsageEnvironment& env,
     if (fcntl(newSocket, F_SETFL, curFlags|O_NONBLOCK) < 0) {
 #endif
       socketErr(env, "failed to make non-blocking: ");
-      _close(newSocket);
+      closeSocket(newSocket);
       return -1;
     }
   }
@@ -225,6 +227,10 @@ static int blockUntilReadable(UsageEnvironment& env,
     if (timeout != NULL && result == 0) {
       break; // this is OK - timeout occurred
     } else if (result <= 0) {
+#if defined(__WIN32__) || defined(_WIN32)
+#else
+      if (errno == EINTR || errno == EAGAIN) continue;
+#endif
       socketErr(env, "select() error: ");
       break;
     }
@@ -332,11 +338,7 @@ Boolean writeSocket(UsageEnvironment& env,
 			}
 		}
 
-		struct sockaddr_in dest;
-    		dest.sin_family = AF_INET;
-		dest.sin_port = port.num();
-		dest.sin_addr = address;
-
+		MAKE_SOCKADDR_IN(dest, address.s_addr, port.num());
 		int bytesSent = sendto(socket, (char*)buffer, bufferSize, 0,
 			               (struct sockaddr*)&dest, sizeof dest);
 		if (bytesSent != (int)bufferSize) {
@@ -426,8 +428,16 @@ Boolean socketJoinGroup(UsageEnvironment& env, int socket,
   imr.imr_interface.s_addr = ReceivingInterfaceAddr;
   if (setsockopt(socket, IPPROTO_IP, IP_ADD_MEMBERSHIP,
 		 (const char*)&imr, sizeof (struct ip_mreq)) < 0) {
-    socketErr(env, "setsockopt(IP_ADD_MEMBERSHIP) error: ");
-    return False;
+#if defined(__WIN32__) || defined(_WIN32)
+    if (env.getErrno() != 0) {
+      // That piece-of-shit toy operating system (Windows) sometimes lies
+      // about setsockopt() failing!
+#endif
+      socketErr(env, "setsockopt(IP_ADD_MEMBERSHIP) error: ");
+      return False;
+#if defined(__WIN32__) || defined(_WIN32)
+    }
+#endif
   }
   
   return True;
@@ -456,8 +466,8 @@ Boolean socketLeaveGroup(UsageEnvironment&, int socket,
 #define IP_ADD_SOURCE_MEMBERSHIP   39
 #define IP_DROP_SOURCE_MEMBERSHIP 40
 #else
-#define IP_ADD_SOURCE_MEMBERSHIP   67
-#define IP_DROP_SOURCE_MEMBERSHIP 68
+#define IP_ADD_SOURCE_MEMBERSHIP   25
+#define IP_DROP_SOURCE_MEMBERSHIP 26
 #endif
 
 struct ip_mreq_source {
@@ -515,10 +525,7 @@ Boolean getSourcePort(UsageEnvironment& env, int socket, Port& port) {
   portNumBits portNum = 0;
   if (!getSourcePort0(socket, portNum) || portNum == 0) {
     // Hack - call bind(), then try again:
-    struct sockaddr_in name;
-    name.sin_family = AF_INET;
-    name.sin_port = 0;
-    name.sin_addr.s_addr = INADDR_ANY;
+    MAKE_SOCKADDR_IN(name, INADDR_ANY, 0);
     bind(socket, (struct sockaddr*)&name, sizeof name);
 
     if (!getSourcePort0(socket, portNum) || portNum == 0) {
@@ -542,126 +549,124 @@ static Boolean badAddress(netAddressBits addr) {
 Boolean loopbackWorks = 1;
 
 netAddressBits ourSourceAddressForMulticast(UsageEnvironment& env) {
-  	static netAddressBits ourAddress = 0;
-	int sock = -1;
-	struct in_addr testAddr;
+  static netAddressBits ourAddress = 0;
+  int sock = -1;
+  struct in_addr testAddr;
+  
+  if (ourAddress == 0) {
+    // We need to find our source address
+    struct sockaddr_in fromAddr;
+    fromAddr.sin_addr.s_addr = 0;
+    
+    // Get our address by sending a (0-TTL) multicast packet,
+    // receiving it, and looking at the source address used.
+    // (This is kinda bogus, but it provides the best guarantee
+    // that other nodes will think our address is the same as we do.)
+    do {
+      loopbackWorks = 0; // until we learn otherwise
 
-	if (ourAddress == 0) do {
-		// We need to find our source address
+      testAddr.s_addr = our_inet_addr("228.67.43.91"); // arbitrary
+      Port testPort(15947); // ditto
+      
+      sock = setupDatagramSocket(env, testPort);
+      if (sock < 0) break;
+      
+      if (!socketJoinGroup(env, sock, testAddr.s_addr)) break;
+      
+      unsigned char testString[] = "hostIdTest";
+      unsigned testStringLength = sizeof testString;
+      
+      if (!writeSocket(env, sock, testAddr, testPort, 0,
+		       testString, testStringLength)) break;
+      
+      unsigned char readBuffer[20];
+      struct timeval timeout;
+      timeout.tv_sec = 5;
+      timeout.tv_usec = 0;
+      int bytesRead = readSocket(env, sock,
+				 readBuffer, sizeof readBuffer,
+				 fromAddr, &timeout);
+      if (bytesRead == 0 // timeout occurred
+	  || bytesRead != (int)testStringLength
+	  || strncmp((char*)readBuffer, (char*)testString,
+		     testStringLength) != 0) {
+	break;
+      }
 
-		// Get our address by sending a (0-TTL) multicast packet,
-		// receiving it, and looking at the source address used.
-		// (This is kinda bogus, but it provides the best guarantee
-		// that other nodes will think our id is the same as we do.)
-		// (This is a gross hack; there must be a better way!!!)
-	
-		testAddr.s_addr = our_inet_addr("228.67.43.91"); // arbitrary
-		Port testPort(15947); // ditto
+      loopbackWorks = 1;
+    } while (0);
 
-		sock = setupDatagramSocket(env, testPort);
-		if (sock < 0) break;
-
-		if (!socketJoinGroup(env, sock, testAddr.s_addr)) break;
-
-		unsigned char testString[] = "hostIdTest";
-		unsigned testStringLength = sizeof testString;
-
-		if (!writeSocket(env, sock, testAddr, testPort, 0,
-				 testString, testStringLength)) break;
-
-		unsigned char readBuffer[20];
-		struct sockaddr_in fromAddr;
-		struct timeval timeout;
-		timeout.tv_sec = 5;
-		timeout.tv_usec = 0;
-		int bytesRead = readSocket(env, sock,
-					   readBuffer, sizeof readBuffer,
-					   fromAddr, &timeout);
-		if (bytesRead == 0 // timeout occurred
-		    || bytesRead != (int)testStringLength
-		    || strncmp((char*)readBuffer, (char*)testString,
-			       testStringLength) != 0) {
-		  loopbackWorks = 0;
-
-		  // We couldn't find our address using multicast loopback
-		  // so try instead to look it up directly.
-		  char hostname[100];
-		  hostname[0] = '\0';
+    if (!loopbackWorks) do {
+      // We couldn't find our address using multicast loopback
+      // so try instead to look it up directly.
+      char hostname[100];
+      hostname[0] = '\0';
 #ifndef CRIS
-		  gethostname(hostname, sizeof hostname);
+      gethostname(hostname, sizeof hostname);
 #endif
-		  if (hostname[0] == '\0') {
-		    env.setResultErrMsg("initial gethostname() failed");
-		    break;
-		  }
-
+      if (hostname[0] == '\0') {
+	env.setResultErrMsg("initial gethostname() failed");
+	break;
+      }
+      
 #if defined(VXWORKS)
 #include <hostLib.h>
-		  if (ERROR == (ourAddress = hostGetByName( hostname ))) {
-		    ourAddress = 0;
-		  }
-
+      if (ERROR == (ourAddress = hostGetByName( hostname ))) break;
 #else
-		  struct hostent* hstent
-		    = (struct hostent*)gethostbyname(hostname);
-		  if (hstent == NULL || hstent->h_length != 4) {
-			env.setResultErrMsg("initial gethostbyname() failed");
-			break;
-		  }
-		  // Take the first address that's not bad
-		  // (This code, like many others, won't handle IPv6)
-		  netAddressBits addr = 0;
-		  for (unsigned i = 0; ; ++i) {
-		    char* addrPtr = hstent->h_addr_list[i];
-		    if (addrPtr == NULL) break;
-
-		    netAddressBits a = *(netAddressBits*)addrPtr;
-		    if (!badAddress(a)) {
-		      addr = a;
-		      break;
-		    }
-		  }
-		  if (addr != 0) {
-		    fromAddr.sin_addr.s_addr = addr;
-		  } else {
-			env.setResultMsg("no address");
-			break;
-		  }
-		}
-#endif
-
-		// Make sure we have a good address:
-		netAddressBits from = fromAddr.sin_addr.s_addr;
-                if (badAddress(from)) {
-			char tmp[100];
-			sprintf(tmp,
-				"This computer has an invalid IP address: 0x%x",
-				(netAddressBits)(ntohl(from)));
-			env.setResultMsg(tmp);
-			break;
-		}
-
-		ourAddress = from;
-	} while (0);
-
-	if (sock >= 0) {
-		socketLeaveGroup(env, sock, testAddr.s_addr);
-		_close(sock);
+      struct hostent* hstent
+	= (struct hostent*)gethostbyname(hostname);
+      if (hstent == NULL || hstent->h_length != 4) {
+	env.setResultErrMsg("initial gethostbyname() failed");
+	break;
+      }
+      // Take the first address that's not bad
+      // (This code, like many others, won't handle IPv6)
+      netAddressBits addr = 0;
+      for (unsigned i = 0; ; ++i) {
+	char* addrPtr = hstent->h_addr_list[i];
+	if (addrPtr == NULL) break;
+	
+	netAddressBits a = *(netAddressBits*)addrPtr;
+	if (!badAddress(a)) {
+	  addr = a;
+	  break;
 	}
-
-	// Use our newly-discovered IP address, and the current time,
-	// to initialize the random number generator's seed:
-	struct timeval timeNow;
-#ifdef BSD
-	struct timezone Idunno;
-#else
-	int Idunno;
+      }
+      if (addr != 0) {
+	fromAddr.sin_addr.s_addr = addr;
+      } else {
+	env.setResultMsg("no address");
+	break;
+      }
+    } while (0);
+    
+    // Make sure we have a good address:
+    netAddressBits from = fromAddr.sin_addr.s_addr;
+    if (badAddress(from)) {
+      char tmp[100];
+      sprintf(tmp,
+	      "This computer has an invalid IP address: 0x%x",
+	      (netAddressBits)(ntohl(from)));
+      env.setResultMsg(tmp);
+      from = 0;
+    }
+    
+    ourAddress = from;
 #endif
-	gettimeofday(&timeNow, &Idunno);
-	unsigned seed = ourAddress^timeNow.tv_sec^timeNow.tv_usec;
-	our_srandom(seed);
-
-	return ourAddress;
+    
+    if (sock >= 0) {
+      socketLeaveGroup(env, sock, testAddr.s_addr);
+      closeSocket(sock);
+    }
+    
+    // Use our newly-discovered IP address, and the current time,
+    // to initialize the random number generator's seed:
+    struct timeval timeNow;
+    gettimeofday(&timeNow, NULL);
+    unsigned seed = ourAddress^timeNow.tv_sec^timeNow.tv_usec;
+    our_srandom(seed);
+  }
+  return ourAddress;
 }
 
 netAddressBits chooseRandomIPv4SSMAddress(UsageEnvironment& env) {
@@ -677,35 +682,57 @@ netAddressBits chooseRandomIPv4SSMAddress(UsageEnvironment& env) {
 }
 
 char const* timestampString() {
-	struct timeval tvNow;
-#ifdef BSD
-	struct timezone Idunno;
+  struct timeval tvNow;
+  gettimeofday(&tvNow, NULL);
+  
+#if !defined(_WIN32_WCE)
+  static char timeString[9]; // holds hh:mm:ss plus trailing '\0'
+  char const* ctimeResult = ctime((time_t*)&tvNow.tv_sec);
+  char const* from = &ctimeResult[11];
+  int i;
+  for (i = 0; i < 8; ++i) {
+    timeString[i] = from[i];
+  }
+  timeString[i] = '\0';
 #else
-	int Idunno;
+  // WinCE apparently doesn't have "ctime()", so instead, construct
+  // a timestamp string just using the integer and fractional parts
+  // of "tvNow":
+  static char timeString[50];
+  sprintf(timeString, "%lu.%06ld", tvNow.tv_sec, tvNow.tv_usec);
 #endif
-	gettimeofday(&tvNow, &Idunno);
-
-	static char timeString[9]; // holds hh:mm:ss plus trailing '\0'
-	char const* ctimeResult = ctime((time_t*)&tvNow.tv_sec);
-	char const* from = &ctimeResult[11];
-	int i;
-	for (i = 0; i < 8; ++i) {
-		timeString[i] = from[i];
-	}
-	timeString[i] = '\0';
-
-	return (char const*)&timeString;
+  
+  return (char const*)&timeString;
 }
 
 #if (defined(__WIN32__) || defined(_WIN32)) && !defined(IMN_PIM)
 // For Windoze, we need to implement our own gettimeofday()
+#if !defined(_WIN32_WCE)
 #include <sys/timeb.h>
+#endif
 
 int gettimeofday(struct timeval* tp, int* /*tz*/) {
-	struct timeb tb;
-	ftime(&tb);
-	tp->tv_sec = tb.time;
-	tp->tv_usec = 1000*tb.millitm;
-	return 0;
+#if defined(_WIN32_WCE)
+  /* FILETIME of Jan 1 1970 00:00:00. */
+  static const unsigned __int64 epoch = 116444736000000000L;
+
+  FILETIME    file_time;
+  SYSTEMTIME  system_time;
+  ULARGE_INTEGER ularge;
+
+  GetSystemTime(&system_time);
+  SystemTimeToFileTime(&system_time, &file_time);
+  ularge.LowPart = file_time.dwLowDateTime;
+  ularge.HighPart = file_time.dwHighDateTime;
+
+  tp->tv_sec = (long) ((ularge.QuadPart - epoch) / 10000000L);
+  tp->tv_usec = (long) (system_time.wMilliseconds * 1000);
+#else
+  struct timeb tb;
+  ftime(&tb);
+  tp->tv_sec = tb.time;
+  tp->tv_usec = 1000*tb.millitm;
+#endif
+  return 0;
 }
 #endif
