@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // AC3 Audio RTP Sources
 // Implementation
 
@@ -74,6 +74,6 @@ Boolean AC3AudioRTPSource
 }    
 
 char const* AC3AudioRTPSource::MIMEtype() const {
-  return "audio/ac3";
+  return "audio/AC3";
 }
 
