@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up an AC3 audio elementary stream into frames
 // Implementation
 
@@ -83,18 +83,12 @@ private:
 
 ////////// AC3AudioStreamFramer implementation //////////
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 AC3AudioStreamFramer::AC3AudioStreamFramer(UsageEnvironment& env,
 					   FramedSource* inputSource,
 					   unsigned char streamCode)
   : FramedFilter(env, inputSource), fOurStreamCode(streamCode) {
   // Use the current wallclock time as the initial 'presentation time':
-  gettimeofday(&fNextFramePresentationTime, &Idunno);
+  gettimeofday(&fNextFramePresentationTime, NULL);
 
   fParser = new AC3AudioStreamParser(this, inputSource);
 }
@@ -122,6 +116,10 @@ unsigned AC3AudioStreamFramer::samplingRate() {
   return fParser->currentFrame().samplingFreq;
 }
 
+void AC3AudioStreamFramer::flushInput() {
+  fParser->flushInput();
+}
+
 void AC3AudioStreamFramer::doGetNextFrame() {
   fParser->registerReadInterest(fTo, fMaxSize);
   parseNextFrame();
@@ -145,7 +143,8 @@ struct timeval AC3AudioStreamFramer::currentFramePlayTime() const {
 }
 
 void AC3AudioStreamFramer
-::handleNewData(void* clientData, unsigned char* ptr, unsigned size) {
+::handleNewData(void* clientData, unsigned char* ptr, unsigned size,
+		struct timeval /*presentationTime*/) {
   AC3AudioStreamFramer* framer = (AC3AudioStreamFramer*)clientData;
   framer->handleNewData(ptr, size);
 }
