@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand, from an AMR audio file.
 // Implementation
@@ -25,14 +25,15 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 AMRAudioFileServerMediaSubsession*
 AMRAudioFileServerMediaSubsession::createNew(UsageEnvironment& env,
-					     char const* fileName) {
-  return new AMRAudioFileServerMediaSubsession(env, fileName);
+					     char const* fileName,
+					     Boolean reuseFirstSource) {
+  return new AMRAudioFileServerMediaSubsession(env, fileName, reuseFirstSource);
 }
 
 AMRAudioFileServerMediaSubsession
 ::AMRAudioFileServerMediaSubsession(UsageEnvironment& env,
-				    char const* fileName)
-  : FileServerMediaSubsession(env, fileName) {
+				    char const* fileName, Boolean reuseFirstSource)
+  : FileServerMediaSubsession(env, fileName, reuseFirstSource) {
 }
 
 AMRAudioFileServerMediaSubsession
