@@ -14,16 +14,12 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A source object for AMR audio files (as defined in RFC 3267, section 5)
 // Implementation
 
-#if defined(__WIN32__) || defined(_WIN32)
-#include <io.h>
-#include <fcntl.h>
-#endif
-
 #include "AMRAudioFileSource.hh"
+#include "InputFile.hh"
 #include "GroupsockHelper.hh"
 
 ////////// AMRAudioFileSource //////////
@@ -34,19 +30,8 @@ AMRAudioFileSource::createNew(UsageEnvironment& env, char const* fileName) {
   Boolean magicNumberOK = True;
   do {
 
-    // Check for a special case file name: "stdin"
-    if (strcmp(fileName, "stdin") == 0) {
-      fid = stdin;
-#if defined(__WIN32__) || defined(_WIN32)
-      _setmode(_fileno(stdin), _O_BINARY); // convert to binary mode
-#endif
-    } else { 
-      fid = fopen(fileName, "rb");
-      if (fid == NULL) {
-	env.setResultMsg("unable to open file \"",fileName, "\"");
-	break;
-      }
-    }
+    fid = OpenInputFile(env, fileName);
+    if (fid == NULL) break;
 
     // Now, having opened the input file, read the first few bytes, to
     // check the required 'magic number':
@@ -92,7 +77,7 @@ AMRAudioFileSource::createNew(UsageEnvironment& env, char const* fileName) {
   } while (0);
 
   // An error occurred:
-  if (fid != NULL) fclose(fid);
+  CloseInputFile(fid);
   if (!magicNumberOK) {
     env.setResultMsg("Bad (or nonexistent) AMR file header");
   }
@@ -107,15 +92,9 @@ AMRAudioFileSource
 }
 
 AMRAudioFileSource::~AMRAudioFileSource() {
-  fclose(fFid);
+  CloseInputFile(fFid);
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 // The mapping from the "FT" field to frame size.
 // Values of 65535 are invalid.
 #define FT_INVALID 65535
@@ -132,6 +111,8 @@ static unsigned short frameSizeWideband[16] = {
   FT_INVALID, FT_INVALID, 0, 0
 };
 
+// Note: We should change the following to use asynchronous file reading, #####
+// as we now do with ByteStreamFileSource. #####
 void AMRAudioFileSource::doGetNextFrame() {
   if (feof(fFid) || ferror(fFid)) {
     handleClosure(this);
@@ -177,7 +158,7 @@ void AMRAudioFileSource::doGetNextFrame() {
   // Set the 'presentation time':
   if (fPresentationTime.tv_sec == 0 && fPresentationTime.tv_usec == 0) {
     // This is the first frame, so use the current time:
-    gettimeofday(&fPresentationTime, &Idunno);
+    gettimeofday(&fPresentationTime, NULL);
   } else {
     // Increment by the play time of the previous frame (20 ms)
     unsigned uSeconds	= fPresentationTime.tv_usec + 20000;
