@@ -14,11 +14,12 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A simple UDP sink (i.e., without RTP or other headers added); one frame per packet
 // Implementation
 
 #include "BasicUDPSink.hh"
+#include <GroupsockHelper.hh>
 
 BasicUDPSink* BasicUDPSink::createNew(UsageEnvironment& env, Groupsock* gs,
 				      unsigned maxPayloadSize) {
@@ -36,15 +37,9 @@ BasicUDPSink::~BasicUDPSink() {
   delete[] fOutputBuffer;
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 Boolean BasicUDPSink::continuePlaying() {
   // Record the fact that we're starting to play now:
-  gettimeofday(&fNextSendTime, &Idunno);
+  gettimeofday(&fNextSendTime, NULL);
 
   // Arrange to get and send the first payload.
   // (This will also schedule any future sends.)
@@ -53,9 +48,11 @@ Boolean BasicUDPSink::continuePlaying() {
 }
 
 void BasicUDPSink::continuePlaying1() {
-  fSource->getNextFrame(fOutputBuffer, fMaxPayloadSize,
-                        afterGettingFrame, this,
-                        onSourceClosure, this);
+  if (fSource != NULL) {
+    fSource->getNextFrame(fOutputBuffer, fMaxPayloadSize,
+			  afterGettingFrame, this,
+			  onSourceClosure, this);
+  }
 }
 
 void BasicUDPSink::afterGettingFrame(void* clientData, unsigned frameSize,
@@ -71,7 +68,7 @@ void BasicUDPSink::afterGettingFrame1(unsigned frameSize, unsigned numTruncatedB
   if (numTruncatedBytes > 0) {
     envir() << "BasicUDPSink::afterGettingFrame1(): The input frame data was too large for our spcified maximum payload size ("
 	    << fMaxPayloadSize << ").  "
-	    << numTruncatedBytes << " of trailing data was dropped!\n";
+	    << numTruncatedBytes << " bytes of trailing data was dropped!\n";
   }
 
   // Send the packet:
@@ -84,7 +81,7 @@ void BasicUDPSink::afterGettingFrame1(unsigned frameSize, unsigned numTruncatedB
   fNextSendTime.tv_usec %= 1000000;
   
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   int uSecondsToGo;
   if (fNextSendTime.tv_sec < timeNow.tv_sec) {
     uSecondsToGo = 0; // prevents integer underflow if too far behind
