@@ -14,11 +14,12 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2002 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A simple UDP source, where every UDP payload is a complete frame
 // Implementation
 
 #include "BasicUDPSource.hh"
+#include <GroupsockHelper.hh>
 
 BasicUDPSource* BasicUDPSource::createNew(UsageEnvironment& env,
 					Groupsock* inputGS) {
@@ -26,7 +27,9 @@ BasicUDPSource* BasicUDPSource::createNew(UsageEnvironment& env,
 }
 
 BasicUDPSource::BasicUDPSource(UsageEnvironment& env, Groupsock* inputGS)
-  : FramedSource(env), fInputGS(inputGS) {
+  : FramedSource(env), fInputGS(inputGS), fHaveStartedReading(False) {
+  // Try to use a large receive buffer (in the OS):
+  increaseReceiveBufferTo(env, inputGS->socketNum(), 50*1024);
 }
 
 BasicUDPSource::~BasicUDPSource(){
@@ -34,20 +37,27 @@ BasicUDPSource::~BasicUDPSource(){
 }
 
 void BasicUDPSource::doGetNextFrame() {
-  // Await the next incoming packet:
-  envir().taskScheduler().turnOnBackgroundReadHandling(fInputGS->socketNum(), 
-       (TaskScheduler::BackgroundHandlerProc*)&incomingPacketHandler, this);
+  if (!fHaveStartedReading) {
+    // Await incoming packets:
+    envir().taskScheduler().turnOnBackgroundReadHandling(fInputGS->socketNum(), 
+	 (TaskScheduler::BackgroundHandlerProc*)&incomingPacketHandler, this);
+    fHaveStartedReading = True;
+  }
 }
 
-float BasicUDPSource::getPlayTime(unsigned /*numFrames*/) const {
-  return 0.0;
+void BasicUDPSource::doStopGettingFrames() {
+  envir().taskScheduler().turnOffBackgroundReadHandling(fInputGS->socketNum());
+  fHaveStartedReading = False;
 }
 
+
 void BasicUDPSource::incomingPacketHandler(BasicUDPSource* source, int /*mask*/){
   source->incomingPacketHandler1();
 }
 
 void BasicUDPSource::incomingPacketHandler1() {
+  if (!isCurrentlyAwaitingData()) return; // we're not ready for the data yet
+
   // Read the packet into our desired destination:
   struct sockaddr_in fromAddress;
   if (!fInputGS->handleRead(fTo, fMaxSize, fFrameSize, fromAddress)) return;
