@@ -14,71 +14,110 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2002 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A file source that is a plain byte stream (rather than frames)
 // Implementation
 
-#if defined(__WIN32__) || defined(_WIN32)
+#if (defined(__WIN32__) || defined(_WIN32)) && !defined(_WIN32_WCE)
 #include <io.h>
 #include <fcntl.h>
 #endif
 
 #include "ByteStreamFileSource.hh"
+#include "InputFile.hh"
 #include "GroupsockHelper.hh"
 
 ////////// ByteStreamFileSource //////////
 
-ByteStreamFileSource::ByteStreamFileSource(UsageEnvironment& env, FILE* fid,
-					   unsigned preferredFrameSize,
-					   unsigned playTimePerFrame)
-  : FramedFileSource(env, fid), fPreferredFrameSize(preferredFrameSize),
-    fPlayTimePerFrame(playTimePerFrame), fLastPlayTime(0) {
-}
-
-ByteStreamFileSource::~ByteStreamFileSource() {
-  fclose(fFid);
-}
-
 ByteStreamFileSource*
 ByteStreamFileSource::createNew(UsageEnvironment& env, char const* fileName,
 				unsigned preferredFrameSize,
 				unsigned playTimePerFrame) {
-  do {
-    FILE* fid;
+  FILE* fid = OpenInputFile(env, fileName);
+  if (fid == NULL) return NULL;
 
-    // Check for a special case file name: "stdin"
-    if (strcmp(fileName, "stdin") == 0) {
-      fid = stdin;
-#if defined(__WIN32__) || defined(_WIN32)
-      _setmode(_fileno(stdin), _O_BINARY); // convert to binary mode
-#endif
-    } else { 
-     fid = fopen(fileName, "rb");
-      if (fid == NULL) {
-	env.setResultMsg("unable to open file \"",fileName, "\"");
-	break;
-      }
-    }
+  Boolean deleteFidOnClose = fid == stdin ? False : True;
+  ByteStreamFileSource* newSource
+    = new ByteStreamFileSource(env, fid, deleteFidOnClose,
+			       preferredFrameSize, playTimePerFrame);
+  newSource->fFileSize = GetFileSize(fileName, fid);
 
-    return new ByteStreamFileSource(env, fid, preferredFrameSize,
-				    playTimePerFrame);
-  } while (0);
-
-  return NULL;
+  return newSource;
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
+ByteStreamFileSource*
+ByteStreamFileSource::createNew(UsageEnvironment& env, FILE* fid,
+				Boolean deleteFidOnClose,
+				unsigned preferredFrameSize,
+				unsigned playTimePerFrame) {
+  if (fid == NULL) return NULL;
+
+  ByteStreamFileSource* newSource
+    = new ByteStreamFileSource(env, fid, deleteFidOnClose,
+			       preferredFrameSize, playTimePerFrame);
+  newSource->fFileSize = GetFileSize(NULL, fid);
+
+  return newSource;
+}
+
+void ByteStreamFileSource::seekToByteAbsolute(u_int64_t byteNumber) {
+  SeekFile64(fFid, (int64_t)byteNumber, SEEK_SET);
+}
+
+void ByteStreamFileSource::seekToByteRelative(int64_t offset) {
+  SeekFile64(fFid, offset, SEEK_CUR);
+}
+
+ByteStreamFileSource::ByteStreamFileSource(UsageEnvironment& env, FILE* fid,
+					   Boolean deleteFidOnClose,
+					   unsigned preferredFrameSize,
+					   unsigned playTimePerFrame)
+  : FramedFileSource(env, fid), fPreferredFrameSize(preferredFrameSize),
+    fPlayTimePerFrame(playTimePerFrame), fLastPlayTime(0), fFileSize(0),
+    fDeleteFidOnClose(deleteFidOnClose), fHaveStartedReading(False) {
+}
+
+ByteStreamFileSource::~ByteStreamFileSource() {
+  if (fFid == NULL) return;
+
+#ifndef READ_FROM_FILES_SYNCHRONOUSLY
+  envir().taskScheduler().turnOffBackgroundReadHandling(fileno(fFid));
 #endif
 
+  if (fDeleteFidOnClose) fclose(fFid);
+}
+
 void ByteStreamFileSource::doGetNextFrame() {
   if (feof(fFid) || ferror(fFid)) {
     handleClosure(this);
     return;
   }
 
+#ifdef READ_FROM_FILES_SYNCHRONOUSLY
+  doReadFromFile();
+#else
+  if (!fHaveStartedReading) {
+    // Await readable data from the file:
+    envir().taskScheduler().turnOnBackgroundReadHandling(fileno(fFid),
+	       (TaskScheduler::BackgroundHandlerProc*)&fileReadableHandler, this);
+    fHaveStartedReading = True;
+  }
+#endif
+}
+
+void ByteStreamFileSource::doStopGettingFrames() {
+#ifndef READ_FROM_FILES_SYNCHRONOUSLY
+  envir().taskScheduler().turnOffBackgroundReadHandling(fileno(fFid));
+  fHaveStartedReading = False;
+#endif
+}
+
+void ByteStreamFileSource::fileReadableHandler(ByteStreamFileSource* source, int /*mask*/) {
+  if (!source->isCurrentlyAwaitingData()) return; // we're not ready for the data yet
+  source->doReadFromFile();
+}
+
+void ByteStreamFileSource::doReadFromFile() {
   // Try to read as many bytes as will fit in the buffer provided
   // (or "fPreferredFrameSize" if less)
   if (fPreferredFrameSize > 0 && fPreferredFrameSize < fMaxSize) {
@@ -90,7 +129,7 @@ void ByteStreamFileSource::doGetNextFrame() {
   if (fPlayTimePerFrame > 0 && fPreferredFrameSize > 0) {
     if (fPresentationTime.tv_sec == 0 && fPresentationTime.tv_usec == 0) {
       // This is the first frame, so use the current time:
-      gettimeofday(&fPresentationTime, &Idunno);
+      gettimeofday(&fPresentationTime, NULL);
     } else {
       // Increment by the play time of the previous data:
       unsigned uSeconds	= fPresentationTime.tv_usec + fLastPlayTime;
@@ -100,17 +139,21 @@ void ByteStreamFileSource::doGetNextFrame() {
 
     // Remember the play time of this data:
     fLastPlayTime = (fPlayTimePerFrame*fFrameSize)/fPreferredFrameSize;
+    fDurationInMicroseconds = fLastPlayTime;
   } else {
     // We don't know a specific play time duration for this data,
     // so just record the current time as being the 'presentation time':
-    gettimeofday(&fPresentationTime, &Idunno);
+    gettimeofday(&fPresentationTime, NULL);
   }
 
-  // Switch to another task, and inform the reader that he has data:
+  // Inform the reader that he has data:
+#ifdef READ_FROM_FILES_SYNCHRONOUSLY
+  // To avoid possible infinite recursion, we need to return to the event loop to do this:
   nextTask() = envir().taskScheduler().scheduleDelayedTask(0,
 				(TaskFunc*)FramedSource::afterGetting, this);
-}
-
-float ByteStreamFileSource::getPlayTime(unsigned numFrames) const {
-  return (float)(numFrames*(fPlayTimePerFrame/1000000.0));
+#else
+  // Because the file read was done from the event loop, we can call the
+  // 'after getting' function directly, without risk of infinite recursion:
+  FramedSource::afterGetting(this);
+#endif
 }
