@@ -14,16 +14,17 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // File sinks
 // Implementation
 
-#if defined(__WIN32__) || defined(_WIN32)
+#if (defined(__WIN32__) || defined(_WIN32)) && !defined(_WIN32_WCE)
 #include <io.h>
 #include <fcntl.h>
 #endif
 #include "FileSink.hh"
 #include "GroupsockHelper.hh"
+#include "OutputFile.hh"
 
 ////////// FileSink //////////
 
@@ -49,8 +50,6 @@ FileSink::~FileSink() {
 
 FileSink* FileSink::createNew(UsageEnvironment& env, char const* fileName,
 			      unsigned bufferSize, Boolean oneFilePerFrame) {
-  FileSink* newSink = NULL;
-
   do {
     FILE* fid;
     char const* perFrameFileNamePrefix;
@@ -60,47 +59,17 @@ FileSink* FileSink::createNew(UsageEnvironment& env, char const* fileName,
       perFrameFileNamePrefix = fileName;
     } else {
       // Normal case: create the fid once
-      fid = openFileByName(env, fileName);
+      fid = OpenOutputFile(env, fileName);
       if (fid == NULL) break;
       perFrameFileNamePrefix = NULL;
     }
 
-    newSink = new FileSink(env, fid, bufferSize, perFrameFileNamePrefix);
-    if (newSink == NULL) break;
-
-    return newSink;
+    return new FileSink(env, fid, bufferSize, perFrameFileNamePrefix);
   } while (0);
 
-  delete newSink;
   return NULL;
 }
 
-FILE* FileSink::openFileByName(UsageEnvironment& env,
-			       char const* fileName) {
-  FILE* fid;
-    
-  // Check for special case 'file names': "stdout" and "stderr"
-  if (strcmp(fileName, "stdout") == 0) {
-    fid = stdout;
-#if defined(__WIN32__) || defined(_WIN32)
-    _setmode(_fileno(stdout), _O_BINARY);	// convert to binary mode
-#endif
-  } else if (strcmp(fileName, "stderr") == 0) {
-    fid = stderr;
-#if defined(__WIN32__) || defined(_WIN32)
-    _setmode(_fileno(stderr), _O_BINARY);	// convert to binary mode
-#endif
-  } else {
-    fid = fopen(fileName, "wb");
-  }
-
-  if (fid == NULL) {
-    env.setResultMsg("unable to open file \"", fileName, "\"");
-  }
-
-  return fid;
-}
-
 Boolean FileSink::continuePlaying() {
   if (fSource == NULL) return False;
 
@@ -125,7 +94,7 @@ void FileSink::addData(unsigned char* data, unsigned dataSize,
     // Special case: Open a new file on-the-fly for this frame
     sprintf(fPerFrameFileNameBuffer, "%s-%lu.%06lu", fPerFrameFileNamePrefix,
 	    presentationTime.tv_sec, presentationTime.tv_usec);
-    fOutFid = openFileByName(envir(), fPerFrameFileNameBuffer);
+    fOutFid = OpenOutputFile(envir(), fPerFrameFileNameBuffer);
   }
 
   // Write to our file:
@@ -140,7 +109,7 @@ void FileSink::addData(unsigned char* data, unsigned dataSize,
   if (!packetIsLost)
 #endif
   if (fOutFid != NULL && data != NULL) {
-    fwrite(data, dataSize, 1, fOutFid);
+    fwrite(data, 1, dataSize, fOutFid);
   }
 }
 
