@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // Framed Filters
 // Implementation
 
@@ -37,13 +37,15 @@ FramedFilter::~FramedFilter() {
 // call the same function in the input source - i.e., act like a 'null filter
 
 char const* FramedFilter::MIMEtype() const {
+  if (fInputSource == NULL) return "";
+
   return fInputSource->MIMEtype();
 }
 
 void FramedFilter::getAttributes() const {
-  fInputSource->getAttributes();
+  if (fInputSource != NULL) fInputSource->getAttributes();
 }
 
 void FramedFilter::doStopGettingFrames() {
-  fInputSource->stopGettingFrames();
+  if (fInputSource != NULL) fInputSource->stopGettingFrames();
 }
