@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // H.261 Video RTP Sources
 // Implementation
 
@@ -63,5 +63,5 @@ Boolean H261VideoRTPSource
 }
 
 char const* H261VideoRTPSource::MIMEtype() const {
-  return "video/h261";
+  return "video/H261";
 }
