@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // H.263+ Video RTP Sources
 // Implementation
 
@@ -33,7 +33,8 @@ H263plusVideoRTPSource
 			 unsigned char rtpPayloadFormat,
 			 unsigned rtpTimestampFrequency)
   : MultiFramedRTPSource(env, RTPgs,
-			 rtpPayloadFormat, rtpTimestampFrequency) {
+			 rtpPayloadFormat, rtpTimestampFrequency),
+  fNumSpecialHeaders(0), fSpecialHeaderBytesLength(0) {  
 }
 
 H263plusVideoRTPSource::~H263plusVideoRTPSource() {
@@ -68,8 +69,7 @@ Boolean H263plusVideoRTPSource
     if (packetSize < expectedHeaderSize) return False;
   }
 
-  fCurrentPacketBeginsFrame = fCurrentPacketCompletesFrame;
-          // whether the *previous* packet ended a frame
+  fCurrentPacketBeginsFrame = P;
   if (fCurrentPacketBeginsFrame) {
     fNumSpecialHeaders = fSpecialHeaderBytesLength = 0;
   }
@@ -102,5 +102,5 @@ Boolean H263plusVideoRTPSource
 }
 
 char const* H263plusVideoRTPSource::MIMEtype() const {
-  return "video/h263-1998";
+  return "video/H263-1998";
 }
