@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // HTTP sinks
 // Implementation
 
@@ -30,7 +30,6 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 HTTPSink* HTTPSink::createNew(UsageEnvironment& env, Port ourPort) {
   int ourSocket = -1;
-  HTTPSink* newSink = NULL;
 
   do {
     int ourSocket = setUpOurSocket(env, ourPort);
@@ -44,8 +43,7 @@ HTTPSink* HTTPSink::createNew(UsageEnvironment& env, Port ourPort) {
     return newSink;
   } while (0);
 
-  if (ourSocket != -1) ::_close(ourSocket);
-  delete newSink;
+  if (ourSocket != -1) ::closeSocket(ourSocket);
   return NULL;
 }
 
@@ -72,7 +70,7 @@ int HTTPSink::setUpOurSocket(UsageEnvironment& env, Port& ourPort) {
     return ourSocket;
   } while (0);  
 
-  if (ourSocket != -1) ::_close(ourSocket);
+  if (ourSocket != -1) ::closeSocket(ourSocket);
   return -1;
 }
 
@@ -89,7 +87,7 @@ HTTPSink::HTTPSink(UsageEnvironment& env, int ourSocket)
 }
 
 HTTPSink::~HTTPSink() {
-  ::_close(fSocket);
+  ::closeSocket(fSocket);
 }
 
  Boolean HTTPSink::isUseableFrame(unsigned char* /*framePtr*/,
@@ -120,7 +118,7 @@ Boolean HTTPSink::continuePlaying() {
       // client player tools happy:
       char okResponse[400];
       const char* responseFmt = "HTTP/1.1 200 OK\r\nCache-Control: no-cache\r\nPragma: no-cache\r\nContent-Length: 2147483647\r\nContent-Type: %s\r\n\r\n";
-#if defined(IRIX) || defined(ALPHA) || defined(_QNX4) || defined(IMN_PIM) || defined(CRIS)
+#if defined(IRIX) || defined(ALPHA) || defined(_QNX4) || defined(IMN_PIM) || defined(CRIS) || defined (VXWORKS)
       /* snprintf() isn't defined, so just use sprintf() - ugh! */
       sprintf(okResponse, responseFmt, fSource->MIMEtype());
 #else
@@ -140,7 +138,7 @@ Boolean HTTPSink::continuePlaying() {
 void HTTPSink::ourOnSourceClosure(void* clientData) {
   // No more input frames - we're done:
   HTTPSink* sink = (HTTPSink*) clientData;
-  ::_close(sink->fClientSocket);
+  ::closeSocket(sink->fClientSocket);
   sink->fClientSocket = -1;
   onSourceClosure(sink);
 }
