@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // JPEG video sources
 // Implementation
 
@@ -27,6 +27,14 @@ JPEGVideoSource::JPEGVideoSource(UsageEnvironment& env)
 JPEGVideoSource::~JPEGVideoSource() {
 }
 
+u_int8_t const* JPEGVideoSource::quantizationTables(u_int8_t& precision,
+						    u_int16_t& length) {
+  // Default implementation
+  precision = 0;
+  length = 0;
+  return NULL;
+}
+
 Boolean JPEGVideoSource::isJPEGVideoSource() const {
   return True;
 }
