@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // Transcoder for ADUized MP3 frames
 // Implementation
 
@@ -39,7 +39,7 @@ MP3ADUTranscoder* MP3ADUTranscoder::createNew(UsageEnvironment& env,
 					      unsigned outBitrate /* in kbps */,
 					      FramedSource* inputSource) {
   // The source must be an MP3 ADU source:
-  if (strcmp(inputSource->MIMEtype(), "audio/mpa-robust") != 0) {
+  if (strcmp(inputSource->MIMEtype(), "audio/MPA-ROBUST") != 0) {
     env.setResultMsg(inputSource->name(), " is not an MP3 ADU source");
     return NULL;
   }
