@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // Interleaving of MP3 ADUs
 // Implementation
 
@@ -55,7 +55,7 @@ FramedSource* MP3ADUinterleaverBase::getInputSource(UsageEnvironment& env,
   if (!FramedSource::lookupByName(env, inputSourceName, inputSource))
     return NULL;
   
-  if (strcmp(inputSource->MIMEtype(), "audio/mpa-robust") != 0) {
+  if (strcmp(inputSource->MIMEtype(), "audio/MPA-ROBUST") != 0) {
     env.setResultMsg(inputSourceName, " is not an MP3 ADU source");
     return NULL;
   }
