@@ -14,24 +14,19 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // MP3 File Sources
 // Implementation
 
-#include <sys/stat.h>
-#if defined(__WIN32__) || defined(_WIN32)
-#include <io.h>
-#include <fcntl.h>
-#endif
-
 #include "MP3FileSource.hh"
 #include "MP3StreamState.hh"
+#include "InputFile.hh"
 
 ////////// MP3FileSource //////////
 
 MP3FileSource::MP3FileSource(UsageEnvironment& env, FILE* fid)
   : FramedFileSource(env, fid),
-    fStreamState(new MP3StreamState) {
+    fStreamState(new MP3StreamState(env)) {
 }
 
 MP3FileSource::~MP3FileSource() {
@@ -39,7 +34,7 @@ MP3FileSource::~MP3FileSource() {
 }
 
 char const* MP3FileSource::MIMEtype() const {
-  return "audio/mpeg";
+  return "audio/MPEG";
 }
 
 MP3FileSource* MP3FileSource::createNew(UsageEnvironment& env, char const* fileName) {
@@ -47,39 +42,36 @@ MP3FileSource* MP3FileSource::createNew(UsageEnvironment& env, char const* fileN
 
   do {
     FILE* fid;
-    unsigned fileSize = 0;
 
-    // Check for a special case file name: "stdin"
-    if (strcmp(fileName, "stdin") == 0) {
-      fid = stdin;
-#if defined(__WIN32__) || defined(_WIN32)
-      _setmode(_fileno(stdin), _O_BINARY); // convert to binary mode
-#endif
-    } else { 
-      fid = fopen(fileName, "rb");
-      if (fid == NULL) {
-	env.setResultMsg("unable to open file \"",fileName, "\"");
-	break;
-      }
-      struct stat sb;
-      if (stat(fileName, &sb) == 0) {
-	fileSize = sb.st_size;
-      }
-    }
+    fid = OpenInputFile(env, fileName);
+    if (fid == NULL) break;
 
     newSource = new MP3FileSource(env, fid);
     if (newSource == NULL) break;
 
+    unsigned fileSize = (unsigned)GetFileSize(fileName, fid);
     newSource->assignStream(fid, fileSize);
     if (!newSource->initializeStream()) break;
 
     return newSource;
   } while (0);
 
-  delete newSource;
+  Medium::close(newSource);
   return NULL;
 }
 
+float MP3FileSource::filePlayTime() const {
+  return fStreamState->filePlayTime();
+}
+
+void MP3FileSource::setPresentationTimeScale(unsigned scale) {
+  fStreamState->setPresentationTimeScale(scale);
+}
+
+void MP3FileSource::seekWithinFile(float seekNPT) {
+  fStreamState->seekWithinFile(seekNPT);
+}
+
 void MP3FileSource::getAttributes() const {
   char buffer[200];
   fStreamState->getAttributes(buffer, sizeof buffer);
@@ -143,5 +135,8 @@ Boolean MP3FileSource::initializeStream() {
   fStreamState->checkForXingHeader(); // in case this is a VBR file
 
   fHaveJustInitialized = True;
+  // Hack: It's possible that our environment's 'result message' has been
+  // reset within this function, so set it again to our name now:
+  envir().setResultMsg(name());
   return True;
 }
