@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A class encapsulating the state of a MP3 stream
 // Implementation
 
@@ -28,7 +28,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #define MILLION 1000000
 
 MP3StreamState::MP3StreamState(UsageEnvironment& env)
-  : fEnv(env), fFid(NULL) {
+  : fEnv(env), fFid(NULL), fPresentationTimeScale(1) {
 }
 
 MP3StreamState::~MP3StreamState() {
@@ -36,19 +36,13 @@ MP3StreamState::~MP3StreamState() {
   if (fFid != NULL && fFid != stdin) {
     if (fFidIsReallyASocket) {
       long fid_long = (long)fFid;
-      _close((int)fid_long);
+      closeSocket((int)fid_long);
     } else {
       fclose(fFid);
     }
   }
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 void MP3StreamState::assignStream(FILE* fid, unsigned fileSize) {
   fFid = fid;
 
@@ -60,10 +54,10 @@ void MP3StreamState::assignStream(FILE* fid, unsigned fileSize) {
     fFileSize = fileSize;
   }
   fNumFramesInFile = 0; // until we know otherwise
-  fIsVBR = False; // ditto
+  fIsVBR = fHasXingTOC = False; // ditto
 
   // Set the first frame's 'presentation time' to the current wall time:
-  gettimeofday(&fNextFramePresentationTime, &Idunno);
+  gettimeofday(&fNextFramePresentationTime, NULL);
 }
 
 struct timeval MP3StreamState::currentFramePlayTime() const {
@@ -80,7 +74,7 @@ struct timeval MP3StreamState::currentFramePlayTime() const {
   return result;
 }
 
-unsigned MP3StreamState::filePlayTime() const {
+float MP3StreamState::filePlayTime() const {
   unsigned numFramesInFile = fNumFramesInFile;
   if (numFramesInFile == 0) {
     // Estimate the number of frames from the file size, and the
@@ -89,8 +83,43 @@ unsigned MP3StreamState::filePlayTime() const {
   }
 
   struct timeval const pt = currentFramePlayTime();
-  float fpt = numFramesInFile*(pt.tv_sec + pt.tv_usec/(float)MILLION);
-  return (unsigned)(fpt + 0.5); // rounds to nearest integer
+  return numFramesInFile*(pt.tv_sec + pt.tv_usec/(float)MILLION);
+}
+
+void MP3StreamState::seekWithinFile(float seekNPT) {
+  if (fFidIsReallyASocket) return; // it's not seekable
+
+  float fileDuration = filePlayTime();
+  if (seekNPT < 0.0) {
+    seekNPT = 0.0;
+  } else if (seekNPT > fileDuration) {
+    seekNPT = fileDuration;
+  }
+  float seekFraction = seekNPT/fileDuration;
+
+  unsigned seekByteNumber;
+  if (fHasXingTOC) {
+    // The file is VBR, with a Xing TOC; use it to determine which byte to seek to:
+    float percent = seekFraction*100.0f;
+    unsigned a = (unsigned)percent;
+    if (a > 99) a = 99;
+
+    unsigned fa = fXingTOC[a];
+    unsigned fb;
+    if (a < 99) {
+      fb = fXingTOC[a+1];
+    } else {
+      fb = 256;
+    }
+    float seekByteFraction = (fa + (fb-fa)*(percent-a))/256.0f;
+
+    seekByteNumber = (unsigned)(seekByteFraction*fFileSize);
+  } else {
+    // Normal case: Treat the file as if it's CBR:
+    seekByteNumber = (unsigned)(seekFraction*fFileSize);
+  }
+
+  fseek(fFid, seekByteNumber, SEEK_SET);
 }
 
 unsigned MP3StreamState::findNextHeader(struct timeval& presentationTime) {
@@ -100,6 +129,14 @@ unsigned MP3StreamState::findNextHeader(struct timeval& presentationTime) {
 
   // From this frame, figure out the *next* frame's presentation time:
   struct timeval framePlayTime = currentFramePlayTime();
+  if (fPresentationTimeScale > 1) {
+    // Scale this value
+    unsigned secondsRem = framePlayTime.tv_sec % fPresentationTimeScale;
+    framePlayTime.tv_sec -= secondsRem;
+    framePlayTime.tv_usec += secondsRem*MILLION;
+    framePlayTime.tv_sec /= fPresentationTimeScale;
+    framePlayTime.tv_usec /= fPresentationTimeScale;
+  }
   fNextFramePresentationTime.tv_usec += framePlayTime.tv_usec;
   fNextFramePresentationTime.tv_sec
     += framePlayTime.tv_sec + fNextFramePresentationTime.tv_usec/MILLION;
@@ -145,15 +182,16 @@ Boolean MP3StreamState::readFrame(unsigned char* outBuf, unsigned outBufSize,
 void MP3StreamState::getAttributes(char* buffer, unsigned bufferSize) const {
   char const* formatStr
     = "bandwidth %d MPEGnumber %d MPEGlayer %d samplingFrequency %d isStereo %d playTime %d isVBR %d";
+  unsigned fpt = (unsigned)(filePlayTime() + 0.5); // rounds to nearest integer
 #if defined(IRIX) || defined(ALPHA) || defined(_QNX4) || defined(IMN_PIM) || defined(CRIS)
   /* snprintf() isn't defined, so just use sprintf() - ugh! */
   sprintf(buffer, formatStr,
 	  fr().bitrate, fr().isMPEG2 ? 2 : 1, fr().layer, fr().samplingFreq, fr().isStereo,
-	  filePlayTime(), fIsVBR);
+	  fpt, fIsVBR);
 #else
   snprintf(buffer, bufferSize, formatStr,
 	  fr().bitrate, fr().isMPEG2 ? 2 : 1, fr().layer, fr().samplingFreq, fr().isStereo,
-	  filePlayTime(), fIsVBR);
+	  fpt, fIsVBR);
 #endif
 }
 
@@ -219,9 +257,11 @@ Boolean MP3StreamState::findNextFrame() {
     fprintf(stderr, "init_resync: fr().hdr: 0x%08x\n", fr().hdr);
 #endif
     if (   (fr().hdr & 0xffe00000) != 0xffe00000
-	|| (fr().hdr & 0x00000C00) == 0x00000C00/* 'stream error' test below */
-	|| (fr().hdr & 0x0000F000) == 0 /* 'free format' test below */
-//#####   || (fr().hdr & 0x00060000) != 0x00020000 /* test for layer 3 */
+	|| (fr().hdr & 0x00060000) == 0 // undefined 'layer' field
+	|| (fr().hdr & 0x0000F000) == 0 // 'free format' bitrate index
+	|| (fr().hdr & 0x0000F000) == 0x0000F000 // undefined bitrate index
+	|| (fr().hdr & 0x00000C00) == 0x00000C00 // undefined frequency index
+	|| (fr().hdr & 0x00000003) != 0x00000000 // 'emphasis' field unexpectedly set
        ) {
       /* RSF: Do the following test even if we're not at the
 	 start of the file, in case we have two or more
@@ -347,7 +387,7 @@ static Boolean socketIsReadable(int socket) {
   timeout.tv_sec = timeout.tv_usec = 0;
 
   int result = select(numFds, &rd_set, NULL, NULL, &timeout);
-  return result > 0;
+  return result != 0; // not > 0, because windows can return -1 for file sockets
 }
 
 static char watchVariable;
@@ -386,23 +426,53 @@ unsigned MP3StreamState::readFromStream(unsigned char* buf,
 
     return totBytesRead;
   } else {
-    waitUntilSocketIsReadable(fEnv, fileno(fFid));
+    waitUntilSocketIsReadable(fEnv, (int)fileno(fFid));
     return fread(buf, 1, numChars, fFid);
   }
 }
 
+#define XING_FRAMES_FLAG       0x0001
+#define XING_BYTES_FLAG        0x0002
+#define XING_TOC_FLAG          0x0004
+#define XING_VBR_SCALE_FLAG    0x0008
+
 void MP3StreamState::checkForXingHeader() {
   // Look for 'Xing' in the first 4 bytes after the 'side info':
-  if (fr().frameSize < fr().sideInfoSize + 12) return;
+  if (fr().frameSize < fr().sideInfoSize) return;
+  unsigned bytesAvailable = fr().frameSize - fr().sideInfoSize;
   unsigned char* p = &(fr().frameBytes[fr().sideInfoSize]);
+
+  if (bytesAvailable < 8) return;
   if (p[0] != 'X' || p[1] != 'i' || p[2] != 'n' || p[3] != 'g') return;
 
   // We found it.
   fIsVBR = True;
 
-  // Next, check whether there's a '# of frames' field:
-  if (!(p[7]&1)) return;
+  u_int32_t flags = (p[4]<<24) | (p[5]<<16) | (p[6]<<8) | p[7];
+  unsigned i = 8;
+  bytesAvailable -= 8;
 
-  // The next 4 bytes are the number of frames:
-  fNumFramesInFile = (p[8]<<24)|(p[9]<<16)|(p[10]<<8)|(p[11]);
+  if (flags&XING_FRAMES_FLAG) {
+    // The next 4 bytes are the number of frames:
+    if (bytesAvailable < 4) return;
+    fNumFramesInFile = (p[i]<<24)|(p[i+1]<<16)|(p[i+2]<<8)|(p[i+3]);
+    i += 4; bytesAvailable -= 4;
+  }
+
+  if (flags&XING_BYTES_FLAG) {
+    // The next 4 bytes is the file size:
+    if (bytesAvailable < 4) return;
+    fFileSize = (p[i]<<24)|(p[i+1]<<16)|(p[i+2]<<8)|(p[i+3]);
+    i += 4; bytesAvailable -= 4;
+  }
+
+  if (flags&XING_TOC_FLAG) {
+    // Fill in the Xing 'table of contents': 
+    if (bytesAvailable < XING_TOC_LENGTH) return;
+    fHasXingTOC = True;
+    for (unsigned j = 0; j < XING_TOC_LENGTH; ++j) {
+      fXingTOC[j] = p[i+j];
+    }
+    i += XING_TOC_FLAG; bytesAvailable -= XING_TOC_FLAG;
+  }
 }
