@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // MPEG-1 or MPEG-2 Audio RTP Sources
 // Implementation
 
@@ -57,6 +57,6 @@ Boolean MPEG1or2AudioRTPSource
 }    
 
 char const* MPEG1or2AudioRTPSource::MIMEtype() const {
-  return "audio/mpeg";
+  return "audio/MPEG";
 }
 
