@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up an MPEG (1,2) audio elementary stream into frames
 // Implementation
 
@@ -50,17 +50,12 @@ private:
 
 ////////// MPEG1or2AudioStreamFramer implementation //////////
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
-MPEG1or2AudioStreamFramer::MPEG1or2AudioStreamFramer(UsageEnvironment& env,
-					     FramedSource* inputSource)
-  : FramedFilter(env, inputSource) {
-  // Use the current wallclock time as the initial 'presentation time':
-  gettimeofday(&fNextFramePresentationTime, &Idunno);
+MPEG1or2AudioStreamFramer
+::MPEG1or2AudioStreamFramer(UsageEnvironment& env, FramedSource* inputSource,
+			    Boolean syncWithInputSource)
+  : FramedFilter(env, inputSource),
+    fSyncWithInputSource(syncWithInputSource) {
+  reset();
 
   fParser = new MPEG1or2AudioStreamParser(this, inputSource);
 }
@@ -71,9 +66,27 @@ MPEG1or2AudioStreamFramer::~MPEG1or2AudioStreamFramer() {
 
 MPEG1or2AudioStreamFramer*
 MPEG1or2AudioStreamFramer::createNew(UsageEnvironment& env,
-				 FramedSource* inputSource) {
+				     FramedSource* inputSource,
+				     Boolean syncWithInputSource) {
   // Need to add source type checking here???  #####
-  return new MPEG1or2AudioStreamFramer(env, inputSource);
+  return new MPEG1or2AudioStreamFramer(env, inputSource, syncWithInputSource);
+}
+
+void MPEG1or2AudioStreamFramer::flushInput() {
+  reset();
+  fParser->flushInput();
+}
+
+void MPEG1or2AudioStreamFramer::reset() {
+  // Use the current wallclock time as the initial 'presentation time':
+  struct timeval timeNow;
+  gettimeofday(&timeNow, NULL);
+  resetPresentationTime(timeNow);
+}
+
+void MPEG1or2AudioStreamFramer
+::resetPresentationTime(struct timeval newPresentationTime) {
+  fNextFramePresentationTime = newPresentationTime;
 }
 
 void MPEG1or2AudioStreamFramer::doGetNextFrame() {
@@ -88,22 +101,32 @@ static unsigned const numSamplesByLayer[4] = {0, 384, 1152, 1152};
 struct timeval MPEG1or2AudioStreamFramer::currentFramePlayTime() const {
   MP3FrameParams const& fr = fParser->currentFrame();
   unsigned const numSamples = numSamplesByLayer[fr.layer];
+
+  struct timeval result;
   unsigned const freq = fr.samplingFreq*(1 + fr.isMPEG2);
+  if (freq == 0) {
+    result.tv_sec = 0;
+    result.tv_usec = 0;
+    return result;
+  }
 
   // result is numSamples/freq
   unsigned const uSeconds
     = ((numSamples*2*MILLION)/freq + 1)/2; // rounds to nearest integer
 
-  struct timeval result;
   result.tv_sec = uSeconds/MILLION;
   result.tv_usec = uSeconds%MILLION;
   return result;
 }
 
-void MPEG1or2AudioStreamFramer::continueReadProcessing(void* clientData,
-						   unsigned char* /*ptr*/,
-						   unsigned /*size*/) {
+void MPEG1or2AudioStreamFramer
+::continueReadProcessing(void* clientData,
+			 unsigned char* /*ptr*/, unsigned /*size*/,
+			 struct timeval presentationTime) {
   MPEG1or2AudioStreamFramer* framer = (MPEG1or2AudioStreamFramer*)clientData;
+  if (framer->fSyncWithInputSource) {
+    framer->resetPresentationTime(presentationTime);
+  }
   framer->continueReadProcessing();
 }
 
@@ -117,7 +140,6 @@ void MPEG1or2AudioStreamFramer::continueReadProcessing() {
     // Also set the presentation time, and increment it for next time,
     // based on the length of this frame:
     fPresentationTime = fNextFramePresentationTime;
-
     struct timeval framePlayTime = currentFramePlayTime();
     fDurationInMicroseconds = framePlayTime.tv_sec*MILLION + framePlayTime.tv_usec;
     fNextFramePresentationTime.tv_usec += framePlayTime.tv_usec;
@@ -159,7 +181,7 @@ unsigned MPEG1or2AudioStreamParser::parse(unsigned& numTruncatedBytes) {
     saveParserState();
     
     // We expect a MPEG audio header (first 11 bits set to 1) at the start:
-    while ((fCurrentFrame.hdr = test4Bytes())&0xFFE00000 != 0xFFE00000) {
+    while (((fCurrentFrame.hdr = test4Bytes())&0xFFE00000) != 0xFFE00000) {
       skipBytes(1);
       saveParserState();
     }
