@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // Demultiplexer for a MPEG 1 or 2 Program Stream
 // Implementation
 
@@ -55,7 +55,24 @@ private:
 private:
   MPEG1or2Demux* fUsingSource;
   MPEGParseState fCurrentParseState;
-  unsigned char fMPEGversion;   
+};
+
+
+////////// MPEG1or2Demux::OutputDescriptor::SavedData definition/implementation //////////
+
+class MPEG1or2Demux::OutputDescriptor::SavedData {
+public:
+  SavedData(unsigned char* buf, unsigned size)
+    : next(NULL), data(buf), dataSize(size), numBytesUsed(0) {
+  }
+  virtual ~SavedData() {
+    delete[] data;
+    delete next;
+  }
+
+  SavedData* next;
+  unsigned char* data;
+  unsigned dataSize, numBytesUsed;
 };
 
 
@@ -64,12 +81,15 @@ private:
 MPEG1or2Demux
 ::MPEG1or2Demux(UsageEnvironment& env,
 		FramedSource* inputSource, Boolean reclaimWhenLastESDies)
-  : Medium(env), fInputSource(inputSource),
+  : Medium(env),
+    fInputSource(inputSource), fMPEGversion(0),
     fNextAudioStreamNumber(0), fNextVideoStreamNumber(0),
     fReclaimWhenLastESDies(reclaimWhenLastESDies), fNumOutstandingESs(0),
     fNumPendingReads(0), fHaveUndeliveredData(False) {
   fParser = new MPEGProgramStreamParser(this, inputSource);
   for (unsigned i = 0; i < 256; ++i) {
+    fOutput[i].savedDataHead = fOutput[i].savedDataTail = NULL;
+    fOutput[i].isPotentiallyReadable = False;
     fOutput[i].isCurrentlyActive = False;
     fOutput[i].isCurrentlyAwaitingData = False;
   } 
@@ -77,6 +97,7 @@ MPEG1or2Demux
 
 MPEG1or2Demux::~MPEG1or2Demux() {
   delete fParser;
+  for (unsigned i = 0; i < 256; ++i) delete fOutput[i].savedDataHead;
   Medium::close(fInputSource);
 }
 
@@ -88,16 +109,25 @@ MPEG1or2Demux* MPEG1or2Demux
   return new MPEG1or2Demux(env, inputSource, reclaimWhenLastESDies);
 }
 
+MPEG1or2Demux::SCR::SCR()
+  : highBit(0), remainingBits(0), extension(0), isValid(False) {
+}
+
 void MPEG1or2Demux
 ::noteElementaryStreamDeletion(MPEG1or2DemuxedElementaryStream* /*es*/) {
   if (--fNumOutstandingESs == 0 && fReclaimWhenLastESDies) {
-    delete this;
+    Medium::close(this);
   }
 }
 
+void MPEG1or2Demux::flushInput() {
+  fParser->flushInput();
+}
+
 MPEG1or2DemuxedElementaryStream*
 MPEG1or2Demux::newElementaryStream(u_int8_t streamIdTag) {
   ++fNumOutstandingESs;
+  fOutput[streamIdTag].isPotentiallyReadable = True;
   return new MPEG1or2DemuxedElementaryStream(envir(), streamIdTag, *this);
 }
 
@@ -113,6 +143,13 @@ MPEG1or2DemuxedElementaryStream* MPEG1or2Demux::newVideoStream() {
   return newElementaryStream(newVideoStreamTag);
 }
 
+// Appropriate one of the reserved stream id tags to mean: return raw PES packets:
+#define RAW_PES 0xFC
+
+MPEG1or2DemuxedElementaryStream* MPEG1or2Demux::newRawPESStream() {
+  return newElementaryStream(RAW_PES);
+}
+
 void MPEG1or2Demux::registerReadInterest(u_int8_t streamIdTag,
 				     unsigned char* to, unsigned maxSize,
 				     FramedSource::afterGettingFunc* afterGettingFunc,
@@ -140,9 +177,48 @@ void MPEG1or2Demux::registerReadInterest(u_int8_t streamIdTag,
   ++fNumPendingReads;
 }
 
-void MPEG1or2Demux::continueReadProcessing(void* clientData,
-				       unsigned char* /*ptr*/,
-				       unsigned /*size*/) {
+Boolean MPEG1or2Demux::useSavedData(u_int8_t streamIdTag,
+				    unsigned char* to, unsigned maxSize,
+				    FramedSource::afterGettingFunc* afterGettingFunc,
+				    void* afterGettingClientData) {
+  struct OutputDescriptor& out = fOutput[streamIdTag];
+  if (out.savedDataHead == NULL) return False; // common case
+
+  unsigned totNumBytesCopied = 0;
+  while (maxSize > 0 && out.savedDataHead != NULL) {
+    OutputDescriptor::SavedData& savedData = *(out.savedDataHead);
+    unsigned char* from = &savedData.data[savedData.numBytesUsed];
+    unsigned numBytesToCopy = savedData.dataSize - savedData.numBytesUsed;
+    if (numBytesToCopy > maxSize) numBytesToCopy = maxSize;
+    memmove(to, from, numBytesToCopy);
+    to += numBytesToCopy;
+    maxSize -= numBytesToCopy;
+    out.savedDataTotalSize -= numBytesToCopy;
+    totNumBytesCopied += numBytesToCopy;
+    savedData.numBytesUsed += numBytesToCopy;
+    if (savedData.numBytesUsed == savedData.dataSize) {
+      out.savedDataHead = savedData.next;
+      if (out.savedDataHead == NULL) out.savedDataTail = NULL;
+      savedData.next = NULL;
+      delete &savedData;
+    }
+  }
+
+  out.isCurrentlyActive = True;
+  if (afterGettingFunc != NULL) {
+    struct timeval presentationTime;
+    presentationTime.tv_sec = 0; presentationTime.tv_usec = 0; // should fix #####
+    (*afterGettingFunc)(afterGettingClientData, totNumBytesCopied,
+			0 /* numTruncatedBytes */, presentationTime,
+			0 /* durationInMicroseconds ?????#####*/);
+  }
+  return True;
+}
+
+void MPEG1or2Demux
+::continueReadProcessing(void* clientData,
+			 unsigned char* /*ptr*/, unsigned /*size*/,
+			 struct timeval /*presentationTime*/) {
   MPEG1or2Demux* demux = (MPEG1or2Demux*)clientData;
   demux->continueReadProcessing();
 }
@@ -180,12 +256,18 @@ void MPEG1or2Demux::continueReadProcessing() {
 }
 
 void MPEG1or2Demux::getNextFrame(u_int8_t streamIdTag,
-			     unsigned char* to, unsigned maxSize,
-			     FramedSource::afterGettingFunc* afterGettingFunc,
-			     void* afterGettingClientData,
-			     FramedSource::onCloseFunc* onCloseFunc,
-			     void* onCloseClientData) {
-  // Begin by saving the parameters of the specified stream id:
+				 unsigned char* to, unsigned maxSize,
+				 FramedSource::afterGettingFunc* afterGettingFunc,
+				 void* afterGettingClientData,
+				 FramedSource::onCloseFunc* onCloseFunc,
+				 void* onCloseClientData) {
+  // First, check whether we have saved data for this stream id:
+  if (useSavedData(streamIdTag, to, maxSize,
+		   afterGettingFunc, afterGettingClientData)) {
+    return;
+  }
+
+  // Then save the parameters of the specified stream id:
   registerReadInterest(streamIdTag, to, maxSize,
 		       afterGettingFunc, afterGettingClientData,
 		       onCloseFunc, onCloseClientData);
@@ -225,13 +307,17 @@ void MPEG1or2Demux::handleClosure(void* clientData) {
 	++numPending;
       }
     }
-    out.isCurrentlyActive = out.isCurrentlyAwaitingData = False;
+    delete out.savedDataHead; out.savedDataHead = out.savedDataTail = NULL;
+    out.savedDataTotalSize = 0;
+    out.isPotentiallyReadable = out.isCurrentlyActive = out.isCurrentlyAwaitingData
+      = False;
   } 
   for (i = 0; i < numPending; ++i) {
     (*savedPending[i].fOnCloseFunc)(savedPending[i].onCloseClientData);
   }
 }
 
+
 ////////// MPEGProgramStreamParser implementation //////////
 
 #include <string.h>
@@ -296,12 +382,15 @@ void MPEGProgramStreamParser::parsePackHeader() {
   fprintf(stderr, "parsing pack header\n"); fflush(stderr);
 #endif
   unsigned first4Bytes;
-  do {
+  while (1) {
     first4Bytes = test4Bytes();
 
     // We're supposed to have a pack header here, but check also for
     // a system header or a PES packet, just in case:
-    if (first4Bytes == SYSTEM_HEADER_START_CODE) {
+    if (first4Bytes == PACK_START_CODE) {
+      skipBytes(4);
+      break;
+    } else if (first4Bytes == SYSTEM_HEADER_START_CODE) {
 #ifdef DEBUG
       fprintf(stderr, "found system header instead of pack header\n");
 #endif
@@ -314,55 +403,55 @@ void MPEGProgramStreamParser::parsePackHeader() {
       setParseState(PARSING_PES_PACKET);
       return;
     }
-    // Perhaps later check for elementary stream data here as well?? #####
 
     setParseState(PARSING_PACK_HEADER); // ensures we progress over bad data
-    skipBytes(4);
-#ifdef DEBUG
-    fprintf(stderr, (first4Bytes == PACK_START_CODE) ? "found pack\n" : "didn't find pack\n"); fflush(stderr);
-#endif
-  } while (first4Bytes != PACK_START_CODE);
+    if ((first4Bytes&0xFF) > 1) { // a system code definitely doesn't start here
+      skipBytes(4);
+    } else {
+      skipBytes(1);
+    }
+  }
   
   // The size of the pack header differs depending on whether it's
   // MPEG-1 or MPEG-2.  The next byte tells us this: 
   unsigned char nextByte = get1Byte();
+  MPEG1or2Demux::SCR& scr = fUsingSource->fLastSeenSCR; // alias
   if ((nextByte&0xF0) == 0x20) { // MPEG-1
-    fMPEGversion = 1;
-#ifdef DEBUG_TIMESTAMPS
-    unsigned char scrb_highBit =  (nextByte&0x08)>>3;
-    unsigned scrb_remainingBits = (nextByte&0x06)<<29;
+    fUsingSource->fMPEGversion = 1;
+    scr.highBit =  (nextByte&0x08)>>3;
+    scr.remainingBits = (nextByte&0x06)<<29;
     unsigned next4Bytes = get4Bytes();
-    scrb_remainingBits |= (next4Bytes&0xFFFE0000)>>2;
-    scrb_remainingBits |= (next4Bytes&0x0000FFFE)>>1;
+    scr.remainingBits |= (next4Bytes&0xFFFE0000)>>2;
+    scr.remainingBits |= (next4Bytes&0x0000FFFE)>>1;
+    scr.extension = 0;
+    scr.isValid = True;
     skipBits(24);
 
+#if defined(DEBUG_TIMESTAMPS) || defined(DEBUG_SCR_TIMESTAMPS)
     fprintf(stderr, "pack hdr system_clock_reference_base: 0x%x",
-	    scrb_highBit);
-    fprintf(stderr, "%08x\n", scrb_remainingBits);
-#else
-    skipBits(56);
+	    scr.highBit);
+    fprintf(stderr, "%08x\n", scr.remainingBits);
 #endif
   } else if ((nextByte&0xC0) == 0x40) { // MPEG-2
-    fMPEGversion = 2;
-#ifdef DEBUG_TIMESTAMPS
-    unsigned char scrb_highBit =  (nextByte&0x20)>>5;
-    unsigned scrb_remainingBits = (nextByte&0x18)<<27;
-    scrb_remainingBits |= (nextByte&0x03)<<28;
+    fUsingSource->fMPEGversion = 2;
+    scr.highBit =  (nextByte&0x20)>>5;
+    scr.remainingBits = (nextByte&0x18)<<27;
+    scr.remainingBits |= (nextByte&0x03)<<28;
     unsigned next4Bytes = get4Bytes();
-    scrb_remainingBits |= (next4Bytes&0xFFF80000)>>4;
-    scrb_remainingBits |= (next4Bytes&0x0003FFF8)>>3;
-    unsigned short scre = (next4Bytes&0x00000003)<<7;
+    scr.remainingBits |= (next4Bytes&0xFFF80000)>>4;
+    scr.remainingBits |= (next4Bytes&0x0003FFF8)>>3;
+    scr.extension = (next4Bytes&0x00000003)<<7;
     next4Bytes = get4Bytes();
-    scre               |= (next4Bytes&0xFE000000)>>25;
+    scr.extension |= (next4Bytes&0xFE000000)>>25;
+    scr.isValid = True;
     skipBits(5);
 
+#if defined(DEBUG_TIMESTAMPS) || defined(DEBUG_SCR_TIMESTAMPS)
     fprintf(stderr, "pack hdr system_clock_reference_base: 0x%x",
-	    scrb_highBit);
-    fprintf(stderr, "%08x\n", scrb_remainingBits);
+	    scr.highBit);
+    fprintf(stderr, "%08x\n", scr.remainingBits);
     fprintf(stderr, "pack hdr system_clock_reference_extension: 0x%03x\n",
-	    scre);
-#else
-    skipBits(69);
+	    scr.extension);
 #endif
     unsigned char pack_stuffing_length = getBits(3);
     skipBytes(pack_stuffing_length);
@@ -412,7 +501,9 @@ void MPEGProgramStreamParser::parseSystemHeader() {
 // A test for stream ids that are exempt from normal PES packet header parsing
 Boolean MPEGProgramStreamParser
 ::isSpecialStreamId(unsigned char stream_id) const {
-  if (fMPEGversion == 1) {
+  if (stream_id == RAW_PES) return True; // hack
+
+  if (fUsingSource->fMPEGversion == 1) {
     return stream_id == private_stream_2;
   } else { // assume MPEG-2
     if (stream_id <= private_stream_2) {
@@ -478,6 +569,10 @@ unsigned char MPEGProgramStreamParser::parsePESPacket() {
 
   // Parse over the rest of the header, until we get to the packet data itself.
   // This varies depending upon the MPEG version:
+  if (fUsingSource->fOutput[RAW_PES].isPotentiallyReadable) {
+    // Hack: We've been asked to return raw PES packets, for every stream:
+    stream_id = RAW_PES;
+  } 
   unsigned savedParserOffset = curOffset();
 #ifdef DEBUG_TIMESTAMPS
   unsigned char pts_highBit = 0;
@@ -485,7 +580,7 @@ unsigned char MPEGProgramStreamParser::parsePESPacket() {
   unsigned char dts_highBit = 0;
   unsigned dts_remainingBits = 0;
 #endif
-  if (fMPEGversion == 1) {
+  if (fUsingSource->fMPEGversion == 1) {
     if (!isSpecialStreamId(stream_id)) {
       unsigned char nextByte;
       while ((nextByte = get1Byte()) == 0xFF) { // stuffing_byte
@@ -584,6 +679,11 @@ unsigned char MPEGProgramStreamParser::parsePESPacket() {
   unsigned char acquiredStreamIdTag = 0;
   unsigned currentParserOffset = curOffset();
   unsigned bytesSkipped = currentParserOffset - savedParserOffset;
+  if (stream_id == RAW_PES) {
+    restoreSavedParserState(); // so we deliver from the beginning of the PES packet
+    PES_packet_length += 6; // to include the whole of the PES packet
+    bytesSkipped = 0;
+  }
   if (PES_packet_length < bytesSkipped) {
     fUsingSource->envir() << "StreamParser::parsePESPacket(): saw inconsistent PES_packet_length "
 			  << PES_packet_length << " < "
@@ -627,6 +727,22 @@ unsigned char MPEGProgramStreamParser::parsePESPacket() {
       restoreSavedParserState(); // so we read from the beginning next time
       fUsingSource->fHaveUndeliveredData = True;
       throw READER_NOT_READY;
+    } else if (out.isPotentiallyReadable &&
+	       out.savedDataTotalSize + PES_packet_length < 1000000 /*limit*/) {
+      // Someone is interested in this stream, but hasn't begun reading it yet.
+      // Save this data, so that the reader will get it when he later asks for it.
+      unsigned char* buf = new unsigned char[PES_packet_length];
+      getBytes(buf, PES_packet_length);
+      MPEG1or2Demux::OutputDescriptor::SavedData* savedData
+	= new MPEG1or2Demux::OutputDescriptor::SavedData(buf, PES_packet_length);
+      if (out.savedDataHead == NULL) {
+	out.savedDataHead = out.savedDataTail = savedData;
+      } else {
+	out.savedDataTail->next = savedData;
+	out.savedDataTail = savedData;
+      }
+      out.savedDataTotalSize += PES_packet_length;
+      PES_packet_length = 0;
     }
     skipBytes(PES_packet_length);
   }
