@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A MPEG 1 or 2 Elementary Stream, demultiplexed from a Program Stream
 // Implementation
 
@@ -26,14 +26,12 @@ MPEG1or2DemuxedElementaryStream::
 MPEG1or2DemuxedElementaryStream(UsageEnvironment& env, u_int8_t streamIdTag,
 			    MPEG1or2Demux& sourceDemux)
   : FramedSource(env),
-    fOurStreamIdTag(streamIdTag), fOurSourceDemux(sourceDemux) {
-  lastSeenSCR.highBit = 0; lastSeenSCR.remainingBits = 0; lastSeenSCR.extension = 0;
-
+    fOurStreamIdTag(streamIdTag), fOurSourceDemux(sourceDemux), fMPEGversion(0) {
   // Set our MIME type string for known media types:
   if ((streamIdTag&0xE0) == 0xC0) {
-    fMIMEtype = "audio/mpeg";
+    fMIMEtype = "audio/MPEG";
   } else if ((streamIdTag&0xF0) == 0xE0) {
-    fMIMEtype = "video/mpeg";
+    fMIMEtype = "video/MPEG";
   } else {
     fMIMEtype = MediaSource::MIMEtype();
   }
@@ -58,9 +56,9 @@ char const* MPEG1or2DemuxedElementaryStream::MIMEtype() const {
 }
 
 unsigned MPEG1or2DemuxedElementaryStream::maxFrameSize() const {
-  return 25000;
-  // This is a hack, which might break for some MPEG sources, because
-  // the MPEG spec allows for PES packets as large as ~65536 bytes. #####
+  return 6+65535;
+      // because the MPEG spec allows for PES packets as large as
+      // (6 + 65535) bytes (header + data)
 }
 
 void MPEG1or2DemuxedElementaryStream
@@ -83,9 +81,8 @@ void MPEG1or2DemuxedElementaryStream
   fPresentationTime = presentationTime;
   fDurationInMicroseconds = durationInMicroseconds;
 
-  lastSeenSCR.highBit = fOurSourceDemux.lastSeenSCR.highBit;
-  lastSeenSCR.remainingBits = fOurSourceDemux.lastSeenSCR.remainingBits;
-  lastSeenSCR.extension = fOurSourceDemux.lastSeenSCR.extension;
+  fLastSeenSCR = fOurSourceDemux.lastSeenSCR();
+  fMPEGversion = fOurSourceDemux.mpegVersion();
 
   FramedSource::afterGetting(this);
 }
