@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand, from a MPEG-1 or 2 demuxer.
 // Implementation
@@ -26,19 +26,21 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #include "MPEG1or2VideoRTPSink.hh"
 #include "AC3AudioStreamFramer.hh"
 #include "AC3AudioRTPSink.hh"
+#include "ByteStreamFileSource.hh"
 
 MPEG1or2DemuxedServerMediaSubsession* MPEG1or2DemuxedServerMediaSubsession
 ::createNew(MPEG1or2FileServerDemux& demux, u_int8_t streamIdTag,
-	    Boolean iFramesOnly, double vshPeriod) {
+	    Boolean reuseFirstSource, Boolean iFramesOnly, double vshPeriod) {
   return new MPEG1or2DemuxedServerMediaSubsession(demux, streamIdTag,
+						  reuseFirstSource,
 						  iFramesOnly, vshPeriod);
 }
 
 MPEG1or2DemuxedServerMediaSubsession
 ::MPEG1or2DemuxedServerMediaSubsession(MPEG1or2FileServerDemux& demux,
-				       u_int8_t streamIdTag,
+				       u_int8_t streamIdTag, Boolean reuseFirstSource,
 				       Boolean iFramesOnly, double vshPeriod)
-  : OnDemandServerMediaSubsession(demux.envir()),
+  : OnDemandServerMediaSubsession(demux.envir(), reuseFirstSource),
     fOurDemux(demux), fStreamIdTag(streamIdTag),
     fIFramesOnly(iFramesOnly), fVSHPeriod(vshPeriod) {
 }
@@ -55,14 +57,14 @@ FramedSource* MPEG1or2DemuxedServerMediaSubsession
 
     if ((fStreamIdTag&0xF0) == 0xC0 /*MPEG audio*/) {
       estBitrate = 128; // kbps, estimate
-      return MPEG1or2AudioStreamFramer::createNew(fOurDemux.envir(), es);
+      return MPEG1or2AudioStreamFramer::createNew(envir(), es);
     } else if ((fStreamIdTag&0xF0) == 0xE0 /*video*/) {
       estBitrate = 500; // kbps, estimate
-      return MPEG1or2VideoStreamFramer::createNew(fOurDemux.envir(), es,
+      return MPEG1or2VideoStreamFramer::createNew(envir(), es,
 						  fIFramesOnly, fVSHPeriod);
     } else if (fStreamIdTag == 0xBD /*AC-3 audio*/) {
       estBitrate = 192; // kbps, estimate
-      return AC3AudioStreamFramer::createNew(fOurDemux.envir(), es);
+      return AC3AudioStreamFramer::createNew(envir(), es);
     } else { // unknown stream type
       break;
     }
@@ -90,3 +92,43 @@ RTPSink* MPEG1or2DemuxedServerMediaSubsession
     return NULL;
   }
 }
+
+void MPEG1or2DemuxedServerMediaSubsession
+::seekStreamSource(FramedSource* inputSource, float seekNPT) {
+  float const dur = duration();
+  unsigned const size = fOurDemux.fileSize();
+  unsigned absBytePosition = dur == 0.0 ? 0 : (unsigned)((seekNPT/dur)*size);
+
+  // "inputSource" is a 'framer'
+  // Flush its data, to account for the seek that we're about to do:
+  if ((fStreamIdTag&0xF0) == 0xC0 /*MPEG audio*/) {
+    MPEG1or2AudioStreamFramer* framer = (MPEG1or2AudioStreamFramer*)inputSource;
+    framer->flushInput();
+  } else if ((fStreamIdTag&0xF0) == 0xE0 /*video*/) {
+    MPEG1or2VideoStreamFramer* framer = (MPEG1or2VideoStreamFramer*)inputSource;
+    framer->flushInput();
+  }
+
+  // "inputSource" is a filter; its input source is the original elem stream source:
+  MPEG1or2DemuxedElementaryStream* elemStreamSource
+    = (MPEG1or2DemuxedElementaryStream*)(((FramedFilter*)inputSource)->inputSource());
+
+  // Next, get the original source demux:
+  MPEG1or2Demux& sourceDemux = elemStreamSource->sourceDemux();
+
+  // and flush its input buffers:
+  sourceDemux.flushInput();
+
+  // Then, get the original input file stream from the source demux:
+  ByteStreamFileSource* inputFileSource
+    = (ByteStreamFileSource*)(sourceDemux.inputSource());
+  // Note: We can make that cast, because we know that the demux was originally
+  // created from a "ByteStreamFileSource".
+
+  // Do the appropriate seek within the input file stream:
+  inputFileSource->seekToByteAbsolute(absBytePosition);
+}
+
+float MPEG1or2DemuxedServerMediaSubsession::duration() const {
+  return fOurDemux.fileDuration();
+}
