@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand, from a MPEG-1 or 2 Elementary Stream video file.
 // Implementation
@@ -27,18 +27,20 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 MPEG1or2VideoFileServerMediaSubsession*
 MPEG1or2VideoFileServerMediaSubsession::createNew(UsageEnvironment& env,
 						  char const* fileName,
+						  Boolean reuseFirstSource,
 						  Boolean iFramesOnly,
 						  double vshPeriod) {
-  return new MPEG1or2VideoFileServerMediaSubsession(env, fileName,
+  return new MPEG1or2VideoFileServerMediaSubsession(env, fileName, reuseFirstSource,
 						    iFramesOnly, vshPeriod);
 }
 
 MPEG1or2VideoFileServerMediaSubsession
 ::MPEG1or2VideoFileServerMediaSubsession(UsageEnvironment& env,
 					 char const* fileName,
+					 Boolean reuseFirstSource,
 					 Boolean iFramesOnly,
 					 double vshPeriod)
-  : FileServerMediaSubsession(env, fileName),
+  : FileServerMediaSubsession(env, fileName, reuseFirstSource),
     fIFramesOnly(iFramesOnly), fVSHPeriod(vshPeriod) {
 }
 
@@ -53,6 +55,7 @@ FramedSource* MPEG1or2VideoFileServerMediaSubsession
   ByteStreamFileSource* fileSource
     = ByteStreamFileSource::createNew(envir(), fFileName);
   if (fileSource == NULL) return NULL;
+  fFileSize = fileSource->fileSize();
 
   return MPEG1or2VideoStreamFramer
     ::createNew(envir(), fileSource, fIFramesOnly, fVSHPeriod);
