@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up an MPEG 1 or 2 video elementary stream into
 //   frames for: Video_Sequence_Header, GOP_Header, Picture_Header
 // Implementation
@@ -30,6 +30,7 @@ enum MPEGParseState {
   PARSING_VIDEO_SEQUENCE_HEADER,
   PARSING_VIDEO_SEQUENCE_HEADER_SEEN_CODE,
   PARSING_GOP_HEADER,
+  PARSING_GOP_HEADER_SEEN_CODE,
   PARSING_PICTURE_HEADER,
   PARSING_SLICE
 }; 
@@ -44,16 +45,19 @@ public:
   virtual ~MPEG1or2VideoStreamParser();
 
 private: // redefined virtual functions:
+  virtual void flushInput();
   virtual unsigned parse();
 
 private:
+  void reset();
+
   MPEG1or2VideoStreamFramer* usingSource() {
     return (MPEG1or2VideoStreamFramer*)fUsingSource;
   }
   void setParseState(MPEGParseState parseState);
 
-  unsigned parseVideoSequenceHeader(Boolean haveSeenStartCode = False);
-  unsigned parseGOPHeader();
+  unsigned parseVideoSequenceHeader(Boolean haveSeenStartCode);
+  unsigned parseGOPHeader(Boolean haveSeenStartCode);
   unsigned parsePictureHeader();
   unsigned parseSlice();
 
@@ -82,12 +86,15 @@ private:
 ////////// MPEG1or2VideoStreamFramer implementation //////////
 
 MPEG1or2VideoStreamFramer::MPEG1or2VideoStreamFramer(UsageEnvironment& env,
-					     FramedSource* inputSource,
-					     Boolean iFramesOnly,
-					     double vshPeriod)
+						     FramedSource* inputSource,
+						     Boolean iFramesOnly,
+						     double vshPeriod,
+						     Boolean createParser)
   : MPEGVideoStreamFramer(env, inputSource) {
-  fParser = new MPEG1or2VideoStreamParser(this, inputSource,
-					  iFramesOnly, vshPeriod);
+  fParser = createParser
+    ? new MPEG1or2VideoStreamParser(this, inputSource,
+				    iFramesOnly, vshPeriod)
+    : NULL;
 }
 
 MPEG1or2VideoStreamFramer::~MPEG1or2VideoStreamFramer() {
@@ -117,10 +124,9 @@ MPEG1or2VideoStreamParser
 			FramedSource* inputSource,
 			Boolean iFramesOnly, double vshPeriod)
   : MPEGVideoStreamParser(usingSource, inputSource),
-  fCurrentParseState(PARSING_VIDEO_SEQUENCE_HEADER),
-  fPicturesSinceLastGOP(0), fCurPicTemporalReference(0),
-  fCurrentSliceNumber(0), fSavedVSHSize(0), fVSHPeriod(vshPeriod),
-  fIFramesOnly(iFramesOnly), fSkippingCurrentPicture(False) {
+    fCurrentParseState(PARSING_VIDEO_SEQUENCE_HEADER),
+    fVSHPeriod(vshPeriod), fIFramesOnly(iFramesOnly) {
+  reset();
 }
 
 MPEG1or2VideoStreamParser::~MPEG1or2VideoStreamParser() {
@@ -131,17 +137,36 @@ void MPEG1or2VideoStreamParser::setParseState(MPEGParseState parseState) {
   MPEGVideoStreamParser::setParseState();
 }
 
+void MPEG1or2VideoStreamParser::reset() {
+  fPicturesSinceLastGOP = 0;
+  fCurPicTemporalReference = 0;
+  fCurrentSliceNumber = 0;
+  fSavedVSHSize = 0;
+  fSkippingCurrentPicture = False;
+}
+
+void MPEG1or2VideoStreamParser::flushInput() {
+  reset();
+  StreamParser::flushInput();
+  if (fCurrentParseState != PARSING_VIDEO_SEQUENCE_HEADER) {
+    setParseState(PARSING_GOP_HEADER); // start from the next GOP
+  }
+}
+
 unsigned MPEG1or2VideoStreamParser::parse() {
   try {
     switch (fCurrentParseState) {
     case PARSING_VIDEO_SEQUENCE_HEADER: {
-      return parseVideoSequenceHeader();
+      return parseVideoSequenceHeader(False);
     }
     case PARSING_VIDEO_SEQUENCE_HEADER_SEEN_CODE: {
       return parseVideoSequenceHeader(True);
     }
     case PARSING_GOP_HEADER: {
-      return parseGOPHeader();
+      return parseGOPHeader(False);
+    }
+    case PARSING_GOP_HEADER_SEEN_CODE: {
+      return parseGOPHeader(True);
     }
     case PARSING_PICTURE_HEADER: {
       return parsePictureHeader();
@@ -222,13 +247,14 @@ unsigned MPEG1or2VideoStreamParser
 #endif
   unsigned first4Bytes;
   if (!haveSeenStartCode) {
-    while ((first4Bytes = get4Bytes()) != VIDEO_SEQUENCE_HEADER_START_CODE) {
+    while ((first4Bytes = test4Bytes()) != VIDEO_SEQUENCE_HEADER_START_CODE) {
 #ifdef DEBUG
       fprintf(stderr, "ignoring non video sequence header: 0x%08x\n", first4Bytes);
 #endif
-      setParseState(PARSING_VIDEO_SEQUENCE_HEADER);
+      get1Byte(); setParseState(PARSING_VIDEO_SEQUENCE_HEADER);
           // ensures we progress over bad data
     }
+    first4Bytes = get4Bytes();
   } else {
     // We've already seen the start code
     first4Bytes = VIDEO_SEQUENCE_HEADER_START_CODE;
@@ -259,7 +285,7 @@ unsigned MPEG1or2VideoStreamParser
   } while (next4Bytes != GROUP_START_CODE && next4Bytes != PICTURE_START_CODE);
   
   setParseState((next4Bytes == GROUP_START_CODE)
-		? PARSING_GOP_HEADER : PARSING_PICTURE_HEADER);
+		? PARSING_GOP_HEADER_SEEN_CODE : PARSING_PICTURE_HEADER);
 
   // Compute this frame's timestamp by noting how many pictures we've seen
   // since the last GOP header:
@@ -272,7 +298,7 @@ unsigned MPEG1or2VideoStreamParser
   return curFrameSize();
 }
 
-unsigned MPEG1or2VideoStreamParser::parseGOPHeader() {
+unsigned MPEG1or2VideoStreamParser::parseGOPHeader(Boolean haveSeenStartCode) {
   // First check whether we should insert a previously-saved
   // 'video_sequence_header' here:
   if (needToUseSavedVSH()) return useSavedVSH();
@@ -280,8 +306,21 @@ unsigned MPEG1or2VideoStreamParser::parseGOPHeader() {
 #ifdef DEBUG
   fprintf(stderr, "parsing GOP header\n");
 #endif
-  // Note that we've already read the GROUP_START_CODE
-  save4Bytes(GROUP_START_CODE);
+  unsigned first4Bytes;
+  if (!haveSeenStartCode) {
+    while ((first4Bytes = test4Bytes()) != GROUP_START_CODE) {
+#ifdef DEBUG
+      fprintf(stderr, "ignoring non GOP start code: 0x%08x\n", first4Bytes);
+#endif
+      get1Byte(); setParseState(PARSING_GOP_HEADER);
+          // ensures we progress over bad data
+    }
+    first4Bytes = get4Bytes();
+  } else {
+    // We've already seen the GROUP_START_CODE
+    first4Bytes = GROUP_START_CODE;
+  }
+  save4Bytes(first4Bytes);
 
   // Next, extract the (25-bit) time code from the next 4 bytes:
   unsigned next4Bytes = get4Bytes();
@@ -377,11 +416,11 @@ unsigned MPEG1or2VideoStreamParser::parsePictureHeader() {
 }
 
 unsigned MPEG1or2VideoStreamParser::parseSlice() {
-#ifdef DEBUG_SLICE
-  fprintf(stderr, "parsing slice\n");
-#endif
   // Note that we've already read the slice_start_code:
   unsigned next4Bytes = PICTURE_START_CODE|fCurrentSliceNumber;
+#ifdef DEBUG_SLICE
+  fprintf(stderr, "parsing slice: 0x%08x\n", next4Bytes);
+#endif
 
   if (fSkippingCurrentPicture) {
     // Skip all bytes that we see, up until we reach a code of some sort:
@@ -412,7 +451,7 @@ unsigned MPEG1or2VideoStreamParser::parseSlice() {
       break;
     }
     case GROUP_START_CODE: {
-      setParseState(PARSING_GOP_HEADER);
+      setParseState(PARSING_GOP_HEADER_SEEN_CODE);
       break;
     }
     case PICTURE_START_CODE: {
