@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2003 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // RTP sink for MPEG-4 Elementary Stream video (RFC 3016)
 // Implementation
 
@@ -23,8 +23,9 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 MPEG4ESVideoRTPSink
 ::MPEG4ESVideoRTPSink(UsageEnvironment& env, Groupsock* RTPgs,
-		      unsigned char rtpPayloadFormat)
-  : VideoRTPSink(env, RTPgs, rtpPayloadFormat, 90000, "MP4V-ES"),
+		      unsigned char rtpPayloadFormat,
+		      u_int32_t rtpTimestampFrequency)
+  : VideoRTPSink(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency, "MP4V-ES"),
     fVOPIsPresent(False), fAuxSDPLine(NULL) {
 }
 
@@ -34,8 +35,10 @@ MPEG4ESVideoRTPSink::~MPEG4ESVideoRTPSink() {
 
 MPEG4ESVideoRTPSink*
 MPEG4ESVideoRTPSink::createNew(UsageEnvironment& env, Groupsock* RTPgs,
-			       unsigned char rtpPayloadFormat) {
-  return new MPEG4ESVideoRTPSink(env, RTPgs, rtpPayloadFormat);
+			       unsigned char rtpPayloadFormat,
+			       u_int32_t rtpTimestampFrequency) {
+  return new MPEG4ESVideoRTPSink(env, RTPgs, rtpPayloadFormat,
+				 rtpTimestampFrequency);
 }
 
 Boolean MPEG4ESVideoRTPSink::sourceIsCompatibleWithUs(MediaSource& source) {
@@ -51,12 +54,14 @@ void MPEG4ESVideoRTPSink
 			 unsigned numBytesInFrame,
 			 struct timeval frameTimestamp,
 			 unsigned numRemainingBytes) {
-  // Begin by inspecting the 4-byte code at the start of the frame:
-  if (numBytesInFrame < 4) return; // shouldn't happen
-  unsigned startCode = (frameStart[0]<<24) | (frameStart[1]<<16)
-    | (frameStart[2]<<8) | frameStart[3];
+  if (fragmentationOffset == 0) {
+    // Begin by inspecting the 4-byte code at the start of the frame:
+    if (numBytesInFrame < 4) return; // shouldn't happen
+    unsigned startCode = (frameStart[0]<<24) | (frameStart[1]<<16)
+      | (frameStart[2]<<8) | frameStart[3];
 
-  fVOPIsPresent = startCode == VOP_START_CODE;
+    fVOPIsPresent = startCode == VOP_START_CODE;
+  }
 
   // Set the RTP 'M' (marker) bit iff this frame ends a VOP
   // (and there are no fragments remaining).
@@ -79,8 +84,8 @@ Boolean MPEG4ESVideoRTPSink::allowFragmentationAfterStart() const {
 }
 
 Boolean MPEG4ESVideoRTPSink
-::frameCanAppearAfterPacketStart(unsigned char const* frameStart,
-				 unsigned numBytesInFrame) const {
+::frameCanAppearAfterPacketStart(unsigned char const* /*frameStart*/,
+				 unsigned /*numBytesInFrame*/) const {
   // Once we've packed a VOP into the packet, then no other
   // frame can be packed into it:
   return !fVOPIsPresent;
@@ -107,7 +112,7 @@ char const* MPEG4ESVideoRTPSink::auxSDPLine() {
   unsigned fmtpFmtSize = strlen(fmtpFmt)
     + 3 /* max char len */
     + 3 /* max char len */
-    + 2*configLength /* 2*, because each by prints as 2 chars */
+    + 2*configLength /* 2*, because each byte prints as 2 chars */
     + 2 /* trailing \r\n */;
   char* fmtp = new char[fmtpFmtSize];
   sprintf(fmtp, fmtpFmt, rtpPayloadType(), profile_level_id);
