@@ -14,12 +14,13 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // MPEG4-GENERIC ("audio", "video", or "application") RTP stream sources
 // Implementation
 
 #include "MPEG4GenericRTPSource.hh"
 #include "BitVector.hh"
+#include "MPEG4LATMAudioRTPSource.hh" // for parseGeneralConfigStr()
 
 ////////// MPEG4GenericBufferedPacket and MPEG4GenericBufferedPacketFactory
 
@@ -37,7 +38,7 @@ private:
 
 class MPEG4GenericBufferedPacketFactory: public BufferedPacketFactory {
 private: // redefined virtual functions
-  virtual BufferedPacket* createNew(MultiFramedRTPSource* ourSource);
+  virtual BufferedPacket* createNewPacket(MultiFramedRTPSource* ourSource);
 };
 
 
@@ -92,7 +93,8 @@ MPEG4GenericRTPSource
 
     fMode = strDup(mode);
     // Check for a "mode" that we don't yet support: //#####
-    if (mode == NULL || strcmp(mode, "aac-hbr") != 0) {
+    if (mode == NULL ||
+	(strcmp(mode, "aac-hbr") != 0 && strcmp(mode, "generic") != 0)) {
       envir() << "MPEG4GenericRTPSource Warning: Unknown or unsupported \"mode\": "
 	      << mode << "\n";
     }
@@ -120,7 +122,7 @@ Boolean MPEG4GenericRTPSource
   resultSpecialHeaderSize = 0;
   fNumAUHeaders = 0;
   fNextAUHeader = 0;
-  fAUHeaders = NULL;
+  delete[] fAUHeaders; fAUHeaders = NULL;
 
   if (fSizeLength > 0) {
     // The packet begins with a "AU Header Section".  Parse it, to
@@ -140,7 +142,7 @@ Boolean MPEG4GenericRTPSource
       fNumAUHeaders = 1 + bitsAvail/(fSizeLength + fIndexDeltaLength);
     }
     if (fNumAUHeaders > 0) {
-      delete[] fAUHeaders; fAUHeaders = new AUHeader[fNumAUHeaders];
+      fAUHeaders = new AUHeader[fNumAUHeaders];
       // Fill in each header:
       BitVector bv(&headerStart[2], 0, AU_headers_length);
       fAUHeaders[0].size = bv.getBits(fSizeLength);
@@ -174,12 +176,13 @@ MPEG4GenericBufferedPacket::~MPEG4GenericBufferedPacket() {
 }
 
 unsigned MPEG4GenericBufferedPacket
-::nextEnclosedFrameSize(unsigned char*& framePtr, unsigned dataSize) {
+::nextEnclosedFrameSize(unsigned char*& /*framePtr*/, unsigned dataSize) {
   // WE CURRENTLY DON'T IMPLEMENT INTERLEAVING.  FIX THIS! #####
   AUHeader* auHeader = fOurSource->fAUHeaders;
+  if (auHeader == NULL) return dataSize;
   unsigned numAUHeaders = fOurSource->fNumAUHeaders;
-  if (auHeader == NULL
-      || fOurSource->fNextAUHeader >= numAUHeaders) {
+
+  if (fOurSource->fNextAUHeader >= numAUHeaders) {
     fOurSource->envir() << "MPEG4GenericBufferedPacket::nextEnclosedFrameSize("
 			<< dataSize << "): data error ("
 			<< auHeader << "," << fOurSource->fNextAUHeader
@@ -192,6 +195,40 @@ unsigned MPEG4GenericBufferedPacket
 }
 
 BufferedPacket* MPEG4GenericBufferedPacketFactory
-::createNew(MultiFramedRTPSource* ourSource) {
+::createNewPacket(MultiFramedRTPSource* ourSource) {
   return new MPEG4GenericBufferedPacket((MPEG4GenericRTPSource*)ourSource);
 }
+
+
+////////// samplingFrequencyFromAudioSpecificConfig() implementation //////////
+
+static unsigned samplingFrequencyFromIndex[16] = {
+  96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050,
+  16000, 12000, 11025, 8000, 7350, 0, 0, 0
+};
+
+unsigned samplingFrequencyFromAudioSpecificConfig(char const* configStr) {
+  unsigned char* config = NULL;
+  unsigned result = 0; // if returned, indicates an error
+
+  do {
+    // Begin by parsing the config string:
+    unsigned configSize;
+    config = parseGeneralConfigStr(configStr, configSize);
+    if (config == NULL) break;
+
+    if (configSize < 2) break;
+    unsigned char samplingFrequencyIndex = ((config[0]&0x07)<<1) | (config[1]>>7);
+    if (samplingFrequencyIndex < 15) {
+      result = samplingFrequencyFromIndex[samplingFrequencyIndex];
+      break;
+    }
+    
+    // Index == 15 means that the actual frequency is next (24 bits):
+    if (configSize < 5) break;
+    result = ((config[1]&0x7F)<<17) | (config[2]<<9) | (config[3]<<1) | (config[4]>>7);
+  } while (0);
+
+  delete[] config;
+  return result;
+}
