@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // MPEG-4 audio, using LATM multiplexing
 // Implementation
 
@@ -23,14 +23,21 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 ////////// LATMBufferedPacket and LATMBufferedPacketFactory //////////
 
 class LATMBufferedPacket: public BufferedPacket {
+public:
+  LATMBufferedPacket(Boolean includeLATMDataLengthField);
+  virtual ~LATMBufferedPacket();
+
 private: // redefined virtual functions
   virtual unsigned nextEnclosedFrameSize(unsigned char*& framePtr,
                                  unsigned dataSize);
+
+private:
+  Boolean fIncludeLATMDataLengthField;
 };
 
 class LATMBufferedPacketFactory: public BufferedPacketFactory {
 private: // redefined virtual functions
-  virtual BufferedPacket* createNew(MultiFramedRTPSource* ourSource);
+  virtual BufferedPacket* createNewPacket(MultiFramedRTPSource* ourSource);
 };
 
 ///////// MPEG4LATMAudioRTPSource implementation ////////
@@ -49,12 +56,17 @@ MPEG4LATMAudioRTPSource
 			  unsigned rtpTimestampFrequency)
   : MultiFramedRTPSource(env, RTPgs,
 			 rtpPayloadFormat, rtpTimestampFrequency,
-			 new LATMBufferedPacketFactory) {
+			 new LATMBufferedPacketFactory),
+    fIncludeLATMDataLengthField(True) {
 }
 
 MPEG4LATMAudioRTPSource::~MPEG4LATMAudioRTPSource() {
 }
 
+void MPEG4LATMAudioRTPSource::omitLATMDataLengthField() {
+  fIncludeLATMDataLengthField = False;
+}
+
 Boolean MPEG4LATMAudioRTPSource
 ::processSpecialHeader(BufferedPacket* packet,
 		       unsigned& resultSpecialHeaderSize) {
@@ -76,6 +88,13 @@ char const* MPEG4LATMAudioRTPSource::MIMEtype() const {
 
 ////////// LATMBufferedPacket and LATMBufferedPacketFactory implementation
 
+LATMBufferedPacket::LATMBufferedPacket(Boolean includeLATMDataLengthField)
+  : fIncludeLATMDataLengthField(includeLATMDataLengthField) {
+}
+
+LATMBufferedPacket::~LATMBufferedPacket() {
+}
+
 unsigned LATMBufferedPacket
 ::nextEnclosedFrameSize(unsigned char*& framePtr, unsigned dataSize) {
   // Look at the LATM data length byte(s), to determine the size
@@ -87,15 +106,20 @@ unsigned LATMBufferedPacket
     if (framePtr[i] != 0xFF) break;
   }
   ++i;
-  framePtr += i;
-  dataSize -= i;
+  if (fIncludeLATMDataLengthField) {
+    resultFrameSize += i;
+  } else {
+    framePtr += i;
+    dataSize -= i;
+  }
 
   return (resultFrameSize <= dataSize) ? resultFrameSize : dataSize;
 }
 
 BufferedPacket* LATMBufferedPacketFactory
-::createNew(MultiFramedRTPSource* /*ourSource*/) {
-  return new LATMBufferedPacket;
+::createNewPacket(MultiFramedRTPSource* ourSource) {
+  MPEG4LATMAudioRTPSource* source = (MPEG4LATMAudioRTPSource*)ourSource;
+  return new LATMBufferedPacket(source->returnedFrameIncludesLATMDataLengthField());
 }
 
 
@@ -234,7 +258,7 @@ unsigned char* parseGeneralConfigStr(char const* configStr,
   } while (0);
 
   configSize = 0;
-  delete[] config; config = NULL;
+  delete[] config;
   return NULL;
 }
 
