@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand, from a MPEG-4 video file.
 // Implementation
@@ -26,14 +26,15 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 MPEG4VideoFileServerMediaSubsession*
 MPEG4VideoFileServerMediaSubsession::createNew(UsageEnvironment& env,
-					       char const* fileName) {
-  return new MPEG4VideoFileServerMediaSubsession(env, fileName);
+					       char const* fileName,
+					       Boolean reuseFirstSource) {
+  return new MPEG4VideoFileServerMediaSubsession(env, fileName, reuseFirstSource);
 }
 
 MPEG4VideoFileServerMediaSubsession
 ::MPEG4VideoFileServerMediaSubsession(UsageEnvironment& env,
-                                      char const* fileName)
-  : FileServerMediaSubsession(env, fileName),
+                                      char const* fileName, Boolean reuseFirstSource)
+  : FileServerMediaSubsession(env, fileName, reuseFirstSource),
     fDoneFlag(0) {
 }
 
@@ -44,8 +45,14 @@ MPEG4VideoFileServerMediaSubsession
 static void afterPlayingDummy(void* clientData) {
   MPEG4VideoFileServerMediaSubsession* subsess
     = (MPEG4VideoFileServerMediaSubsession*)clientData;
+  subsess->afterPlayingDummy1();
+}
+
+void MPEG4VideoFileServerMediaSubsession::afterPlayingDummy1() {
+  // Unschedule any pending 'checking' task:
+  envir().taskScheduler().unscheduleDelayedTask(nextTask());
   // Signal the event loop that we're done:
-  subsess->setDoneFlag();
+  setDoneFlag();
 }
 
 static void checkForAuxSDPLine(void* clientData) {
@@ -91,9 +98,10 @@ FramedSource* MPEG4VideoFileServerMediaSubsession
   estBitrate = 500; // kbps, estimate
 
   // Create the video source:
-  FramedSource* fileSource
+  ByteStreamFileSource* fileSource
     = ByteStreamFileSource::createNew(envir(), fFileName);
   if (fileSource == NULL) return NULL;
+  fFileSize = fileSource->fileSize();
 
   // Create a framer for the Video Elementary Stream:
   return MPEG4VideoStreamFramer::createNew(envir(), fileSource);
