@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up an MPEG-4 video elementary stream into
 //   frames for:
 // - Visual Object Sequence (VS) Header + Visual Object (VO) Header
@@ -47,6 +47,7 @@ public:
   virtual ~MPEG4VideoStreamParser();
 
 private: // redefined virtual functions:
+  virtual void flushInput();
   virtual unsigned parse();
 
 private:
@@ -79,6 +80,7 @@ private:
   unsigned fixed_vop_time_increment; // used if 'fixed_vop_rate' is set
   unsigned fSecondsSinceLastTimeCode, fTotalTicksSinceLastTimeCode, fPrevNewTotalTicks; 
   unsigned fPrevPictureCountDelta;
+  Boolean fJustSawTimeCode;
 };
 
 
@@ -98,20 +100,23 @@ unsigned char* MPEG4VideoStreamFramer
 }
 
 MPEG4VideoStreamFramer::MPEG4VideoStreamFramer(UsageEnvironment& env,
-					       FramedSource* inputSource)
+					       FramedSource* inputSource,
+					       Boolean createParser)
   : MPEGVideoStreamFramer(env, inputSource),
     fProfileAndLevelIndication(0),
     fConfigBytes(NULL), fNumConfigBytes(0),
     fNewConfigBytes(NULL), fNumNewConfigBytes(0) {
-  fParser
-    = new MPEG4VideoStreamParser(this, inputSource);
+  fParser = createParser
+    ? new MPEG4VideoStreamParser(this, inputSource)
+    : NULL;
 }
 
 MPEG4VideoStreamFramer::~MPEG4VideoStreamFramer() {
+  delete[] fConfigBytes; delete[] fNewConfigBytes;
 }
 
 void MPEG4VideoStreamFramer::startNewConfig() {
-  delete fNewConfigBytes; fNewConfigBytes = NULL;
+  delete[] fNewConfigBytes; fNewConfigBytes = NULL;
   fNumNewConfigBytes = 0;
 }
 
@@ -125,12 +130,12 @@ void MPEG4VideoStreamFramer
   memmove(configNew, fNewConfigBytes, fNumNewConfigBytes);
   memmove(&configNew[fNumNewConfigBytes], newConfigBytes, numNewBytes);
   
-  delete fNewConfigBytes; fNewConfigBytes = configNew;
+  delete[] fNewConfigBytes; fNewConfigBytes = configNew;
   fNumNewConfigBytes += numNewBytes;
 }
 
 void MPEG4VideoStreamFramer::completeNewConfig() {
-  delete fConfigBytes; fConfigBytes = fNewConfigBytes;
+  delete[] fConfigBytes; fConfigBytes = fNewConfigBytes;
   fNewConfigBytes = NULL;
   fNumConfigBytes = fNumNewConfigBytes;
   fNumNewConfigBytes = 0;
@@ -150,7 +155,7 @@ MPEG4VideoStreamParser
     vop_time_increment_resolution(0), fNumVTIRBits(0),
     fixed_vop_rate(0), fixed_vop_time_increment(0),
     fSecondsSinceLastTimeCode(0), fTotalTicksSinceLastTimeCode(0),
-    fPrevNewTotalTicks(0), fPrevPictureCountDelta(1) {
+    fPrevNewTotalTicks(0), fPrevPictureCountDelta(1), fJustSawTimeCode(False) {
 }
 
 MPEG4VideoStreamParser::~MPEG4VideoStreamParser() {
@@ -161,6 +166,19 @@ void MPEG4VideoStreamParser::setParseState(MPEGParseState parseState) {
   MPEGVideoStreamParser::setParseState();
 }
 
+void MPEG4VideoStreamParser::flushInput() {
+  fSecondsSinceLastTimeCode = 0;
+  fTotalTicksSinceLastTimeCode = 0;
+  fPrevNewTotalTicks = 0;
+  fPrevPictureCountDelta = 1;
+
+  StreamParser::flushInput();
+  if (fCurrentParseState != PARSING_VISUAL_OBJECT_SEQUENCE) {
+    setParseState(PARSING_VISUAL_OBJECT_SEQUENCE); // later, change to GOV or VOP? #####
+  }
+}
+
+
 unsigned MPEG4VideoStreamParser::parse() {
   try {
     switch (fCurrentParseState) {
@@ -211,14 +229,14 @@ unsigned MPEG4VideoStreamParser
   usingSource()->startNewConfig();
   u_int32_t first4Bytes;
   if (!haveSeenStartCode) {
-    while ((first4Bytes = get4Bytes())
-	   != VISUAL_OBJECT_SEQUENCE_START_CODE) {
+    while ((first4Bytes = test4Bytes()) != VISUAL_OBJECT_SEQUENCE_START_CODE) {
 #ifdef DEBUG
       fprintf(stderr, "ignoring non VS header: 0x%08x\n", first4Bytes);
 #endif
-      setParseState(PARSING_VISUAL_OBJECT_SEQUENCE);
+      get1Byte(); setParseState(PARSING_VISUAL_OBJECT_SEQUENCE);
           // ensures we progress over bad data
     }
+    first4Bytes = get4Bytes();
   } else {
     // We've already seen the start code
     first4Bytes = VISUAL_OBJECT_SEQUENCE_START_CODE;
@@ -464,6 +482,7 @@ unsigned MPEG4VideoStreamParser::parseGroupOfVideoObjectPlane() {
 #if defined(DEBUG) || defined(DEBUG_TIMESTAMPS)
   fprintf(stderr, "time_code: 0x%05x, hours %d, minutes %d, marker_bit %d, seconds %d\n", time_code, time_code_hours, time_code_minutes, marker_bit, time_code_seconds);
 #endif
+  fJustSawTimeCode = True;
 
   // Now, copy all bytes that we see, up until we reach a VOP_START_CODE:
   u_int32_t next4Bytes = get4Bytes();
@@ -563,11 +582,11 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
       fSecondsSinceLastTimeCode += modulo_time_base;
     } else {
       if (newTotalTicks < fPrevNewTotalTicks && vop_coding_type != 2/*B*/
-	  && modulo_time_base == 0 && vop_time_increment == 0) {
+	  && modulo_time_base == 0 && vop_time_increment == 0 && !fJustSawTimeCode) {
 	// This is another kind of buggy MPEG-4 video stream, in which
 	// "vop_time_increment" wraps around, but without
-	// "modulo_time_base" changing.  Overcome this by pretending that
-	// "vop_time_increment" really did wrap around:
+	// "modulo_time_base" changing (or just having had a new time code).
+	// Overcome this by pretending that "vop_time_increment" *did* wrap around:
 #ifdef DEBUG
 	fprintf(stderr, "Buggy MPEG-4 video stream: \"vop_time_increment\" wrapped around, but without \"modulo_time_base\" changing!\n");
 #endif
@@ -575,17 +594,18 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
 	newTotalTicks += vop_time_increment_resolution;
       }
       fPrevNewTotalTicks = newTotalTicks;
-      int pictureCountDelta = newTotalTicks - fTotalTicksSinceLastTimeCode;
-      if (pictureCountDelta <= 0) pictureCountDelta = fPrevPictureCountDelta;
-          // ensures that the picture count always increases
-      usingSource()->fPictureCount += pictureCountDelta;
-      fPrevPictureCountDelta = pictureCountDelta;
-      fTotalTicksSinceLastTimeCode = newTotalTicks;
       if (vop_coding_type != 2/*B*/) {
+	int pictureCountDelta = newTotalTicks - fTotalTicksSinceLastTimeCode;
+	if (pictureCountDelta <= 0) pictureCountDelta = fPrevPictureCountDelta;
+	    // ensures that the picture count is always increasing
+	usingSource()->fPictureCount += pictureCountDelta;
+	fPrevPictureCountDelta = pictureCountDelta;
+	fTotalTicksSinceLastTimeCode = newTotalTicks;
 	fSecondsSinceLastTimeCode += modulo_time_base;
       }
     }
   }
+  fJustSawTimeCode = False; // for next time
   
   // The next thing to parse depends on the code that we just saw,
   // but we are assumed to have ended the current picture: 
@@ -599,6 +619,10 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
     setParseState(PARSING_VISUAL_OBJECT_SEQUENCE_SEEN_CODE);
     break;
   }
+  case VISUAL_OBJECT_START_CODE: {
+    setParseState(PARSING_VISUAL_OBJECT);
+    break;
+  }
   case GROUP_VOP_START_CODE: {
     setParseState(PARSING_GROUP_OF_VIDEO_OBJECT_PLANE);
     break;
@@ -608,9 +632,20 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
     break;
   }
   default: {
-    usingSource()->envir() << "MPEG4VideoStreamParser::parseVideoObjectPlane(): Saw unexpected code "
-			   << (void*)next4Bytes << "\n";
-    setParseState(PARSING_VIDEO_OBJECT_PLANE); // the safest way to recover...
+    if (isVideoObjectStartCode(next4Bytes)) {
+      setParseState(PARSING_VIDEO_OBJECT_LAYER);
+    } else if (isVideoObjectLayerStartCode(next4Bytes)){
+      // copy all bytes that we see, up until we reach a VOP_START_CODE:
+      u_int32_t next4Bytes = get4Bytes();
+      while (next4Bytes != VOP_START_CODE) {
+	saveToNextCode(next4Bytes);
+      }
+      setParseState(PARSING_VIDEO_OBJECT_PLANE);
+    } else {
+      usingSource()->envir() << "MPEG4VideoStreamParser::parseVideoObjectPlane(): Saw unexpected code "
+			     << (void*)next4Bytes << "\n";
+      setParseState(PARSING_VIDEO_OBJECT_PLANE); // the safest way to recover...
+    }
     break;
   }
   }
