@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // An abstract parser for MPEG video streams
 // Implementation
 
@@ -34,10 +34,12 @@ MPEGVideoStreamParser::~MPEGVideoStreamParser() {
 void MPEGVideoStreamParser::restoreSavedParserState() {
   StreamParser::restoreSavedParserState();
   fTo = fSavedTo;
+  fNumTruncatedBytes = fSavedNumTruncatedBytes;
 }
 
 void MPEGVideoStreamParser::setParseState() {
   fSavedTo = fTo;
+  fSavedNumTruncatedBytes = fNumTruncatedBytes;
   saveParserState();
 }
 
@@ -45,4 +47,5 @@ void MPEGVideoStreamParser::registerReadInterest(unsigned char* to,
 						 unsigned maxSize) {
   fStartOfFrame = fTo = fSavedTo = to;
   fLimit = to + maxSize;
+  fNumTruncatedBytes = fSavedNumTruncatedBytes = 0;
 }
