@@ -14,15 +14,14 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A data structure that represents a session that consists of
 // potentially multiple (audio and/or video) sub-sessions
 // Implementation
 
 #include "liveMedia.hh"
 #ifdef SUPPORT_REAL_RTSP
-#include "../RealRTSP/include/RealRDTSource.hh"
-#include "../RealRTSP/include/RealSDP.hh"
+#include "../RealRTSP/include/RealRTSP.hh"
 #endif
 #include "GroupsockHelper.hh"
 #include <ctype.h>
@@ -62,7 +61,8 @@ Boolean MediaSession::lookupByName(UsageEnvironment& env,
 MediaSession::MediaSession(UsageEnvironment& env)
   : Medium(env),
     fSubsessionsHead(NULL), fSubsessionsTail(NULL),
-    fConnectionEndpointName(NULL), fMaxPlayEndTime(0.0) {
+    fConnectionEndpointName(NULL), fMaxPlayEndTime(0.0f),
+    fScale(1.0f), fMediaSessionType(NULL), fSessionName(NULL), fSessionDescription(NULL) {
 #ifdef SUPPORT_REAL_RTSP
   RealInitSDPAttributes(this);
 #endif
@@ -85,6 +85,9 @@ MediaSession::~MediaSession() {
   delete fSubsessionsHead;
   delete[] fCNAME;
   delete[] fConnectionEndpointName;
+  delete[] fMediaSessionType;
+  delete[] fSessionName;
+  delete[] fSessionDescription;
 #ifdef SUPPORT_REAL_RTSP
   RealReclaimSDPAttributes(this);
 #endif
@@ -110,8 +113,11 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
     if (sdpLine == NULL) break; // there are no m= lines at all 
 
     // Check for various special SDP lines that we understand:
+    if (parseSDPLine_s(sdpLine)) continue;
+    if (parseSDPLine_i(sdpLine)) continue;
     if (parseSDPLine_c(sdpLine)) continue;
     if (parseSDPAttribute_range(sdpLine)) continue;
+    if (parseSDPAttribute_type(sdpLine)) continue;
     if (parseSDPAttribute_source_filter(sdpLine)) continue;
 #ifdef SUPPORT_REAL_RTSP
     if (RealParseSDPAttributes(this, sdpLine)) continue;
@@ -126,6 +132,53 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
       return False;
     }
 
+    // Parse the line as "m=<medium_name> <client_portNum> RTP/AVP <fmt>"
+    // or "m=<medium_name> <client_portNum>/<num_ports> RTP/AVP <fmt>"
+    // (Should we be checking for >1 payload format number here?)#####
+    char* mediumName = strDupSize(sdpLine); // ensures we have enough space
+    char* protocolName = NULL;
+    unsigned payloadFormat;
+    if ((sscanf(sdpLine, "m=%s %hu RTP/AVP %u",
+		mediumName, &subsession->fClientPortNum, &payloadFormat) == 3 ||
+	 sscanf(sdpLine, "m=%s %hu/%*u RTP/AVP %u",
+		mediumName, &subsession->fClientPortNum, &payloadFormat) == 3)
+	&& payloadFormat <= 127) {
+      protocolName = "RTP";
+    } else if ((sscanf(sdpLine, "m=%s %hu UDP %u",
+		       mediumName, &subsession->fClientPortNum, &payloadFormat) == 3 ||
+		sscanf(sdpLine, "m=%s %hu udp %u",
+		       mediumName, &subsession->fClientPortNum, &payloadFormat) == 3 ||
+		sscanf(sdpLine, "m=%s %hu RAW/RAW/UDP %u",
+		       mediumName, &subsession->fClientPortNum, &payloadFormat) == 3)
+	       && payloadFormat <= 127) {
+      // This is a RAW UDP source
+      protocolName = "UDP";
+    } else {
+      // This "m=" line is bad; output an error message saying so:
+      char* sdpLineStr;
+      if (nextSDPLine == NULL) {
+	sdpLineStr = (char*)sdpLine;
+      } else {
+	sdpLineStr = strDup(sdpLine);
+	sdpLineStr[nextSDPLine-sdpLine] = '\0';
+      }
+      envir() << "Bad SDP \"m=\" line: " <<  sdpLineStr << "\n";
+      if (sdpLineStr != (char*)sdpLine) delete[] sdpLineStr;
+
+      delete[] mediumName;
+      delete subsession;
+
+      // Skip the following SDP lines, up until the next "m=":
+      while (1) {
+	sdpLine = nextSDPLine;
+	if (sdpLine == NULL) break; // we've reached the end
+	if (!parseSDPLine(sdpLine, nextSDPLine)) return False;
+
+	if (sdpLine[0] == 'm') break; // we've reached the next subsession
+      }
+      continue;
+    }
+
     // Insert this subsession at the end of the list:
     if (fSubsessionsTail == NULL) {
       fSubsessionsHead = fSubsessionsTail = subsession;
@@ -134,25 +187,6 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
       fSubsessionsTail = subsession;
     }
 
-    // Parse the line as "m=<medium_name> <client_portNum> RTP/AVP <fmt>"
-    // (Should we be checking for >1 payload format number here?)#####
-    char* mediumName = strDupSize(sdpLine); // ensures we have enough space
-    unsigned payloadFormat;
-    if (sscanf(sdpLine, "m=%s %hu RTP/AVP %u",
-	       mediumName, &subsession->fClientPortNum, &payloadFormat) != 3
-	|| payloadFormat > 127) {
-      char* sdpLineStr;
-      if (nextSDPLine == NULL) {
-	sdpLineStr = (char*)sdpLine;
-      } else {
-	sdpLineStr = strDup(sdpLine);
-	sdpLineStr[nextSDPLine-sdpLine] = '\0';
-      }
-      envir().setResultMsg("Bad SDP \"m=\" line: ", sdpLineStr);
-      if (sdpLineStr != (char*)sdpLine) delete[] sdpLineStr;
-      delete[] mediumName;
-      return False;
-    }
     subsession->serverPortNum = subsession->fClientPortNum; // by default
 
     char const* mStart = sdpLine;
@@ -160,6 +194,7 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
 
     subsession->fMediumName = strDup(mediumName);
     delete[] mediumName;
+    subsession->fProtocolName = strDup(protocolName);
     subsession->fRTPPayloadFormat = payloadFormat;
 
     // Process the following SDP lines, up until the next "m=":
@@ -177,9 +212,8 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
       if (subsession->parseSDPAttribute_range(sdpLine)) continue;
       if (subsession->parseSDPAttribute_fmtp(sdpLine)) continue;
       if (subsession->parseSDPAttribute_source_filter(sdpLine)) continue;
-      if (subsession->parseSDPAttribute_x_mct_slap(sdpLine)) continue;
       if (subsession->parseSDPAttribute_x_dimensions(sdpLine)) continue;
-      if (subsession->parseSDPAttribute_x_framerate(sdpLine)) continue;
+      if (subsession->parseSDPAttribute_framerate(sdpLine)) continue;
 #ifdef SUPPORT_REAL_RTSP
       if (RealParseSDPAttributes(subsession, sdpLine)) continue;
 #endif
@@ -257,6 +291,34 @@ static char* parseCLine(char const* sdpLine) {
   return resultStr;
 }
 
+Boolean MediaSession::parseSDPLine_s(char const* sdpLine) {
+  // Check for "s=<session name>" line
+  char* buffer = strDupSize(sdpLine);
+  Boolean parseSuccess = False;
+  
+  if (sscanf(sdpLine, "s=%[^\r\n]", buffer) == 1) {
+    delete[] fSessionName; fSessionName = strDup(buffer);
+    parseSuccess = True;
+  }
+  delete[] buffer;
+
+  return parseSuccess;
+}
+
+Boolean MediaSession::parseSDPLine_i(char const* sdpLine) {
+  // Check for "i=<session description>" line
+  char* buffer = strDupSize(sdpLine);
+  Boolean parseSuccess = False;
+  
+  if (sscanf(sdpLine, "i=%[^\r\n]", buffer) == 1) {
+    delete[] fSessionDescription; fSessionDescription = strDup(buffer);
+    parseSuccess = True;
+  }
+  delete[] buffer;
+
+  return parseSuccess;
+}  
+
 Boolean MediaSession::parseSDPLine_c(char const* sdpLine) {
   // Check for "c=IN IP4 <connection-endpoint>"
   // or "c=IN IP4 <connection-endpoint>/<ttl+numAddresses>"
@@ -271,6 +333,21 @@ Boolean MediaSession::parseSDPLine_c(char const* sdpLine) {
   return False;
 }
 
+Boolean MediaSession::parseSDPAttribute_type(char const* sdpLine) {
+  // Check for a "a=type:broadcast|meeting|moderated|test|H.332|recvonly" line:
+  Boolean parseSuccess = False;
+
+  char* buffer = strDupSize(sdpLine);
+  if (sscanf(sdpLine, "a=type: %[^ ]", buffer) == 1) {
+    delete[] fMediaSessionType;
+    fMediaSessionType = strDup(buffer);
+    parseSuccess = True;
+  }
+  delete[] buffer;
+
+  return parseSuccess;
+}
+
 static Boolean parseRangeAttribute(char const* sdpLine, float& endTime) {
   return sscanf(sdpLine, "a=range: npt = %*g - %g", &endTime) == 1;
 }
@@ -376,85 +453,36 @@ unsigned MediaSession::guessRTPTimestampFrequency(char const* mediumName,
   return 8000; // for "audio", and any other medium
 }
 
-static unsigned computeSeqNumStagger(unsigned staggerSeconds) {
-  // To compute the sequence number stagger, assume
-  // - one MP3 frame per packet
-  // - 1152 samples per MP3 frame (defined by MP3 standard)
-  // - a sampling frequency of 44100 Hz
-  // (Later, allow this to be parameterized)#####
-  unsigned const samplesPerFrame = 1152;
-  unsigned samplesPerSecond = 44100;
-  double secondsPerFrame = (double)samplesPerFrame/samplesPerSecond;
-  return (unsigned)(staggerSeconds/secondsPerFrame);
-}
-
 Boolean MediaSession
 ::initiateByMediaType(char const* mimeType,
 		      MediaSubsession*& resultSubsession,
-		      PrioritizedRTPStreamSelector*& resultMultiSource,
-		      int& resultMultiSourceSessionId,
 		      int useSpecialRTPoffset) {
   // Look through this session's subsessions for media that match "mimeType"
   resultSubsession = NULL;
-  resultMultiSource = NULL;
-  resultMultiSourceSessionId = 0;
-  unsigned maxStaggerSeconds = 0;
   MediaSubsessionIterator iter(*this);
   MediaSubsession* subsession;
   while ((subsession = iter.next()) != NULL) {
-    if (resultMultiSourceSessionId != 0
-	&& subsession->mctSLAPSessionId() != resultMultiSourceSessionId) {
-      // We're using a multi-source SLAP session, but this subsession
-      // isn't part of it
-      continue;
+    Boolean wasAlreadyInitiated = subsession->readSource() != NULL;
+    if (!wasAlreadyInitiated) {
+      // Try to create a source for this subsession:
+      if (!subsession->initiate(useSpecialRTPoffset)) return False;
     }
 
-    // Try to create a source for this subsession:
-    if (!subsession->initiate(useSpecialRTPoffset)) return False;
-
     // Make sure the source's MIME type is one that we handle:
     if (strcmp(subsession->readSource()->MIMEtype(), mimeType) != 0) {
-      subsession->deInitiate();
+      if (!wasAlreadyInitiated) subsession->deInitiate();
       continue;
     }
 
-    if (subsession->mctSLAPSessionId() == 0) {
-      // Normal case: a single session
-      resultSubsession = subsession;
-      break; // use this
-    } else {
-      // Special case: a multi-source SLAP session
-      resultMultiSourceSessionId = subsession->mctSLAPSessionId();
-      unsigned subsessionStaggerSeconds = subsession->mctSLAPStagger();
-      if (subsessionStaggerSeconds > maxStaggerSeconds) {
-	maxStaggerSeconds = subsessionStaggerSeconds;
-      }
-    }
+    resultSubsession = subsession;
+    break; // use this
   }
 
-  if (resultSubsession == NULL && resultMultiSourceSessionId == 0) {
+  if (resultSubsession == NULL) {
     envir().setResultMsg("Session has no usable media subsession");
     return False;
   }
 
-  if (resultMultiSourceSessionId != 0) {
-    // We have a multi-source MCT SLAP session; create a selector for it:
-    unsigned seqNumStagger = computeSeqNumStagger(maxStaggerSeconds);
-    resultMultiSource
-      = PrioritizedRTPStreamSelector::createNew(envir(), seqNumStagger);
-    if (resultMultiSource == NULL) return False;
-    // Note: each subsession has its own RTCP instance; we don't return them
-
-    // Then run through the subsessions again, adding each of the sources:
-    iter.reset();
-    while ((subsession = iter.next()) != NULL) {
-      if (subsession->mctSLAPSessionId() == resultMultiSourceSessionId) {
-	resultMultiSource->addInputRTPStream(subsession->rtpSource(),
-					     subsession->rtcpInstance());
-      }
-    }
-  }
-
   return True;
 }
 
@@ -488,7 +516,7 @@ MediaSubsession::MediaSubsession(MediaSession& parent)
     fParent(parent), fNext(NULL),
     fConnectionEndpointName(NULL),
     fClientPortNum(0), fRTPPayloadFormat(0xFF),
-    fSavedSDPLines(NULL), fMediumName(NULL), fCodecName(NULL),
+    fSavedSDPLines(NULL), fMediumName(NULL), fCodecName(NULL), fProtocolName(NULL),
     fRTPTimestampFrequency(0), fControlPath(NULL),
     fSourceFilterAddr(parent.sourceFilterAddr()),
     fAuxiliarydatasizelength(0), fConstantduration(0), fConstantsize(0),
@@ -497,10 +525,9 @@ MediaSubsession::MediaSubsession(MediaSession& parent)
     fObjecttype(0), fOctetalign(0), fProfile_level_id(0), fRobustsorting(0),
     fSizelength(0), fStreamstateindication(0), fStreamtype(0),
     fCpresent(False), fRandomaccessindication(False),
-    fConfig(NULL), fMode(NULL),
+    fConfig(NULL), fMode(NULL), fSpropParameterSets(NULL),
     fPlayEndTime(0.0),
-    fMCT_SLAP_SessionId(0), fMCT_SLAP_Stagger(0),
-    fVideoWidth(0), fVideoHeight(0), fVideoFPS(0), fNumChannels(1),
+    fVideoWidth(0), fVideoHeight(0), fVideoFPS(0), fNumChannels(1), fScale(1.0f),
     fRTPSocket(NULL), fRTCPSocket(NULL),
     fRTPSource(NULL), fRTCPInstance(NULL), fReadSource(NULL) {
 #ifdef SUPPORT_REAL_RTSP
@@ -511,9 +538,9 @@ MediaSubsession::MediaSubsession(MediaSession& parent)
 MediaSubsession::~MediaSubsession() {
   deInitiate();
 
-  delete[] fConnectionEndpointName;
-  delete[] fSavedSDPLines; delete[] fMediumName; delete[] fCodecName;
-  delete[] fControlPath; delete[] fConfig; delete[] fMode;
+  delete[] fConnectionEndpointName; delete[] fSavedSDPLines;
+  delete[] fMediumName; delete[] fCodecName; delete[] fProtocolName;
+  delete[] fControlPath; delete[] fConfig; delete[] fMode; delete[] fSpropParameterSets;
 
   delete fNext;
 #ifdef SUPPORT_REAL_RTSP
@@ -596,158 +623,186 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
       break;
     }
 
-    // Check "fCodecName" against the set of codecs that we support,
-    // and create our RTP source accordingly
-    // (Later make this code more efficient, as this set grows #####)
-    // (Also, add more fmts that can be implemented by SimpleRTPSource#####)
-    Boolean createSimpleRTPSource = False;
-    if (strcmp(fCodecName, "QCELP") == 0) { // QCELP audio
-      fReadSource =
-	QCELPAudioRTPSource::createNew(env(), fRTPSocket, fRTPSource,
-				       fRTPPayloadFormat,
-				       fRTPTimestampFrequency);
-          // Note that fReadSource will differ from fRTPSource in this case
-    } else if (strcmp(fCodecName, "AMR") == 0) { // AMR audio (narrowband)
-      fReadSource =
-	AMRAudioRTPSource::createNew(env(), fRTPSocket, fRTPSource,
-				     fRTPPayloadFormat, 0 /*isWideband*/,
-				     fNumChannels, fOctetalign, fInterleaving,
-				     fRobustsorting, fCRC);
-          // Note that fReadSource will differ from fRTPSource in this case
-    } else if (strcmp(fCodecName, "AMR-WB") == 0) { // AMR audio (wideband)
-      fReadSource =
-	AMRAudioRTPSource::createNew(env(), fRTPSocket, fRTPSource,
-				     fRTPPayloadFormat, 1 /*isWideband*/,
-				     fNumChannels, fOctetalign, fInterleaving,
-				     fRobustsorting, fCRC);
-          // Note that fReadSource will differ from fRTPSource in this case
-    } else if (strcmp(fCodecName, "MPA") == 0) { // MPEG-1 or 2 audio
-      fReadSource = fRTPSource
-	= MPEG1or2AudioRTPSource::createNew(env(), fRTPSocket,
-					fRTPPayloadFormat,
-					fRTPTimestampFrequency);
-    } else if (strcmp(fCodecName, "MPA-ROBUST") == 0) { // robust MP3 audio
-      fRTPSource
-	= MP3ADURTPSource::createNew(env(), fRTPSocket, fRTPPayloadFormat,
-				     fRTPTimestampFrequency);
-      if (fRTPSource == NULL) break;
-
-      // Add a filter that deinterleaves the ADUs after depacketizing them:
-      MP3ADUdeinterleaver* deinterleaver
-	= MP3ADUdeinterleaver::createNew(env(), fRTPSource);
-      if (deinterleaver == NULL) break;
-
-      // Add another filter that converts these ADUs to MP3 frames:
-      fReadSource = MP3FromADUSource::createNew(env(), deinterleaver);
-    } else if (strcmp(fCodecName, "X-MP3-DRAFT-00") == 0) {
-      // a non-standard variant of "MPA-ROBUST" used by RealNetworks
-      // (one 'ADU'ized MP3 frame per packet; no headers)
-      fRTPSource
-	= SimpleRTPSource::createNew(env(), fRTPSocket, fRTPPayloadFormat,
-				     90000, "audio/mpa-robust" /*hack*/);
-      if (fRTPSource == NULL) break;
-
-      // Add a filter that converts these ADUs to MP3 frames:
-      fReadSource = MP3FromADUSource::createNew(env(), fRTPSource,
-						False /*no ADU header*/);
-    } else if (strcmp(fCodecName, "MP4A-LATM") == 0) { // MPEG-4 LATM audio
-      fReadSource = fRTPSource
-	= MPEG4LATMAudioRTPSource::createNew(env(), fRTPSocket,
-					     fRTPPayloadFormat,
-					     fRTPTimestampFrequency);
-    } else if (strcmp(fCodecName, "AC3") == 0) { // AC3 audio
-      fReadSource = fRTPSource
-	= AC3AudioRTPSource::createNew(env(), fRTPSocket,
-				       fRTPPayloadFormat,
-				       fRTPTimestampFrequency);
-    } else if (strcmp(fCodecName, "MP4V-ES") == 0) { // MPEG-4 Elem Str vid
-      fReadSource = fRTPSource
-	= MPEG4ESVideoRTPSource::createNew(env(), fRTPSocket,
-					   fRTPPayloadFormat,
-					   fRTPTimestampFrequency);
-    } else if (strcmp(fCodecName, "MPEG4-GENERIC") == 0) {
-      fReadSource = fRTPSource
-	= MPEG4GenericRTPSource::createNew(env(), fRTPSocket,
-					   fRTPPayloadFormat,
-					   fRTPTimestampFrequency,
-					   fMediumName, fMode,
-					   fSizelength, fIndexlength,
-					   fIndexdeltalength);
-    } else if (strcmp(fCodecName, "MPV") == 0) { // MPEG-1 or 2 video
-      fReadSource = fRTPSource
-	= MPEG1or2VideoRTPSource::createNew(env(), fRTPSocket,
-					fRTPPayloadFormat,
-					fRTPTimestampFrequency);
-    } else if (strcmp(fCodecName, "H261") == 0) { // H.261
-      fReadSource = fRTPSource
-	= H261VideoRTPSource::createNew(env(), fRTPSocket,
-					fRTPPayloadFormat,
-					fRTPTimestampFrequency);
-    } else if (strcmp(fCodecName, "H263-1998") == 0 ||
-	       strcmp(fCodecName, "H263-2000") == 0) { // H.263+
-      fReadSource = fRTPSource
-	= H263plusVideoRTPSource::createNew(env(), fRTPSocket,
-					    fRTPPayloadFormat,
-					    fRTPTimestampFrequency);
-    } else if (strcmp(fCodecName, "JPEG") == 0) { // motion JPEG
-      fReadSource = fRTPSource
-	= JPEGVideoRTPSource::createNew(env(), fRTPSocket,
-					fRTPPayloadFormat,
-					fRTPTimestampFrequency);
-    } else if (strcmp(fCodecName, "X-QT") == 0
-	       || strcmp(fCodecName, "X-QUICKTIME") == 0) {
-      // Generic QuickTime streams, as defined in
-      // <http://developer.apple.com/quicktime/icefloe/dispatch026.html>
-      char* mimeType
-	= new char[strlen(mediumName()) + strlen(codecName()) + 2] ;
-      sprintf(mimeType, "%s/%s", mediumName(), codecName());
-      fReadSource = fRTPSource
-	= QuickTimeGenericRTPSource::createNew(env(), fRTPSocket,
-					       fRTPPayloadFormat,
-					       fRTPTimestampFrequency,
-					       mimeType);
-      delete[] mimeType;
-    } else if (strcmp(fCodecName, "X-MCT-TEXT") == 0) {
-      // A UDP-packetized text stream (*not* a RTP stream)
+    // Check "fProtocolName"
+    if (strcmp(fProtocolName, "UDP") == 0) {
+      // A UDP-packetized stream (*not* a RTP stream)
       fReadSource = BasicUDPSource::createNew(env(), fRTPSocket);
       fRTPSource = NULL; // Note!
-#ifdef SUPPORT_REAL_RTSP
-    } else if (strcmp(fCodecName, "X-PN-REALAUDIO") == 0 ||
-	       strcmp(fCodecName, "X-PN-REALVIDEO") == 0) {
-      // A RealNetworks 'RDT' stream (*not* a RTP stream)
-      fReadSource = RealRDTSource::createNew(env());
-      fRTPSource = NULL; // Note!
-#endif
-    } else if (  strcmp(fCodecName, "PCMU") == 0 // PCM u-law audio
-	       || strcmp(fCodecName, "GSM") == 0 // GSM audio
-	       || strcmp(fCodecName, "PCMA") == 0 // PCM a-law audio
-	       || strcmp(fCodecName, "L16") == 0 // 16-bit linear audio
-	       || strcmp(fCodecName, "MP1S") == 0 // MPEG-1 System Stream
-	       || strcmp(fCodecName, "MP2T") == 0 // MPEG-2 Transport Str
-	       || strcmp(fCodecName, "MP2P") == 0 // MPEG-2 Program Stream
-	       || strcmp(fCodecName, "L8") == 0 // 8-bit linear audio
-	       || strcmp(fCodecName, "SPEEX") == 0 // SPEEX audio
-	      ) {
-      createSimpleRTPSource = True;
-      useSpecialRTPoffset = 0;
-    } else if (useSpecialRTPoffset >= 0) {
-      // We don't know this RTP payload format, but try to receive
-      // it using a 'SimpleRTPSource' with the specified header offset:
-      createSimpleRTPSource = True;
-    } else {
-      env().setResultMsg("RTP payload format unknown or not supported");
-      break;
-    }
 
-    if (createSimpleRTPSource) {
-      char* mimeType
-	= new char[strlen(mediumName()) + strlen(codecName()) + 2] ;
-      sprintf(mimeType, "%s/%s", mediumName(), codecName());
-      fReadSource = fRTPSource
-	= SimpleRTPSource::createNew(env(), fRTPSocket, fRTPPayloadFormat,
-				     fRTPTimestampFrequency, mimeType,
-				     (unsigned)useSpecialRTPoffset);
-      delete[] mimeType;
+      if (strcmp(fCodecName, "MP2T") == 0) { // MPEG-2 Transport Stream
+	fReadSource = MPEG2TransportStreamFramer::createNew(env(), fReadSource);
+	    // this sets "durationInMicroseconds" correctly, based on the PCR values
+      }
+    } else {
+      // Check "fCodecName" against the set of codecs that we support,
+      // and create our RTP source accordingly
+      // (Later make this code more efficient, as this set grows #####)
+      // (Also, add more fmts that can be implemented by SimpleRTPSource#####)
+      Boolean createSimpleRTPSource = False;
+      Boolean doNormalMBitRule = False; // used if "createSimpleRTPSource"
+      if (strcmp(fCodecName, "QCELP") == 0) { // QCELP audio
+	fReadSource =
+	  QCELPAudioRTPSource::createNew(env(), fRTPSocket, fRTPSource,
+					 fRTPPayloadFormat,
+					 fRTPTimestampFrequency);
+	// Note that fReadSource will differ from fRTPSource in this case
+      } else if (strcmp(fCodecName, "AMR") == 0) { // AMR audio (narrowband)
+	fReadSource =
+	  AMRAudioRTPSource::createNew(env(), fRTPSocket, fRTPSource,
+				       fRTPPayloadFormat, 0 /*isWideband*/,
+				       fNumChannels, fOctetalign, fInterleaving,
+				       fRobustsorting, fCRC);
+	// Note that fReadSource will differ from fRTPSource in this case
+      } else if (strcmp(fCodecName, "AMR-WB") == 0) { // AMR audio (wideband)
+	fReadSource =
+	  AMRAudioRTPSource::createNew(env(), fRTPSocket, fRTPSource,
+				       fRTPPayloadFormat, 1 /*isWideband*/,
+				       fNumChannels, fOctetalign, fInterleaving,
+				       fRobustsorting, fCRC);
+	// Note that fReadSource will differ from fRTPSource in this case
+      } else if (strcmp(fCodecName, "MPA") == 0) { // MPEG-1 or 2 audio
+	fReadSource = fRTPSource
+	  = MPEG1or2AudioRTPSource::createNew(env(), fRTPSocket,
+					      fRTPPayloadFormat,
+					      fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "MPA-ROBUST") == 0) { // robust MP3 audio
+	fRTPSource
+	  = MP3ADURTPSource::createNew(env(), fRTPSocket, fRTPPayloadFormat,
+				       fRTPTimestampFrequency);
+	if (fRTPSource == NULL) break;
+	
+	// Add a filter that deinterleaves the ADUs after depacketizing them:
+	MP3ADUdeinterleaver* deinterleaver
+	  = MP3ADUdeinterleaver::createNew(env(), fRTPSource);
+	if (deinterleaver == NULL) break;
+	
+	// Add another filter that converts these ADUs to MP3 frames:
+	fReadSource = MP3FromADUSource::createNew(env(), deinterleaver);
+      } else if (strcmp(fCodecName, "X-MP3-DRAFT-00") == 0) {
+	// a non-standard variant of "MPA-ROBUST" used by RealNetworks
+	// (one 'ADU'ized MP3 frame per packet; no headers)
+	fRTPSource
+	  = SimpleRTPSource::createNew(env(), fRTPSocket, fRTPPayloadFormat,
+				       fRTPTimestampFrequency,
+				       "audio/MPA-ROBUST" /*hack*/);
+	if (fRTPSource == NULL) break;
+	
+	// Add a filter that converts these ADUs to MP3 frames:
+	fReadSource = MP3FromADUSource::createNew(env(), fRTPSource,
+						  False /*no ADU header*/);
+      } else if (strcmp(fCodecName, "MP4A-LATM") == 0) { // MPEG-4 LATM audio
+	fReadSource = fRTPSource
+	  = MPEG4LATMAudioRTPSource::createNew(env(), fRTPSocket,
+					       fRTPPayloadFormat,
+					       fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "AC3") == 0) { // AC3 audio
+	fReadSource = fRTPSource
+	  = AC3AudioRTPSource::createNew(env(), fRTPSocket,
+					 fRTPPayloadFormat,
+					 fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "MP4V-ES") == 0) { // MPEG-4 Elem Str vid
+	fReadSource = fRTPSource
+	  = MPEG4ESVideoRTPSource::createNew(env(), fRTPSocket,
+					     fRTPPayloadFormat,
+					     fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "MPEG4-GENERIC") == 0) {
+	fReadSource = fRTPSource
+	  = MPEG4GenericRTPSource::createNew(env(), fRTPSocket,
+					     fRTPPayloadFormat,
+					     fRTPTimestampFrequency,
+					     fMediumName, fMode,
+					     fSizelength, fIndexlength,
+					     fIndexdeltalength);
+      } else if (strcmp(fCodecName, "MPV") == 0) { // MPEG-1 or 2 video
+	fReadSource = fRTPSource
+	  = MPEG1or2VideoRTPSource::createNew(env(), fRTPSocket,
+					      fRTPPayloadFormat,
+					      fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "MP2T") == 0) { // MPEG-2 Transport Stream
+	fRTPSource = SimpleRTPSource::createNew(env(), fRTPSocket, fRTPPayloadFormat,
+						fRTPTimestampFrequency, "video/MP2T",
+						0, False);
+	fReadSource = MPEG2TransportStreamFramer::createNew(env(), fRTPSource);
+	    // this sets "durationInMicroseconds" correctly, based on the PCR values
+      } else if (strcmp(fCodecName, "H261") == 0) { // H.261
+	fReadSource = fRTPSource
+	  = H261VideoRTPSource::createNew(env(), fRTPSocket,
+					  fRTPPayloadFormat,
+					  fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "H263-1998") == 0 ||
+		 strcmp(fCodecName, "H263-2000") == 0) { // H.263+
+	fReadSource = fRTPSource
+	  = H263plusVideoRTPSource::createNew(env(), fRTPSocket,
+					      fRTPPayloadFormat,
+					      fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "H264") == 0) {
+	fReadSource = fRTPSource
+	  = H264VideoRTPSource::createNew(env(), fRTPSocket,
+					  fRTPPayloadFormat,
+					  fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "JPEG") == 0) { // motion JPEG
+	fReadSource = fRTPSource
+	  = JPEGVideoRTPSource::createNew(env(), fRTPSocket,
+					  fRTPPayloadFormat,
+					  fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "X-QT") == 0
+		 || strcmp(fCodecName, "X-QUICKTIME") == 0) {
+	// Generic QuickTime streams, as defined in
+	// <http://developer.apple.com/quicktime/icefloe/dispatch026.html>
+	char* mimeType
+	  = new char[strlen(mediumName()) + strlen(codecName()) + 2] ;
+	sprintf(mimeType, "%s/%s", mediumName(), codecName());
+	fReadSource = fRTPSource
+	  = QuickTimeGenericRTPSource::createNew(env(), fRTPSocket,
+						 fRTPPayloadFormat,
+						 fRTPTimestampFrequency,
+						 mimeType);
+	delete[] mimeType;
+#ifdef SUPPORT_REAL_RTSP
+      } else if (strcmp(fCodecName, "X-PN-REALAUDIO") == 0 ||
+		 strcmp(fCodecName, "X-PN-MULTIRATE-REALAUDIO-LIVE") == 0 ||
+		 strcmp(fCodecName, "X-PN-REALVIDEO") == 0 ||
+		 strcmp(fCodecName, "X-PN-MULTIRATE-REALVIDEO-LIVE") == 0) {
+	// A RealNetworks 'RDT' stream (*not* a RTP stream)
+	fReadSource = RealRDTSource::createNew(env());
+	fRTPSource = NULL; // Note!
+	parentSession().isRealNetworksRDT = True;
+#endif
+      } else if (  strcmp(fCodecName, "PCMU") == 0 // PCM u-law audio
+		   || strcmp(fCodecName, "GSM") == 0 // GSM audio
+		   || strcmp(fCodecName, "PCMA") == 0 // PCM a-law audio
+		   || strcmp(fCodecName, "L16") == 0 // 16-bit linear audio
+		   || strcmp(fCodecName, "MP1S") == 0 // MPEG-1 System Stream
+		   || strcmp(fCodecName, "MP2P") == 0 // MPEG-2 Program Stream
+		   || strcmp(fCodecName, "L8") == 0 // 8-bit linear audio
+		   || strcmp(fCodecName, "G726-16") == 0 // G.726, 16 kbps
+		   || strcmp(fCodecName, "G726-24") == 0 // G.726, 24 kbps
+		   || strcmp(fCodecName, "G726-32") == 0 // G.726, 32 kbps
+		   || strcmp(fCodecName, "G726-40") == 0 // G.726, 40 kbps
+		   || strcmp(fCodecName, "SPEEX") == 0 // SPEEX audio
+		   ) {
+	createSimpleRTPSource = True;
+	useSpecialRTPoffset = 0;
+      } else if (useSpecialRTPoffset >= 0) {
+	// We don't know this RTP payload format, but try to receive
+	// it using a 'SimpleRTPSource' with the specified header offset:
+	createSimpleRTPSource = True;
+      } else {
+	env().setResultMsg("RTP payload format unknown or not supported");
+	break;
+      }
+      
+      if (createSimpleRTPSource) {
+	char* mimeType
+	  = new char[strlen(mediumName()) + strlen(codecName()) + 2] ;
+	sprintf(mimeType, "%s/%s", mediumName(), codecName());
+	fReadSource = fRTPSource
+	  = SimpleRTPSource::createNew(env(), fRTPSocket, fRTPPayloadFormat,
+				       fRTPTimestampFrequency, mimeType,
+				       (unsigned)useSpecialRTPoffset,
+				       doNormalMBitRule);
+	delete[] mimeType;
+      }
     }
 
     if (fReadSource == NULL) {
@@ -970,7 +1025,8 @@ Boolean MediaSubsession::parseSDPAttribute_fmtp(char const* sdpLine) {
 	fObjecttype = u;
       } else if (sscanf(line, " octet-align = %u", &u) == 1) {
 	fOctetalign = u;
-      } else if (sscanf(line, " profile-level-id = %u", &u) == 1) {
+      } else if (sscanf(line, " profile-level-id = %x", &u) == 1) {
+	// Note that the "profile-level-id" parameter is assumed to be hexadecimal
 	fProfile_level_id = u;
       } else if (sscanf(line, " robust-sorting = %u", &u) == 1) {
 	fRobustsorting = u;
@@ -988,6 +1044,25 @@ Boolean MediaSubsession::parseSDPAttribute_fmtp(char const* sdpLine) {
 	delete[] fConfig; fConfig = strDup(valueStr);
       } else if (sscanf(line, " mode = %[^; \t\r\n]", valueStr) == 1) {
 	delete[] fMode; fMode = strDup(valueStr);
+      } else if (sscanf(sdpLine, " sprop-parameter-sets = %[^; \t\r\n]", valueStr) == 1) {
+	// Note: We used "sdpLine" here, because the value is case-sensitive.
+	delete[] fSpropParameterSets; fSpropParameterSets = strDup(valueStr); 
+      } else {
+	// Some of the above parameters are Boolean.  Check whether the parameter
+	// names appear alone, without a "= 1" at the end:
+	if (sscanf(line, " %[^; \t\r\n]", valueStr) == 1) {
+	  if (strcmp(valueStr, "octet-align") == 0) {
+	    fOctetalign = 1;
+	  } else if (strcmp(valueStr, "cpresent") == 0) {
+            fCpresent = True;
+	  } else if (strcmp(valueStr, "crc") == 0) {
+	    fCRC = 1;
+	  } else if (strcmp(valueStr, "robust-sorting") == 0) {
+	    fRobustsorting = 1;
+	  } else if (strcmp(valueStr, "randomaccessindication") == 0) {
+	    fRandomaccessindication = True;
+	  }
+	}
       }
       delete[] valueStr;
 
@@ -995,6 +1070,11 @@ Boolean MediaSubsession::parseSDPAttribute_fmtp(char const* sdpLine) {
       while (*line != '\0' && *line != '\r' && *line != '\n'
 	     && *line != ';') ++line;
       while (*line == ';') ++line;
+
+      // Do the same with sdpLine; needed for finding case sensitive values:
+      while (*sdpLine != '\0' && *sdpLine != '\r' && *sdpLine != '\n'
+	     && *sdpLine != ';') ++sdpLine;
+      while (*sdpLine == ';') ++sdpLine;
     }
     delete[] lineCopy;
     return True;
@@ -1008,22 +1088,6 @@ Boolean MediaSubsession
   return parseSourceFilterAttribute(sdpLine, fSourceFilterAddr);
 }
 
-Boolean MediaSubsession::parseSDPAttribute_x_mct_slap(char const* sdpLine) {
-  // Check for a "a=x-mct-slap:<slap-session-id> <stagger>" line:
-  Boolean parseSuccess = False;
-
-  int slapSessionId;
-  int slapStagger;
-  if (sscanf(sdpLine, "a=x-mct-slap: %d %d",
-	     &slapSessionId, &slapStagger) == 2) {
-    parseSuccess = True;
-    fMCT_SLAP_SessionId = slapSessionId;
-    fMCT_SLAP_Stagger = (unsigned)slapStagger;
-  }
-
-  return parseSuccess;
-}
-
 Boolean MediaSubsession::parseSDPAttribute_x_dimensions(char const* sdpLine) {
   // Check for a "a=x-dimensions:<width>,<height>" line:
   Boolean parseSuccess = False;
@@ -1038,12 +1102,16 @@ Boolean MediaSubsession::parseSDPAttribute_x_dimensions(char const* sdpLine) {
   return parseSuccess;
 }
 
-Boolean MediaSubsession::parseSDPAttribute_x_framerate(char const* sdpLine) {
-  // Check for a "a=x-framerate:<fps>" line:
+Boolean MediaSubsession::parseSDPAttribute_framerate(char const* sdpLine) {
+  // Check for a "a=framerate: <fps>" or "a=x-framerate: <fps>" line:
   Boolean parseSuccess = False;
 
+  float frate;
   int rate;
-  if (sscanf(sdpLine, "a=x-framerate:%d", &rate) == 1) {
+  if (sscanf(sdpLine, "a=framerate: %f", &frate) == 1) {
+    parseSuccess = True;
+    fVideoFPS = (unsigned)frate;
+  } else if (sscanf(sdpLine, "a=x-framerate: %d", &rate) == 1) {
     parseSuccess = True;
     fVideoFPS = (unsigned)rate;
   }
