@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // Media Sinks
 // Implementation
 
@@ -104,13 +104,15 @@ Boolean MediaSink::isRTPSink() const {
 
 ////////// OutPacketBuffer //////////
 
-unsigned OutPacketBuffer::numPacketsLimit = 20; // by default
+unsigned OutPacketBuffer::maxSize = 60000; // by default
 
 OutPacketBuffer::OutPacketBuffer(unsigned preferredPacketSize,
 				 unsigned maxPacketSize)
   : fPreferred(preferredPacketSize), fMax(maxPacketSize),
-    fLimit(numPacketsLimit*maxPacketSize), fBuf(new unsigned char[fLimit]),
     fOverflowDataSize(0) {
+  unsigned maxNumPackets = (maxSize + (maxPacketSize-1))/maxPacketSize;
+  fLimit = maxNumPackets*maxPacketSize;
+  fBuf = new unsigned char[fLimit];
   resetPacketStart();
   resetOffset();
   resetOverflowData();
