@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2002 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // Media Sources
 // Implementation
 
@@ -34,7 +34,7 @@ Boolean MediaSource::isSource() const {
 }
 
 char const* MediaSource::MIMEtype() const {
-  return "application/octet-stream"; // default type
+  return "application/OCTET-STREAM"; // default type
 }
 
 Boolean MediaSource::isFramedSource() const {
@@ -49,6 +49,9 @@ Boolean MediaSource::isMPEG1or2VideoStreamFramer() const {
 Boolean MediaSource::isMPEG4VideoStreamFramer() const {
   return False; // default implementation
 }
+Boolean MediaSource::isH264VideoStreamFramer() const {
+  return False; // default implementation
+}
 Boolean MediaSource::isJPEGVideoSource() const {
   return False; // default implementation
 }
