@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // RTP sink for a common kind of payload format: Those which pack multiple,
 // complete codec frames (as many as possible) into each RTP packet.
 // Implementation
@@ -24,9 +24,15 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 ////////// MultiFramedRTPSink //////////
 
-static unsigned const maxPacketSize = 1450;
-	// bytes (1500, minus some allowance for IP, UDP, UMTP headers)
-static unsigned const preferredPacketSize = 1000; // bytes
+void MultiFramedRTPSink::setPacketSizes(unsigned preferredPacketSize,
+					unsigned maxPacketSize) {
+  if (preferredPacketSize > maxPacketSize || preferredPacketSize == 0) return;
+      // sanity check
+
+  delete fOutBuf;
+  fOutBuf = new OutPacketBuffer(preferredPacketSize, maxPacketSize);
+  fOurMaxPacketSize = maxPacketSize; // save value, in case subclasses need it 
+}
 
 MultiFramedRTPSink::MultiFramedRTPSink(UsageEnvironment& env,
 				       Groupsock* rtpGS,
@@ -36,8 +42,10 @@ MultiFramedRTPSink::MultiFramedRTPSink(UsageEnvironment& env,
 				       unsigned numChannels)
   : RTPSink(env, rtpGS, rtpPayloadType, rtpTimestampFrequency,
 	    rtpPayloadFormatName, numChannels),
-    fCurFragmentationOffset(0), fPreviousFrameEndedFragmentation(False) {
-  fOutBuf = new OutPacketBuffer(preferredPacketSize, maxPacketSize);
+  fOutBuf(NULL), fCurFragmentationOffset(0), fPreviousFrameEndedFragmentation(False) {
+  setPacketSizes(1000, 1448);
+      // Default max packet size (1500, minus allowance for IP, UDP, UMTP headers)
+      // (Also, make it a multiple of 4 bytes, just in case that matters.)
 }
 
 MultiFramedRTPSink::~MultiFramedRTPSink() {
@@ -76,6 +84,11 @@ unsigned MultiFramedRTPSink::specialHeaderSize() const {
   return 0;
 }
 
+unsigned MultiFramedRTPSink::frameSpecificHeaderSize() const {
+  // default implementation: Assume no frame-specific header:
+  return 0;
+}
+
 void MultiFramedRTPSink::setMarkerBit() {
   unsigned rtpHdr = fOutBuf->extractWord(0);
   rtpHdr |= 0x00800000;
@@ -101,16 +114,18 @@ void MultiFramedRTPSink::setSpecialHeaderBytes(unsigned char const* bytes,
   fOutBuf->insert(bytes, numBytes, fSpecialHeaderPosition + bytePosition);
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
+void MultiFramedRTPSink::setFrameSpecificHeaderWord(unsigned word,
+						    unsigned wordPosition) {
+  fOutBuf->insertWord(word, fCurFrameSpecificHeaderPosition + 4*wordPosition);
+}
+
+void MultiFramedRTPSink::setFrameSpecificHeaderBytes(unsigned char const* bytes,
+						     unsigned numBytes,
+						     unsigned bytePosition) {
+  fOutBuf->insert(bytes, numBytes, fCurFrameSpecificHeaderPosition + bytePosition);
+}
 
 Boolean MultiFramedRTPSink::continuePlaying() {
-  // Record the fact that we're starting to play now:
-  gettimeofday(&fNextSendTime, &Idunno);
-
   // Send the first packet.
   // (This will also schedule any future sends.)
   buildAndSendPacket(True);
@@ -149,6 +164,7 @@ void MultiFramedRTPSink::buildAndSendPacket(Boolean isFirstPacket) {
   fOutBuf->skipBytes(fSpecialHeaderSize);
 
   // Begin packing as many (complete) frames into the packet as we can:
+  fTotalFrameSpecificHeaderSizes = 0;
   fNoFramesLeft = False;
   fNumFramesUsedSoFar = 0;
   packFrame();
@@ -169,6 +185,12 @@ void MultiFramedRTPSink::packFrame() {
   } else {
     // Normal case: we need to read a new frame from the source
     if (fSource == NULL) return;
+
+    fCurFrameSpecificHeaderPosition = fOutBuf->curPacketSize();
+    fCurFrameSpecificHeaderSize = frameSpecificHeaderSize();
+    fOutBuf->skipBytes(fCurFrameSpecificHeaderSize);
+    fTotalFrameSpecificHeaderSizes += fCurFrameSpecificHeaderSize;
+
     fSource->getNextFrame(fOutBuf->curPtr(), fOutBuf->totalBytesAvailable(),
 			  afterGettingFrame, this, ourHandleClosure, this);
   }
@@ -188,16 +210,19 @@ void MultiFramedRTPSink
 ::afterGettingFrame1(unsigned frameSize, unsigned numTruncatedBytes,
 		     struct timeval presentationTime,
 		     unsigned durationInMicroseconds) {
+  if (fIsFirstPacket) {
+    // Record the fact that we're starting to play now:
+    gettimeofday(&fNextSendTime, NULL);
+  }
+
   if (numTruncatedBytes > 0) {
     unsigned const bufferSize = fOutBuf->totalBytesAvailable();
-    unsigned newNumPacketsLimit
-      = (frameSize + numTruncatedBytes + (maxPacketSize-1)) / maxPacketSize;
+    unsigned newMaxSize = frameSize + numTruncatedBytes;
     envir() << "MultiFramedRTPSink::afterGettingFrame1(): The input frame data was too large for our buffer size ("
 	    << bufferSize << ").  "
-	    << numTruncatedBytes << " of trailing data was dropped!  Correct this by increasing \"OutPacketBuffer::numPacketsLimit\" to at least "
-	    << newNumPacketsLimit << " (current value: "
-	    << OutPacketBuffer::numPacketsLimit
-	    << ").\n";
+	    << numTruncatedBytes << " bytes of trailing data was dropped!  Correct this by increasing \"OutPacketBuffer::maxSize\" to at least "
+	    << newMaxSize << ", *before* creating this 'RTPSink'.  (Current value is "
+	    << OutPacketBuffer::maxSize << ".)\n";
   }
   unsigned curFragmentationOffset = fCurFragmentationOffset;
   unsigned numFrameBytesToUse = frameSize;
@@ -227,19 +252,19 @@ void MultiFramedRTPSink
       // big to fit in a packet by itself, then we need to fragment it (and
       // use some of it in this packet, if the payload format permits this.)
       if (isTooBigForAPacket(frameSize)
-	  && (fNumFramesUsedSoFar == 0 || allowFragmentationAfterStart())) {
-	// We need to fragment this frame, and use some of it now:
-	overflowBytes = fOutBuf->numOverflowBytes(frameSize);
-	numFrameBytesToUse -= overflowBytes;
-	fCurFragmentationOffset += numFrameBytesToUse;
+          && (fNumFramesUsedSoFar == 0 || allowFragmentationAfterStart())) {
+        // We need to fragment this frame, and use some of it now:
+        overflowBytes = fOutBuf->numOverflowBytes(frameSize);
+        numFrameBytesToUse -= overflowBytes;
+        fCurFragmentationOffset += numFrameBytesToUse;
       } else {
-	// We don't use any of this frame now:
-	overflowBytes = frameSize;
-	numFrameBytesToUse = 0;
+        // We don't use any of this frame now:
+        overflowBytes = frameSize;
+        numFrameBytesToUse = 0;
       }
       fOutBuf->setOverflowData(fOutBuf->curPacketSize() + numFrameBytesToUse,
-			       overflowBytes, presentationTime,
-			       durationInMicroseconds);
+          overflowBytes, presentationTime,
+          durationInMicroseconds);
     } else if (fCurFragmentationOffset > 0) {
       // This is the last fragment of a frame that was fragmented over
       // more than one packet.  Do any special handling for this case:
@@ -274,13 +299,16 @@ void MultiFramedRTPSink
 
     // Send our packet now if (i) it's already at our preferred size, or
     // (ii) (heuristic) another frame of the same size as the one we just
-    // read would overflow the packet, or
+    //      read would overflow the packet, or
     // (iii) it contains the last fragment of a fragmented frame, and we
-    // don't allow anything else to follow this:
+    //      don't allow anything else to follow this or
+    // (iv) one frame per packet is allowed:
     if (fOutBuf->isPreferredSize()
-	|| fOutBuf->wouldOverflow(numFrameBytesToUse)
-	|| (fPreviousFrameEndedFragmentation
-	    && !allowOtherFramesAfterLastFragment())) {
+        || fOutBuf->wouldOverflow(numFrameBytesToUse)
+        || (fPreviousFrameEndedFragmentation && 
+            !allowOtherFramesAfterLastFragment()) 
+        || !frameCanAppearAfterPacketStart(fOutBuf->curPtr() - frameSize,
+					   frameSize) ) {
       // The packet is ready to be sent now
       sendPacketIfNecessary();
     } else {
@@ -294,9 +322,9 @@ static unsigned const rtpHeaderSize = 12;
 
 Boolean MultiFramedRTPSink::isTooBigForAPacket(unsigned numBytes) const {
   // Check whether a 'numBytes'-byte frame - together with a RTP header and
-  // (possible) special header - would be too big for an output packet:
+  // (possible) special headers - would be too big for an output packet:
   // (Later allow for RTP extension header!) #####
-  numBytes += rtpHeaderSize + specialHeaderSize();
+  numBytes += rtpHeaderSize + specialHeaderSize() + frameSpecificHeaderSize();
   return fOutBuf->isTooBigForAPacket(numBytes);
 }
 
@@ -309,8 +337,8 @@ void MultiFramedRTPSink::sendPacketIfNecessary() {
     fRTPInterface.sendPacket(fOutBuf->packet(), fOutBuf->curPacketSize());
     ++fPacketCount;
     fTotalOctetCount += fOutBuf->curPacketSize();
-    fOctetCount
-      += fOutBuf->curPacketSize() - rtpHeaderSize - fSpecialHeaderSize;
+    fOctetCount += fOutBuf->curPacketSize()
+      - rtpHeaderSize - fSpecialHeaderSize - fTotalFrameSpecificHeaderSizes;
 
     ++fSeqNo; // for next time
   }
@@ -318,17 +346,18 @@ void MultiFramedRTPSink::sendPacketIfNecessary() {
   if (fOutBuf->haveOverflowData()
       && fOutBuf->totalBytesAvailable() > fOutBuf->totalBufferSize()/2) {
     // Efficiency hack: Reset the packet start pointer to just in front of
-    // the overflow data (allowing for the RTP header and special header),
+    // the overflow data (allowing for the RTP header and special headers),
     // so that we probably don't have to "memmove()" the overflow data
     // into place when building the next packet:
-    unsigned newPacketStart
-      = fOutBuf->curPacketSize() - (rtpHeaderSize + fSpecialHeaderSize);
+    unsigned newPacketStart = fOutBuf->curPacketSize()
+      - (rtpHeaderSize + fSpecialHeaderSize + frameSpecificHeaderSize());
     fOutBuf->adjustPacketStart(newPacketStart);
   } else {
     // Normal case: Reset the packet start pointer back to the start:
     fOutBuf->resetPacketStart();
   }
   fOutBuf->resetOffset();
+  fNumFramesUsedSoFar = 0;
 
   if (fNoFramesLeft) {
     // We're done:
@@ -338,7 +367,7 @@ void MultiFramedRTPSink::sendPacketIfNecessary() {
     // is due to start playing, then make sure that we wait this long before
     // sending the next packet.
     struct timeval timeNow;
-    gettimeofday(&timeNow, &Idunno);
+    gettimeofday(&timeNow, NULL);
     int uSecondsToGo;
     if (fNextSendTime.tv_sec < timeNow.tv_sec) {
       uSecondsToGo = 0; // prevents integer underflow if too far behind
