@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // RTP source for a common kind of payload format: Those that pack multiple,
 // complete codec frames (as many as possible) into each RTP packet.
 // Implementation
@@ -29,13 +29,18 @@ class ReorderingPacketBuffer {
 public:
   ReorderingPacketBuffer(BufferedPacketFactory* packetFactory);
   virtual ~ReorderingPacketBuffer();
+  void reset();
 
   BufferedPacket* getFreePacket(MultiFramedRTPSource* ourSource);
-  void storePacket(BufferedPacket* bPacket);
+  Boolean storePacket(BufferedPacket* bPacket);
   BufferedPacket* getNextCompletedPacket(Boolean& packetLossPreceded);
   void releaseUsedPacket(BufferedPacket* packet);
   void freePacket(BufferedPacket* packet) {
-    if (packet != fSavedPacket) delete packet;
+    if (packet != fSavedPacket) {
+      delete packet;
+    } else {
+      fSavedPacketFree = True;
+    }
   }
 
   void setThresholdTime(unsigned uSeconds) { fThresholdTime = uSeconds; }
@@ -48,6 +53,7 @@ private:
   BufferedPacket* fHeadPacket;
   BufferedPacket* fSavedPacket;
       // to avoid calling new/free in the common case
+  Boolean fSavedPacketFree;
 };
 
 
@@ -58,17 +64,22 @@ MultiFramedRTPSource
 		       unsigned char rtpPayloadFormat,
 		       unsigned rtpTimestampFrequency,
 		       BufferedPacketFactory* packetFactory)
-  : RTPSource(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency),
-    fCurrentPacketBeginsFrame(True/*by default*/),
-    fCurrentPacketCompletesFrame(True/*by default*/),
-    fAreDoingNetworkReads(False), fNeedDelivery(False),
-    fPacketLossInFragmentedFrame(False) {
+  : RTPSource(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency) {
+  reset();
   fReorderingBuffer = new ReorderingPacketBuffer(packetFactory);
 
   // Try to use a big receive buffer for RTP:
   increaseReceiveBufferTo(env, RTPgs->socketNum(), 50*1024);
 }
 
+void MultiFramedRTPSource::reset() {
+  fCurrentPacketBeginsFrame = True; // by default
+  fCurrentPacketCompletesFrame = True; // by default
+  fAreDoingNetworkReads = False;
+  fNeedDelivery = False;
+  fPacketLossInFragmentedFrame = False;
+}
+
 MultiFramedRTPSource::~MultiFramedRTPSource() {
   fRTPInterface.stopNetworkReading();
   delete fReorderingBuffer;
@@ -89,6 +100,12 @@ Boolean MultiFramedRTPSource
   return True;
 }
 
+void MultiFramedRTPSource::doStopGettingFrames() {
+  fRTPInterface.stopNetworkReading();
+  fReorderingBuffer->reset();
+  reset();
+}
+
 void MultiFramedRTPSource::doGetNextFrame() {
   if (!fAreDoingNetworkReads) {
     // Turn on background read handling of incoming packets:
@@ -106,80 +123,78 @@ void MultiFramedRTPSource::doGetNextFrame() {
 }
 
 void MultiFramedRTPSource::doGetNextFrame1() {
-  // If we already have packet data available, then deliver it now.
-  if (fNeedDelivery) {
+  while (fNeedDelivery) {
+    // If we already have packet data available, then deliver it now.
     Boolean packetLossPrecededThis;
     BufferedPacket* nextPacket
       = fReorderingBuffer->getNextCompletedPacket(packetLossPrecededThis);
-    if (nextPacket != NULL) {
-      fNeedDelivery = False;
+    if (nextPacket == NULL) break;
 
-      if (nextPacket->useCount() == 0) {
-	// Before using the packet, check whether it has a special header
-	// that needs to be processed:
-	unsigned specialHeaderSize;
-	if (!processSpecialHeader(nextPacket, specialHeaderSize)) {
-	  // Something's wrong with the header; reject the packet:
-	  fReorderingBuffer->releaseUsedPacket(nextPacket);
-	  fNeedDelivery = True;
-	  return;
-	}
-	nextPacket->skip(specialHeaderSize);
-      }
+    fNeedDelivery = False;
 
-      // Check whether we're part of a multi-packet frame, and whether
-      // there was packet loss that would render this packet unusable:
-      if (fCurrentPacketBeginsFrame) {
-	if (packetLossPrecededThis || fPacketLossInFragmentedFrame) {
-	  // We didn't get all of the previous frame.
-	  // Forget any data that we used from it:
-	  fTo = fSavedTo; fMaxSize = fSavedMaxSize;
-	  fFrameSize = 0;
-	}
-	fPacketLossInFragmentedFrame = False;
-      } else if (packetLossPrecededThis) {
-	// We're in a multi-packet frame, with preceding packet loss
-	fPacketLossInFragmentedFrame = True;
-      }
-      if (fPacketLossInFragmentedFrame) {
-	// This packet is unusable; reject it:
-	  fReorderingBuffer->releaseUsedPacket(nextPacket);
-	  fNeedDelivery = True;
-	  return;
-      }
-
-      // The packet is usable. Deliver all or part of it to our caller:
-      unsigned frameSize;
-      nextPacket->use(fTo, fMaxSize, frameSize, fNumTruncatedBytes,
-		      fCurPacketRTPSeqNum, fCurPacketRTPTimestamp,
-		      fPresentationTime,
-		      fCurPacketHasBeenSynchronizedUsingRTCP,
-		      fCurPacketMarkerBit);
-      fFrameSize += frameSize;
-    
-      if (!nextPacket->hasUsableData()) {
-	// We're completely done with this packet now
+    if (nextPacket->useCount() == 0) {
+      // Before using the packet, check whether it has a special header
+      // that needs to be processed:
+      unsigned specialHeaderSize;
+      if (!processSpecialHeader(nextPacket, specialHeaderSize)) {
+	// Something's wrong with the header; reject the packet:
 	fReorderingBuffer->releaseUsedPacket(nextPacket);
-      }
-
-      if (fCurrentPacketCompletesFrame || fNumTruncatedBytes > 0) {
-	// We have all the data that the client wants.
-	if (fNumTruncatedBytes > 0) {
-	  envir() << "MultiFramedRTPSource::doGetNextFrame1(): The total received frame size exceeds the client's buffer size ("
-		  << fSavedMaxSize << ").  "
-		  << fNumTruncatedBytes << " bytes of trailing data will be dropped!\n";
-	}
-	// Call our own 'after getting' function.  Because we're preceded
-	// by a network read, we can call this directly, without risking
-	// infinite recursion.
-	afterGetting(this);
-      } else {
-	// This packet contained fragmented data, and does not complete
-	// the data that the client wants.  Keep getting data:
-	fTo += frameSize; fMaxSize -= frameSize;
 	fNeedDelivery = True;
-	doGetNextFrame1();
+	break;
       }
+      nextPacket->skip(specialHeaderSize);
+    }
+
+    // Check whether we're part of a multi-packet frame, and whether
+    // there was packet loss that would render this packet unusable:
+    if (fCurrentPacketBeginsFrame) {
+      if (packetLossPrecededThis || fPacketLossInFragmentedFrame) {
+	// We didn't get all of the previous frame.
+	// Forget any data that we used from it:
+	fTo = fSavedTo; fMaxSize = fSavedMaxSize;
+	fFrameSize = 0;
+      }
+      fPacketLossInFragmentedFrame = False;
+    } else if (packetLossPrecededThis) {
+      // We're in a multi-packet frame, with preceding packet loss
+      fPacketLossInFragmentedFrame = True;
+    }
+    if (fPacketLossInFragmentedFrame) {
+      // This packet is unusable; reject it:
+      fReorderingBuffer->releaseUsedPacket(nextPacket);
+      fNeedDelivery = True;
+      break;
+    }
+
+    // The packet is usable. Deliver all or part of it to our caller:
+    unsigned frameSize;
+    nextPacket->use(fTo, fMaxSize, frameSize, fNumTruncatedBytes,
+		    fCurPacketRTPSeqNum, fCurPacketRTPTimestamp,
+		    fPresentationTime, fCurPacketHasBeenSynchronizedUsingRTCP,
+		    fCurPacketMarkerBit);
+    fFrameSize += frameSize;
+    
+    if (!nextPacket->hasUsableData()) {
+      // We're completely done with this packet now
+      fReorderingBuffer->releaseUsedPacket(nextPacket);
+    }
+
+    if (fCurrentPacketCompletesFrame || fNumTruncatedBytes > 0) {
+      // We have all the data that the client wants.
+      if (fNumTruncatedBytes > 0) {
+	envir() << "MultiFramedRTPSource::doGetNextFrame1(): The total received frame size exceeds the client's buffer size ("
+		<< fSavedMaxSize << ").  "
+		<< fNumTruncatedBytes << " bytes of trailing data will be dropped!\n";
+      }
+      // Call our own 'after getting' function.  Because we're preceded
+      // by a network read, we can call this directly, without risking
+      // infinite recursion.
+      afterGetting(this);
+    } else {
+      // This packet contained fragmented data, and does not complete
+      // the data that the client wants.  Keep getting data:
+      fTo += frameSize; fMaxSize -= frameSize;
+      fNeedDelivery = True;
     }
   }
 }
@@ -189,12 +204,6 @@ void MultiFramedRTPSource
   fReorderingBuffer->setThresholdTime(uSeconds);
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 #define ADVANCE(n) do { bPacket->skip(n); } while (0)
 
 void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
@@ -252,6 +261,7 @@ void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
     }
     
     // The rest of the packet is the usable data.  Record and save it:
+    source->fLastReceivedSSRC = rtpSSRC;
     unsigned short rtpSeqNo = (unsigned short)(rtpHdr&0xFFFF);
     Boolean usableInJitterCalculation
       = source->packetIsUsableInJitterCalculation((bPacket->data()),
@@ -266,11 +276,11 @@ void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
   
     // Fill in the rest of the packet descriptor, and store it:
     struct timeval timeNow;
-    gettimeofday(&timeNow, &Idunno);
+    gettimeofday(&timeNow, NULL);
     bPacket->assignMiscParams(rtpSeqNo, rtpTimestamp, presentationTime,
 			      hasBeenSyncedUsingRTCP, rtpMarkerBit,
 			      timeNow);
-    source->fReorderingBuffer->storePacket(bPacket);
+    if (!source->fReorderingBuffer->storePacket(bPacket)) break;
 
     readSuccess = True;
   } while (0);
@@ -301,6 +311,7 @@ void BufferedPacket::reset() {
   fUseCount = 0;
 }
 
+// The following function has been deprecated:
 unsigned BufferedPacket
 ::nextEnclosedFrameSize(unsigned char*& /*framePtr*/, unsigned dataSize) {
   // By default, use the entire buffered data, even though it may consist
@@ -309,6 +320,21 @@ unsigned BufferedPacket
   return dataSize;
 }
 
+void BufferedPacket
+::getNextEnclosedFrameParameters(unsigned char*& framePtr, unsigned dataSize,
+				 unsigned& frameSize,
+				 unsigned& frameDurationInMicroseconds) {
+  // By default, use the entire buffered data, even though it may consist
+  // of more than one frame, on the assumption that the client doesn't
+  // care.  (This is more efficient than delivering a frame at a time)
+
+  // For backwards-compatibilty with existing uses of (the now deprecated)
+  // "nextEnclosedFrameSize()", call that function to implement this one:
+  frameSize = nextEnclosedFrameSize(framePtr, dataSize);
+
+  frameDurationInMicroseconds = 0; // by default.  Subclasses should correct this.  
+}
+
 Boolean BufferedPacket::fillInData(RTPInterface& rtpInterface) {
   reset();
 
@@ -358,8 +384,10 @@ void BufferedPacket::use(unsigned char* to, unsigned toSize,
 			 Boolean& hasBeenSyncedUsingRTCP,
 			 Boolean& rtpMarkerBit) {
   unsigned char* origFramePtr = &fBuf[fHead];
-  unsigned char* newFramePtr = origFramePtr; //may change in the call below
-  unsigned frameSize = nextEnclosedFrameSize(newFramePtr, fTail - fHead);
+  unsigned char* newFramePtr = origFramePtr; // may change in the call below
+  unsigned frameSize, frameDurationInMicroseconds;
+  getNextEnclosedFrameParameters(newFramePtr, fTail - fHead,
+				 frameSize, frameDurationInMicroseconds);
   if (frameSize > toSize) {
     bytesTruncated = frameSize - toSize;
     bytesUsed = toSize;
@@ -377,46 +405,67 @@ void BufferedPacket::use(unsigned char* to, unsigned toSize,
   presentationTime = fPresentationTime;
   hasBeenSyncedUsingRTCP = fHasBeenSyncedUsingRTCP;
   rtpMarkerBit = fRTPMarkerBit;
+
+  // Update "fPresentationTime" for the next enclosed frame (if any):
+  fPresentationTime.tv_usec += frameDurationInMicroseconds;
+  if (fPresentationTime.tv_usec >= 1000000) {
+    fPresentationTime.tv_sec += fPresentationTime.tv_usec/1000000;
+    fPresentationTime.tv_usec = fPresentationTime.tv_usec%1000000;
+  }
+}
+
+BufferedPacketFactory::BufferedPacketFactory() {
+}
+
+BufferedPacketFactory::~BufferedPacketFactory() {
 }
 
 BufferedPacket* BufferedPacketFactory
-::createNew(MultiFramedRTPSource* /*ourSource*/) {
+::createNewPacket(MultiFramedRTPSource* /*ourSource*/) {
   return new BufferedPacket;
 }
 
 
-////////// ReorderingPacketBuffer definition //////////
+////////// ReorderingPacketBuffer implementation //////////
 
 ReorderingPacketBuffer
 ::ReorderingPacketBuffer(BufferedPacketFactory* packetFactory)
   : fThresholdTime(100000) /* default reordering threshold: 100 ms */,
-    fHaveSeenFirstPacket(False), fHeadPacket(NULL), fSavedPacket(NULL) {
+    fHaveSeenFirstPacket(False), fHeadPacket(NULL), fSavedPacket(NULL), fSavedPacketFree(True) {
   fPacketFactory = (packetFactory == NULL)
     ? (new BufferedPacketFactory)
     : packetFactory;
 }
 
 ReorderingPacketBuffer::~ReorderingPacketBuffer() {
-  if (fHeadPacket == NULL) {
-    delete fSavedPacket;
-  } else {
-    delete fHeadPacket; // will also delete fSavedPacket, because it's on the list
-  }
+  reset();
   delete fPacketFactory;
 }
 
+void ReorderingPacketBuffer::reset() {
+  if (fSavedPacketFree) delete fSavedPacket; // because fSavedPacket is not in the list
+  delete fHeadPacket; // will also delete fSavedPacket if it's in the list
+  fHaveSeenFirstPacket = False;
+  fHeadPacket = NULL;
+  fSavedPacket = NULL;
+}
+
 BufferedPacket* ReorderingPacketBuffer
 ::getFreePacket(MultiFramedRTPSource* ourSource) {
   if (fSavedPacket == NULL) { // we're being called for the first time
-    fSavedPacket = fPacketFactory->createNew(ourSource);
+    fSavedPacket = fPacketFactory->createNewPacket(ourSource);
+    fSavedPacketFree = True;
+  }
+  
+  if (fSavedPacketFree == True) {
+    fSavedPacketFree = False;
+    return fSavedPacket;
+  } else {
+    return fPacketFactory->createNewPacket(ourSource);
   }
-
-  return fHeadPacket == NULL
-    ? fSavedPacket
-    : fPacketFactory->createNew(ourSource);
 }
 
-void ReorderingPacketBuffer::storePacket(BufferedPacket* bPacket) {
+Boolean ReorderingPacketBuffer::storePacket(BufferedPacket* bPacket) {
   unsigned short rtpSeqNo = bPacket->rtpSeqNo();
 
   if (!fHaveSeenFirstPacket) {
@@ -431,7 +480,7 @@ void ReorderingPacketBuffer::storePacket(BufferedPacket* bPacket) {
   unsigned short const seqNoThreshold = 100;
   if (seqNumLT(rtpSeqNo, fNextExpectedSeqNo)
       && seqNumLT(fNextExpectedSeqNo, rtpSeqNo+seqNoThreshold)) {
-    return;
+    return False;
   }
   
   // Figure out where the new packet will be stored in the queue:
@@ -441,7 +490,7 @@ void ReorderingPacketBuffer::storePacket(BufferedPacket* bPacket) {
     if (seqNumLT(rtpSeqNo, afterPtr->rtpSeqNo())) break; // it comes here
     if (rtpSeqNo == afterPtr->rtpSeqNo()) {
       // This is a duplicate packet - ignore it
-      return;
+      return False;
     }
     
     beforePtr = afterPtr;
@@ -455,6 +504,8 @@ void ReorderingPacketBuffer::storePacket(BufferedPacket* bPacket) {
   } else {
     beforePtr->nextPacket() = bPacket;
   }
+
+  return True;
 }
 
 void ReorderingPacketBuffer::releaseUsedPacket(BufferedPacket* packet) {
@@ -484,7 +535,7 @@ BufferedPacket* ReorderingPacketBuffer
   // our time threshold has been exceeded, then forget it, and return
   // the head packet instead:
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   unsigned uSecondsSinceReceived
     = (timeNow.tv_sec - fHeadPacket->timeReceived().tv_sec)*1000000
     + (timeNow.tv_usec - fHeadPacket->timeReceived().tv_usec);
