@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that represents an existing
 // 'RTPSink', rather than one that creates new 'RTPSink's on demand.
 // Implementation
@@ -36,7 +36,8 @@ PassiveServerMediaSubsession
     fRTPSink(rtpSink), fRTCPInstance(rtcpInstance), fSDPLines(NULL) {
 }
 
-char const* PassiveServerMediaSubsession::sdpLines() {
+char const*
+PassiveServerMediaSubsession::sdpLines() {
   if (fSDPLines == NULL ) {
     // Construct a set of SDP lines that describe this subsession:
     // Use the components from "rtpSink":
@@ -44,48 +45,12 @@ char const* PassiveServerMediaSubsession::sdpLines() {
     struct in_addr const& ipAddress = gs.groupAddress();
     unsigned short portNum = ntohs(gs.port().num());
     unsigned char ttl = gs.ttl();
-    unsigned rtpTimestampFrequency = fRTPSink.rtpTimestampFrequency();
-    unsigned numChannels = fRTPSink.numChannels();
     unsigned char rtpPayloadType = fRTPSink.rtpPayloadType();
     char const* mediaType = fRTPSink.sdpMediaType();
-    char const* rtpPayloadFormatName = fRTPSink.rtpPayloadFormatName();
+    char* rtpmapLine = fRTPSink.rtpmapLine();
+    char const* rangeLine = rangeSDPLine();
     char const* auxSDPLine = fRTPSink.auxSDPLine();
-
-    // For dynamic payload types, we need a "a=rtpmap:" line also:
-    char* rtpmapLine;
-    unsigned rtpmapLineSize;
-    if (rtpPayloadType >= 96) {
-      char* encodingParamsPart;
-      if (numChannels != 1) {
-	encodingParamsPart = new char[1 + 20 /* max int len */];
-	sprintf(encodingParamsPart, "/%d", numChannels);
-      } else {
-	encodingParamsPart = strDup("");
-      }
-      char const* const rtpmapFmt = "a=rtpmap:%d %s/%d%s\r\n";
-      unsigned rtpmapFmtSize = strlen(rtpmapFmt)
-	+ 3 /* max char len */ + strlen(rtpPayloadFormatName)
-	+ 20 /* max int len */ + strlen(encodingParamsPart);
-      rtpmapLine = new char[rtpmapFmtSize];
-      sprintf(rtpmapLine, rtpmapFmt,
-	      rtpPayloadType, rtpPayloadFormatName,
-	      rtpTimestampFrequency, encodingParamsPart);
-      rtpmapLineSize = strlen(rtpmapLine);
-      delete[] encodingParamsPart;
-    } else {
-      // There's no "a=rtpmap:" line:
-      // Static payload type => no "a=rtpmap:" line
-      rtpmapLine = strDup("");
-      rtpmapLineSize = 0;
-    }
-    
-    unsigned auxSDPLineSize;
-    if (auxSDPLine == NULL) {
-      auxSDPLine = "";
-      auxSDPLineSize = 0;
-    } else {
-      auxSDPLineSize = strlen(auxSDPLine);
-    }
+    if (auxSDPLine == NULL) auxSDPLine = "";
     
     char* const ipAddressStr = strDup(our_inet_ntoa(ipAddress));
     
@@ -94,12 +59,14 @@ char const* PassiveServerMediaSubsession::sdpLines() {
       "c=IN IP4 %s/%d\r\n" 
       "%s"
       "%s"
+      "%s"
       "a=control:%s\r\n";
     unsigned sdpFmtSize = strlen(sdpFmt)
       + strlen(mediaType) + 5 /* max short len */ + 3 /* max char len */
       + strlen(ipAddressStr) + 3 /* max char len */
-      + rtpmapLineSize
-      + auxSDPLineSize
+      + strlen(rtpmapLine)
+      + strlen(rangeLine)
+      + strlen(auxSDPLine)
       + strlen(trackId());
     char* sdpLines = new char[sdpFmtSize];
     sprintf(sdpLines, sdpFmt, 
@@ -109,9 +76,10 @@ char const* PassiveServerMediaSubsession::sdpLines() {
 	    ipAddressStr, // c= <connection address>
 	    ttl, // c= TTL
 	    rtpmapLine, // a=rtpmap:... (if present)
+	    rangeLine, // a=range:... (if present)
 	    auxSDPLine, // optional extra SDP line
 	    trackId()); // a=control:<track-id>
-    delete[] ipAddressStr; delete[] rtpmapLine;
+    delete[] ipAddressStr; delete[] (char*)rangeLine; delete[] rtpmapLine;
     
     fSDPLines = strDup(sdpLines);
     delete[] sdpLines;
@@ -148,9 +116,26 @@ void PassiveServerMediaSubsession
     }
   }
   serverRTPPort = gs.port();
-  // serverRTCPPort is not needed, so we don't set it
+  if (fRTCPInstance != NULL) {
+    Groupsock* rtcpGS = fRTCPInstance->RTCPgs();
+    serverRTCPPort = rtcpGS->port();
+  }
   streamToken = NULL; // not used
 }
 
+void PassiveServerMediaSubsession::startStream(unsigned /*clientSessionId*/,
+					       void* /*streamToken*/,
+					       TaskFunc* /*rtcpRRHandler*/,
+					       void* /*rtcpRRHandlerClientData*/,
+					       unsigned short& rtpSeqNum,
+					       unsigned& rtpTimestamp) {
+  // Note: We don't set a RTCP RR handler, because (i) we're called potentially
+  // many times on the same "RTCPInstance", and (ii) the "RTCPInstance" remains
+  // in existence after "deleteStream()" is called.
+  rtpSeqNum = fRTPSink.currentSeqNo();
+  rtpTimestamp = fRTPSink.currentTimestamp();
+}
+
 PassiveServerMediaSubsession::~PassiveServerMediaSubsession() {
+  delete[] fSDPLines;
 }
