@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // Qualcomm "PureVoice" (aka. "QCELP") Audio RTP Sources
 // Implementation
 
@@ -131,14 +131,8 @@ private:
 };
 
 class QCELPBufferedPacketFactory: public BufferedPacketFactory {
-public:
-  QCELPBufferedPacketFactory(RawQCELPRTPSource& ourSource);
-
 private: // redefined virtual functions
-  virtual BufferedPacket* createNew(MultiFramedRTPSource* ourSource);
-
-private:
-  RawQCELPRTPSource& fOurSource;
+  virtual BufferedPacket* createNewPacket(MultiFramedRTPSource* ourSource);
 };
 
 
@@ -158,7 +152,7 @@ RawQCELPRTPSource::RawQCELPRTPSource(UsageEnvironment& env,
 				     unsigned rtpTimestampFrequency)
   : MultiFramedRTPSource(env, RTPgs, rtpPayloadFormat,
 			 rtpTimestampFrequency,
-                         new QCELPBufferedPacketFactory(*this)),
+                         new QCELPBufferedPacketFactory),
   fInterleaveL(0), fInterleaveN(0), fFrameIndex(0),
   fNumSuccessiveSyncedPackets(0) {
 }
@@ -201,7 +195,7 @@ Boolean RawQCELPRTPSource
 } 
 
 char const* RawQCELPRTPSource::MIMEtype() const {
-  return "audio/qcelp";
+  return "audio/QCELP";
 }
 
 Boolean RawQCELPRTPSource::hasBeenSynchronizedUsingRTCP() {
@@ -251,14 +245,9 @@ unsigned QCELPBufferedPacket::
   return frameSize;
 }
 
-QCELPBufferedPacketFactory
-::QCELPBufferedPacketFactory(RawQCELPRTPSource& ourSource)
-  : fOurSource(ourSource) {
-}
-
 BufferedPacket* QCELPBufferedPacketFactory
-::createNew(MultiFramedRTPSource* ourSource) {
-  return new QCELPBufferedPacket(fOurSource);
+::createNewPacket(MultiFramedRTPSource* ourSource) {
+  return new QCELPBufferedPacket((RawQCELPRTPSource&)(*ourSource));
 }
 
 ///////// QCELPDeinterleavingBuffer /////////
