@@ -14,20 +14,19 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A sink that generates a QuickTime file from a composite media session
 // Implementation
 
-#if defined(__WIN32__) || defined(_WIN32)
-#include <io.h>
-#include <fcntl.h>
-#endif
-#include <ctype.h>
-
 #include "QuickTimeFileSink.hh"
 #include "QuickTimeGenericRTPSource.hh"
 #include "GroupsockHelper.hh"
-#include "H263plusVideoRTPSource.hh"
+#include "OutputFile.hh"
+#include "H263plusVideoRTPSource.hh" // for the special header
+#include "MPEG4GenericRTPSource.hh" //for "samplingFrequencyFromAudioSpecificConfig()"
+#include "MPEG4LATMAudioRTPSource.hh" // for "parseGeneralConfigStr()"
+
+#include <ctype.h>
 
 #define fourChar(x,y,z,w) ( ((x)<<24)|((y)<<16)|((z)<<8)|(w) )
 
@@ -57,14 +56,19 @@ public:
 
 class SubsessionBuffer {
 public:
-  SubsessionBuffer() { reset(); }
+  SubsessionBuffer(unsigned bufferSize)
+    : fBufferSize(bufferSize) {
+    reset();
+    fData = new unsigned char[bufferSize];
+  }
+  virtual ~SubsessionBuffer() { delete fData; }
   void reset() { fBytesInUse = 0; }
   void addBytes(unsigned numBytes) { fBytesInUse += numBytes; }
 
   unsigned char* dataStart() { return &fData[0]; }
   unsigned char* dataEnd() { return &fData[fBytesInUse]; }
   unsigned bytesInUse() const { return fBytesInUse; }
-  unsigned bytesAvailable() const { return sizeof fData - fBytesInUse; }
+  unsigned bytesAvailable() const { return fBufferSize - fBytesInUse; }
   
   void setPresentationTime(struct timeval const& presentationTime) {
     fPresentationTime = presentationTime;
@@ -72,8 +76,9 @@ public:
   struct timeval const& presentationTime() const {return fPresentationTime;}
 
 private:
+  unsigned fBufferSize;
   struct timeval fPresentationTime;
-  unsigned char fData[20000];
+  unsigned char* fData;
   unsigned fBytesInUse;
 };
 
@@ -185,8 +190,9 @@ private:
 
     // The remaining fields are used for hint tracks only:
     unsigned startSampleNumber;
+    unsigned short seqNum;
     unsigned rtpHeader;
-    unsigned char numSpecialHeaders; // used when our RTP source is H.263+
+    unsigned char numSpecialHeaders; // used when our RTP source has special headers
     unsigned specialHeaderBytesLength; // ditto
     unsigned char specialHeaderBytes[SPECIAL_HEADER_BUFFER_SIZE]; // ditto
     unsigned packetSizes[256];
@@ -196,24 +202,21 @@ private:
 
 ////////// QuickTimeFileSink implementation //////////
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 QuickTimeFileSink::QuickTimeFileSink(UsageEnvironment& env,
 				     MediaSession& inputSession,
 				     FILE* outFid,
+				     unsigned bufferSize,
 				     unsigned short movieWidth,
 				     unsigned short movieHeight,
 				     unsigned movieFPS,
 				     Boolean packetLossCompensate,
 				     Boolean syncStreams,
-				     Boolean generateHintTracks)
+				     Boolean generateHintTracks,
+				     Boolean generateMP4Format)
   : Medium(env), fInputSession(inputSession), fOutFid(outFid),
-    fPacketLossCompensate(packetLossCompensate),
-    fSyncStreams(syncStreams), fAreCurrentlyBeingPlayed(False),
+    fBufferSize(bufferSize), fPacketLossCompensate(packetLossCompensate),
+    fSyncStreams(syncStreams), fGenerateMP4Format(generateMP4Format),
+    fAreCurrentlyBeingPlayed(False),
     fLargestRTPtimestampFrequency(0),
     fNumSubsessions(0), fNumSyncedSubsessions(0),
     fHaveCompletedOutputFile(False),
@@ -279,7 +282,7 @@ QuickTimeFileSink::QuickTimeFileSink(UsageEnvironment& env,
   // Use the current time as the file's creation and modification
   // time.  Use Apple's time format: seconds since January 1, 1904
 
-  gettimeofday(&fStartTime, &Idunno);
+  gettimeofday(&fStartTime, NULL);
   fAppleCreationTime = fStartTime.tv_sec - 0x83dac000;
 
   // Begin by writing a "mdat" atom at the start of the file.
@@ -305,32 +308,28 @@ QuickTimeFileSink::~QuickTimeFileSink() {
   }
 }
 
-static FILE* openFileByName(UsageEnvironment& env, char const* fileName);
-    // forward
-
 QuickTimeFileSink*
 QuickTimeFileSink::createNew(UsageEnvironment& env,
 			     MediaSession& inputSession,
 			     char const* outputFileName,
+			     unsigned bufferSize,
 			     unsigned short movieWidth,
 			     unsigned short movieHeight,
 			     unsigned movieFPS,
 			     Boolean packetLossCompensate,
 			     Boolean syncStreams,
-			     Boolean generateHintTracks) {
-  QuickTimeFileSink* newSink = NULL;
-
+			     Boolean generateHintTracks,
+			     Boolean generateMP4Format) {
   do {
-    FILE* fid = openFileByName(env, outputFileName);
+    FILE* fid = OpenOutputFile(env, outputFileName);
     if (fid == NULL) break;
 
-    return new QuickTimeFileSink(env, inputSession, fid,
+    return new QuickTimeFileSink(env, inputSession, fid, bufferSize,
 				 movieWidth, movieHeight, movieFPS,
 				 packetLossCompensate, syncStreams,
-				 generateHintTracks);
+				 generateHintTracks, generateMP4Format);
   } while (0);
 
-  delete newSink;
   return NULL;
 }
 
@@ -424,7 +423,7 @@ void QuickTimeFileSink::onRTCPBye(void* clientData) {
   SubsessionIOState* ioState = (SubsessionIOState*)clientData;
 
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   unsigned secsDiff
     = timeNow.tv_sec - ioState->fOurSink.fStartTime.tv_sec;
 
@@ -483,6 +482,11 @@ void QuickTimeFileSink::completeOutputFile() {
     }
   }
 
+  if (fGenerateMP4Format) {
+    // Begin with a "ftyp" atom:
+    addAtom_ftyp();
+  }
+
   // Then, add a "moov" atom for the file metadata:
   addAtom_moov();
 
@@ -490,33 +494,6 @@ void QuickTimeFileSink::completeOutputFile() {
   fHaveCompletedOutputFile = True;
 }
 
-// NOTE: The following is the same code that's in "FileSink"
-// Someday we should share it #####
-static FILE* openFileByName(UsageEnvironment& env, char const* fileName) {
-  FILE* fid;
-    
-  // Check for special case 'file names': "stdout" and "stderr"
-  if (strcmp(fileName, "stdout") == 0) {
-    fid = stdout;
-#if defined(__WIN32__) || defined(_WIN32)
-    _setmode(_fileno(stdout), _O_BINARY);       // convert to binary mode
-#endif
-  } else if (strcmp(fileName, "stderr") == 0) {
-    fid = stderr;
-#if defined(__WIN32__) || defined(_WIN32)
-    _setmode(_fileno(stderr), _O_BINARY);       // convert to binary mode
-#endif
-  } else {
-    fid = fopen(fileName, "wb");
-  }
-
-  if (fid == NULL) {
-    env.setResultMsg("unable to open file \"", fileName, "\"");
-  }
-
-  return fid;
-}
-
 
 ////////// SubsessionIOState, ChunkDescriptor implementation ///////////
 
@@ -530,14 +507,16 @@ SubsessionIOState::SubsessionIOState(QuickTimeFileSink& sink,
     fHeadChunk(NULL), fTailChunk(NULL), fNumChunks(0) {
   fTrackID = ++fCurrentTrackNumber;
 
-  fBuffer = new SubsessionBuffer;
-  fPrevBuffer = sink.fPacketLossCompensate ? new SubsessionBuffer : NULL;
+  fBuffer = new SubsessionBuffer(fOurSink.fBufferSize);
+  fPrevBuffer = sink.fPacketLossCompensate
+    ? new SubsessionBuffer(fOurSink.fBufferSize) : NULL;
 
   FramedSource* subsessionSource = subsession.readSource();
   fOurSourceIsActive = subsessionSource != NULL;
 
   fPrevFrameState.presentationTime.tv_sec = 0;
   fPrevFrameState.presentationTime.tv_usec = 0;
+  fPrevFrameState.seqNum = 0;
 }
 
 SubsessionIOState::~SubsessionIOState() {
@@ -548,7 +527,7 @@ SubsessionIOState::~SubsessionIOState() {
 Boolean SubsessionIOState::setQTstate() {
   char const* noCodecWarning1 = "Warning: We don't implement a QuickTime ";
   char const* noCodecWarning2 = " Media Data Type for the \"";
-  char const* noCodecWarning3 = "\" track, so we'll insert a dummy \"???\" Media Data Atom instead.  A separate, codec-specific editing pass will be needed before this track can be played.\n";
+  char const* noCodecWarning3 = "\" track, so we'll insert a dummy \"????\" Media Data Atom instead.  A separate, codec-specific editing pass will be needed before this track can be played.\n";
   Boolean supportPartiallyOnly = False;
 
   do {
@@ -593,6 +572,15 @@ Boolean SubsessionIOState::setQTstate() {
       } else if (strcmp(fOurSubsession.codecName(), "QCELP") == 0) {
 	fQTMediaDataAtomCreator = &QuickTimeFileSink::addAtom_Qclp;
 	fQTSamplesPerFrame = 160;
+      } else if (strcmp(fOurSubsession.codecName(), "MPEG4-GENERIC") == 0 ||
+		 strcmp(fOurSubsession.codecName(), "MP4A-LATM") == 0) {
+	fQTMediaDataAtomCreator = &QuickTimeFileSink::addAtom_mp4a;
+	fQTTimeUnitsPerSample = 1024; // QT considers each frame to be a 'sample'
+	// The time scale (frequency) comes from the 'config' information.
+	// It might be different from the RTP timestamp frequency (e.g., aacPlus).
+	unsigned frequencyFromConfig
+	  = samplingFrequencyFromAudioSpecificConfig(fOurSubsession.fmtp_config());
+	if (frequencyFromConfig != 0) fQTTimeScale = frequencyFromConfig;
       } else {
 	envir() << noCodecWarning1 << "Audio" << noCodecWarning2
 		<< fOurSubsession.codecName() << noCodecWarning3;
@@ -613,6 +601,10 @@ Boolean SubsessionIOState::setQTstate() {
 	fQTMediaDataAtomCreator = &QuickTimeFileSink::addAtom_h263;
 	fQTTimeScale = 600;
 	fQTTimeUnitsPerSample = fQTTimeScale/fOurSink.fMovieFPS;
+      } else if (strcmp(fOurSubsession.codecName(), "MP4V-ES") == 0) {
+	fQTMediaDataAtomCreator = &QuickTimeFileSink::addAtom_mp4v;
+	fQTTimeScale = 600;
+	fQTTimeUnitsPerSample = fQTTimeScale/fOurSink.fMovieFPS;
       } else {
 	envir() << noCodecWarning1 << "Video" << noCodecWarning2
 		<< fOurSubsession.codecName() << noCodecWarning3;
@@ -649,9 +641,7 @@ void SubsessionIOState::setFinalQTstate() {
   ChunkDescriptor* chunk = fHeadChunk;
   while (chunk != NULL) {
     unsigned const numFrames = chunk->fNumFrames;
-    unsigned const dur
-      = numFrames*chunk->fFrameDuration/fOurSubsession.numChannels();
-
+    unsigned const dur = numFrames*chunk->fFrameDuration;
     fQTDurationT += dur;
 
     chunk = chunk->fNextChunk;
@@ -791,7 +781,7 @@ void SubsessionIOState::useFrame(SubsessionBuffer& buffer) {
   }
 
   // Write the data into the file:
-  fwrite(frameSource, frameSize, 1, fOurSink.fOutFid);
+  fwrite(frameSource, 1, frameSize, fOurSink.fOutFid);
 
   // If we have a hint track, then write to it also:
   if (hasHintTrack()) {
@@ -811,14 +801,17 @@ void SubsessionIOState::useFrame(SubsessionBuffer& buffer) {
 void SubsessionIOState::useFrameForHinting(unsigned frameSize,
 					   struct timeval presentationTime,
 					   unsigned startSampleNumber) {
-  // Normally, assume that each frame has just a single RTP packet.
-  // This simplifies the hinting code, even though it might not be
-  // true in practice.  (Instead of multiple RTP packets, we'll get
-  // a single, large RTP packet that may get fragmented at the IP level.)
-  // However, if the source is H.263+, then we need to break the frame
-  // back into separate RTP packets, because we want to reuse the special
+  // At this point, we have a single, combined frame - not individual packets.
+  // For the hint track, we need to split the frame back up into separate packets.
+  // However, for some RTP sources, then we also need to reuse the special
   // header bytes that were at the start of each of the RTP packets.
   Boolean hack263 = strcmp(fOurSubsession.codecName(), "H263-1998") == 0;
+  Boolean hackm4a_generic = strcmp(fOurSubsession.mediumName(), "audio") == 0
+    && strcmp(fOurSubsession.codecName(), "MPEG4-GENERIC") == 0;
+  Boolean hackm4a_latm = strcmp(fOurSubsession.mediumName(), "audio") == 0
+    && strcmp(fOurSubsession.codecName(), "MP4A-LATM") == 0;
+  Boolean hackm4a = hackm4a_generic || hackm4a_latm;
+  Boolean haveSpecialHeaders = (hack263 || hackm4a_generic);
 
   // If there has been a previous frame, then output a 'hint sample' for it.
   // (We use the current frame's presentation time to compute the previous
@@ -833,13 +826,30 @@ void SubsessionIOState::useFrameForHinting(unsigned frameSize,
     if (msDuration > fHINF.dmax) fHINF.dmax = msDuration;
     unsigned hintSampleDuration
       = (unsigned)((2*duration*fQTTimeScale+1)/2); // round
+    if (hackm4a) {
+      // Because multiple AAC frames can appear in a RTP packet, the presentation
+      // times of the second and subsequent frames will not be accurate.
+      // So, use the known "hintSampleDuration" instead:
+      hintSampleDuration = fTrackHintedByUs->fQTTimeUnitsPerSample;
+
+      // Also, if the 'time scale' was different from the RTP timestamp frequency,
+      // (as can happen with aacPlus), then we need to scale "hintSampleDuration"
+      // accordingly:
+      if (fTrackHintedByUs->fQTTimeScale != fOurSubsession.rtpTimestampFrequency()) {
+	unsigned const scalingFactor
+	  = fOurSubsession.rtpTimestampFrequency()/fTrackHintedByUs->fQTTimeScale ;
+	hintSampleDuration *= scalingFactor;
+      }
+    }
 
     unsigned const hintSampleDestFileOffset = ftell(fOurSink.fOutFid);
 
-    unsigned short numPTEntries = 1; // normal case
+    unsigned const maxPacketSize = 1450;
+    unsigned short numPTEntries
+      = (fPrevFrameState.frameSize + (maxPacketSize-1))/maxPacketSize; // normal case
     unsigned char* immediateDataPtr = NULL;
     unsigned immediateDataBytesRemaining = 0;
-    if (hack263) { // special case
+    if (haveSpecialHeaders) { // special case
       numPTEntries = fPrevFrameState.numSpecialHeaders;
       immediateDataPtr = fPrevFrameState.specialHeaderBytes;
       immediateDataBytesRemaining
@@ -853,47 +863,53 @@ void SubsessionIOState::useFrameForHinting(unsigned frameSize,
     for (unsigned i = 0; i < numPTEntries; ++i) {
       // Output a Packet Table entry (representing a single RTP packet):
       unsigned short numDTEntries = 1;
+      unsigned short seqNum = fPrevFrameState.seqNum++;
+          // Note: This assumes that the input stream had no packets lost #####
       unsigned rtpHeader = fPrevFrameState.rtpHeader;
-      unsigned dataFrameSize = fPrevFrameState.frameSize;
+      if (i+1 < numPTEntries) {
+	// This is not the last RTP packet, so clear the marker bit:
+	rtpHeader &=~ (1<<23);
+      }
+      unsigned dataFrameSize = (i+1 < numPTEntries)
+	? maxPacketSize : fPrevFrameState.frameSize - i*maxPacketSize; // normal case
       unsigned sampleNumber = fPrevFrameState.startSampleNumber;
 
       unsigned char immediateDataLen = 0;
-      if (hack263) {
+      if (haveSpecialHeaders) { // special case
 	++numDTEntries; // to include a Data Table entry for the special hdr
 	if (immediateDataBytesRemaining > 0) {
-	  immediateDataLen = *immediateDataPtr++;
-	  --immediateDataBytesRemaining;
-	  if (immediateDataLen > immediateDataBytesRemaining) {
-	    // shouldn't happen (length byte was bad)
-	    immediateDataLen = immediateDataBytesRemaining;
+	  if (hack263) {
+	    immediateDataLen = *immediateDataPtr++;
+	    --immediateDataBytesRemaining;
+	    if (immediateDataLen > immediateDataBytesRemaining) {
+	      // shouldn't happen (length byte was bad)
+	      immediateDataLen = immediateDataBytesRemaining;
+	    }
+	  } else {
+	    immediateDataLen = fPrevFrameState.specialHeaderBytesLength;
 	  }
 	}
-	dataFrameSize
-	  = fPrevFrameState.packetSizes[i] - immediateDataLen;
+	dataFrameSize = fPrevFrameState.packetSizes[i] - immediateDataLen;
 
-	Boolean PbitSet
-	  = immediateDataLen >= 1 && (immediateDataPtr[0]&0x4) != 0;
-	if (PbitSet) {
-	  offsetWithinSample += 2; // to omit the two leading 0 bytes
-	}
-
-	if (i+1 < numPTEntries) {
-	  // This is not the last RTP packet, so clear the marker bit:
-	  rtpHeader &=~ (1<<23);
+	if (hack263) {
+	  Boolean PbitSet
+	    = immediateDataLen >= 1 && (immediateDataPtr[0]&0x4) != 0;
+	  if (PbitSet) {
+	    offsetWithinSample += 2; // to omit the two leading 0 bytes
+	  }
 	}
       }
 
       // Output the Packet Table:
       hintSampleSize += fOurSink.addWord(0); // Relative transmission time
-      unsigned seqNumBehind = numPTEntries - 1 - i;
-      hintSampleSize += fOurSink.addWord(rtpHeader - seqNumBehind);
-          // RTP header info + RTP sequence number (modified by packet #)
+      hintSampleSize += fOurSink.addWord(rtpHeader|seqNum);
+          // RTP header info + RTP sequence number
       hintSampleSize += fOurSink.addHalfWord(0x0000); // Flags
       hintSampleSize += fOurSink.addHalfWord(numDTEntries); // Entry count
       unsigned totalPacketSize = 0;
 
       // Output the Data Table:
-      if (hack263) {
+      if (haveSpecialHeaders) {
 	//   use the "Immediate Data" format (1):
 	hintSampleSize += fOurSink.addByte(1); // Source
 	unsigned char len = immediateDataLen > 14 ? 14 : immediateDataLen;
@@ -946,21 +962,35 @@ void SubsessionIOState::useFrameForHinting(unsigned frameSize,
   fPrevFrameState.startSampleNumber = startSampleNumber;
   fPrevFrameState.rtpHeader
     = rs->curPacketMarkerBit()<<23
-    | (rs->rtpPayloadFormat()&0x7F)<<16
-    | rs->curPacketRTPSeqNum();
+    | (rs->rtpPayloadFormat()&0x7F)<<16;
   if (hack263) {
-      H263plusVideoRTPSource* rs_263 = (H263plusVideoRTPSource*)rs;
-      fPrevFrameState.numSpecialHeaders = rs_263->fNumSpecialHeaders;
-      fPrevFrameState.specialHeaderBytesLength
-	= rs_263->fSpecialHeaderBytesLength;
-      unsigned i;
-      for (i = 0; i < rs_263->fSpecialHeaderBytesLength; ++i) {
-	fPrevFrameState.specialHeaderBytes[i]
-	  = rs_263->fSpecialHeaderBytes[i];
-      }
-      for (i = 0; i < rs_263->fNumSpecialHeaders; ++i) {
-	fPrevFrameState.packetSizes[i] = rs_263->fPacketSizes[i];
-      }
+    H263plusVideoRTPSource* rs_263 = (H263plusVideoRTPSource*)rs;
+    fPrevFrameState.numSpecialHeaders = rs_263->fNumSpecialHeaders;
+    fPrevFrameState.specialHeaderBytesLength = rs_263->fSpecialHeaderBytesLength;
+    unsigned i;
+    for (i = 0; i < rs_263->fSpecialHeaderBytesLength; ++i) {
+      fPrevFrameState.specialHeaderBytes[i] = rs_263->fSpecialHeaderBytes[i];
+    }
+    for (i = 0; i < rs_263->fNumSpecialHeaders; ++i) {
+      fPrevFrameState.packetSizes[i] = rs_263->fPacketSizes[i];
+    }
+  } else if (hackm4a_generic) {
+    // Synthesize a special header, so that this frame can be in its own RTP packet.
+    unsigned const sizeLength = fOurSubsession.fmtp_sizelength();
+    unsigned const indexLength = fOurSubsession.fmtp_indexlength();
+    if (sizeLength + indexLength != 16) {
+      envir() << "Warning: unexpected 'sizeLength' " << sizeLength
+	      << " and 'indexLength' " << indexLength
+	      << "seen when creating hint track\n";
+    }
+    fPrevFrameState.numSpecialHeaders = 1;
+    fPrevFrameState.specialHeaderBytesLength = 4;
+    fPrevFrameState.specialHeaderBytes[0] = 0; // AU_headers_length (high byte)
+    fPrevFrameState.specialHeaderBytes[1] = 16; // AU_headers_length (low byte)
+    fPrevFrameState.specialHeaderBytes[2] = ((frameSize<<indexLength)&0xFF00)>>8;
+    fPrevFrameState.specialHeaderBytes[3] = (frameSize<<indexLength);
+    fPrevFrameState.packetSizes[0]
+      = fPrevFrameState.specialHeaderBytesLength + frameSize;
   }
 }
 
@@ -1172,18 +1202,30 @@ void QuickTimeFileSink::setWord(unsigned filePosn, unsigned size) {
   return size; \
 }
 
+addAtom(ftyp);
+  size += add4ByteString("mp42");
+  size += addWord(0x00000000);
+  size += add4ByteString("mp42");
+  size += add4ByteString("isom");
+addAtomEnd;
+
 addAtom(moov);
   size += addAtom_mvhd();
 
+  if (fGenerateMP4Format) {
+    size += addAtom_iods();
+  }
+
   // Add a 'trak' atom for each subsession:
   // (For some unknown reason, QuickTime Player (5.0 at least)
   //  doesn't display the movie correctly unless the audio track
   //  (if present) appears before the video track.  So ensure this here.)
   MediaSubsessionIterator iter(fInputSession);
-  while ((fCurrentSubsession = iter.next()) != NULL) {
-    fCurrentIOState = (SubsessionIOState*)(fCurrentSubsession->miscPtr); 
+  MediaSubsession* subsession;
+  while ((subsession = iter.next()) != NULL) {
+    fCurrentIOState = (SubsessionIOState*)(subsession->miscPtr); 
     if (fCurrentIOState == NULL) continue;
-    if (strcmp(fCurrentSubsession->mediumName(), "audio") != 0) continue;
+    if (strcmp(subsession->mediumName(), "audio") != 0) continue;
 
     size += addAtom_trak();
 
@@ -1194,10 +1236,10 @@ addAtom(moov);
     }
   }
   iter.reset();
-  while ((fCurrentSubsession = iter.next()) != NULL) {
-    fCurrentIOState = (SubsessionIOState*)(fCurrentSubsession->miscPtr); 
+  while ((subsession = iter.next()) != NULL) {
+    fCurrentIOState = (SubsessionIOState*)(subsession->miscPtr); 
     if (fCurrentIOState == NULL) continue;
-    if (strcmp(fCurrentSubsession->mediumName(), "audio") == 0) continue;
+    if (strcmp(subsession->mediumName(), "audio") == 0) continue;
 
     size += addAtom_trak();
 
@@ -1234,6 +1276,13 @@ addAtom(mvhd);
   size += addWord(SubsessionIOState::fCurrentTrackNumber+1);// Next track ID
 addAtomEnd;
 
+addAtom(iods);
+  size += addWord(0x00000000); // Version + Flags
+  size += addWord(0x10808080);
+  size += addWord(0x07004FFF);
+  size += addWord(0xFF0FFFFF);
+addAtomEnd;
+
 addAtom(trak);
   size += addAtom_tkhd();
 
@@ -1275,8 +1324,12 @@ addAtom(tkhd);
   size += addWord(0x00010000); // matrix center
   size += addZeroWords(3); // matrix
   size += addWord(0x40000000); // matrix bottom right corner
-  size += addWord(fMovieWidth<<16); // Track width
-  size += addWord(fMovieHeight<<16); // Track height
+  if (strcmp(fCurrentIOState->fOurSubsession.mediumName(), "video") == 0) {
+    size += addWord(fMovieWidth<<16); // Track width
+    size += addWord(fMovieHeight<<16); // Track height
+  } else {
+    size += addZeroWords(2); // not video: leave width and height fields zero
+  }
 addAtomEnd;
 
 addAtom(edts);
@@ -1528,23 +1581,17 @@ unsigned QuickTimeFileSink::addAtom_soundMediaGeneral() {
     = (unsigned short)(fCurrentIOState->fOurSubsession.numChannels());
   size += addHalfWord(numChannels); // Number of channels
   size += addHalfWord(0x0010); // Sample size
-  size += addWord(0x00000000); // Compression ID+Packet size
+  //  size += addWord(0x00000000); // Compression ID+Packet size
+  size += addWord(0xfffe0000); // Compression ID+Packet size #####
 
-  unsigned const timeScale = fCurrentIOState->fQTTimeScale;
-  unsigned const timeUnitsPerSample
-    = fCurrentIOState->fQTTimeUnitsPerSample;
-  double const sampleRate = timeScale/(double)timeUnitsPerSample;
-  unsigned const sampleRateIntPart = (unsigned)sampleRate;
-  unsigned const sampleRateFracPart = (unsigned)(sampleRate*65536);
-  unsigned const sampleRateFixedPoint
-    = (sampleRateIntPart<<16) | sampleRateFracPart;
+  unsigned const sampleRateFixedPoint = fCurrentIOState->fQTTimeScale << 16;
   size += addWord(sampleRateFixedPoint); // Sample rate
 addAtomEnd;
 
 unsigned QuickTimeFileSink::addAtom_Qclp() {
   // The beginning of this atom looks just like a general Sound Media atom,
   // except with a version field of 1:
-  unsigned initFilePosn = ftell(fOutFid); \
+  unsigned initFilePosn = ftell(fOutFid);
   fCurrentIOState->fQTAudioDataType = "Qclp";
   fCurrentIOState->fQTSoundSampleVersion = 1;
   unsigned size = addAtom_soundMediaGeneral();
@@ -1552,8 +1599,8 @@ unsigned QuickTimeFileSink::addAtom_Qclp() {
   // Next, add the four fields that are particular to version 1:
   // (Later, parameterize these #####)
   size += addWord(0x000000a0); // samples per packet
-  size += addWord(fCurrentIOState->fQTBytesPerFrame); // bytes per packet
-  size += addWord(fCurrentIOState->fQTBytesPerFrame); // bytes per frame
+  size += addWord(0x00000000); // ???
+  size += addWord(0x00000000); // ???
   size += addWord(0x00000002); // bytes per sample (uncompressed)
 
   // Other special fields are in a 'wave' atom that follows:
@@ -1562,21 +1609,30 @@ addAtomEnd;
 
 addAtom(wave);
   size += addAtom_frma();
-  size += addWord(0x00000014); // ???
-  size += add4ByteString("Qclp"); // ???
-  if (fCurrentIOState->fQTBytesPerFrame == 35) {
-    size += addAtom_Fclp(); // full-rate QCELP
-  } else {
-    size += addAtom_Hclp(); // half-rate QCELP
-  } // what about other QCELP 'rates'??? #####
-  size += addWord(0x00000008); // ???
-  size += addWord(0x00000000); // ???
-  size += addWord(0x00000000); // ???
-  size += addWord(0x00000008); // ???
+  if (strcmp(fCurrentIOState->fQTAudioDataType, "Qclp") == 0) {
+    size += addWord(0x00000014); // ???
+    size += add4ByteString("Qclp"); // ???
+    if (fCurrentIOState->fQTBytesPerFrame == 35) {
+      size += addAtom_Fclp(); // full-rate QCELP
+    } else {
+      size += addAtom_Hclp(); // half-rate QCELP
+    } // what about other QCELP 'rates'??? #####
+    size += addWord(0x00000008); // ???
+    size += addWord(0x00000000); // ???
+    size += addWord(0x00000000); // ???
+    size += addWord(0x00000008); // ???
+  } else if (strcmp(fCurrentIOState->fQTAudioDataType, "mp4a") == 0) {
+    size += addWord(0x0000000c); // ???
+    size += add4ByteString("mp4a"); // ???
+    size += addWord(0x00000000); // ???
+    size += addAtom_esds(); // ESDescriptor
+    size += addWord(0x00000008); // ???
+    size += addWord(0x00000000); // ???
+  }
 addAtomEnd;
 
 addAtom(frma);
-  size += add4ByteString("Qclp"); // ???
+  size += add4ByteString(fCurrentIOState->fQTAudioDataType); // ???
 addAtomEnd;
 
 addAtom(Fclp);
@@ -1587,6 +1643,82 @@ addAtom(Hclp);
  size += addWord(0x00000000); // ???
 addAtomEnd;
 
+unsigned QuickTimeFileSink::addAtom_mp4a() {
+  // The beginning of this atom looks just like a general Sound Media atom,
+  // except with a version field of 1:
+  unsigned initFilePosn = ftell(fOutFid);
+  fCurrentIOState->fQTAudioDataType = "mp4a";
+  fCurrentIOState->fQTSoundSampleVersion = 1;
+  unsigned size = addAtom_soundMediaGeneral();
+
+  if (fGenerateMP4Format) {
+    size += addAtom_esds();
+  } else {
+    // Next, add the four fields that are particular to version 1:
+    // (Later, parameterize these #####)
+    size += addWord(fCurrentIOState->fQTTimeUnitsPerSample);
+    size += addWord(0x00000001); // ???
+    size += addWord(0x00000001); // ???
+    size += addWord(0x00000002); // bytes per sample (uncompressed)
+
+    // Other special fields are in a 'wave' atom that follows:
+    size += addAtom_wave();
+  }
+addAtomEnd;
+
+addAtom(esds);
+  //#####
+  MediaSubsession& subsession = fCurrentIOState->fOurSubsession;
+  if (strcmp(subsession.mediumName(), "audio") == 0) {
+    // MPEG-4 audio
+    size += addWord(0x00000000); // ???
+    size += addWord(0x03808080); // ???
+    size += addWord(0x2a000000); // ???
+    size += addWord(0x04808080); // ???
+    size += addWord(0x1c401500); // ???
+    size += addWord(0x18000000); // ???
+    size += addWord(0x6d600000); // ???
+    size += addWord(0x6d600580); // ???
+    size += addByte(0x80); size += addByte(0x80); // ???
+  } else if (strcmp(subsession.mediumName(), "video") == 0) {
+    // MPEG-4 video
+    size += addWord(0x00000000); // ???
+    size += addWord(0x03370000); // ???
+    size += addWord(0x1f042f20); // ???
+    size += addWord(0x1104fd46); // ???
+    size += addWord(0x000d4e10); // ???
+    size += addWord(0x000d4e10); // ???
+    size += addByte(0x05); // ???
+  }
+
+  // Add the source's 'config' information:
+  unsigned configSize;
+  unsigned char* config
+    = parseGeneralConfigStr(subsession.fmtp_config(), configSize);
+  if (configSize > 0) --configSize; // remove trailing '\0';
+  size += addByte(configSize);
+  for (unsigned i = 0; i < configSize; ++i) {
+    size += addByte(config[i]);
+  }
+
+  if (strcmp(subsession.mediumName(), "audio") == 0) {
+    // MPEG-4 audio
+    size += addWord(0x06808080); // ???
+    size += addByte(0x01); // ???
+  } else {
+    // MPEG-4 video
+    size += addHalfWord(0x0601); // ???
+    size += addByte(0x02); // ???
+  }
+  //#####
+addAtomEnd;
+
+addAtom(srcq);
+  //#####
+  size += addWord(0x00000040); // ???
+  //#####
+addAtomEnd;
+
 addAtom(h263);
 // General sample description fields:
   size += addWord(0x00000000); // Reserved
@@ -1603,12 +1735,38 @@ addAtom(h263);
   size += addWord(0x00000000); // Data size
   size += addWord(0x00010548); // Frame count+Compressor name (start)
     // "H.263"
-  size += addWord(0x2e323633); // Frame count+Compressor name (cont)
+  size += addWord(0x2e323633); // Compressor name (continued)
   size += addZeroWords(6); // Compressor name (continued - zero)
   size += addWord(0x00000018); // Compressor name (final)+Depth
   size += addHalfWord(0xffff); // Color table id
 addAtomEnd;
 
+addAtom(mp4v);
+// General sample description fields:
+  size += addWord(0x00000000); // Reserved
+  size += addWord(0x00000001); // Reserved+Data reference index
+// Video sample description fields:
+  size += addWord(0x00020001); // Version+Revision level
+  size += add4ByteString("appl"); // Vendor
+  size += addWord(0x00000200); // Temporal quality
+  size += addWord(0x00000400); // Spatial quality
+  unsigned const widthAndHeight = (fMovieWidth<<16)|fMovieHeight;
+  size += addWord(widthAndHeight); // Width+height
+  size += addWord(0x00480000); // Horizontal resolution
+  size += addWord(0x00480000); // Vertical resolution
+  size += addWord(0x00000000); // Data size
+  size += addWord(0x00010c4d); // Frame count+Compressor name (start)
+    // "MPEG-4 Video"
+  size += addWord(0x5045472d); // Compressor name (continued)
+  size += addWord(0x34205669); // Compressor name (continued)
+  size += addWord(0x64656f00); // Compressor name (continued)
+  size += addZeroWords(4); // Compressor name (continued - zero)
+  size += addWord(0x00000018); // Compressor name (final)+Depth
+  size += addHalfWord(0xffff); // Color table id
+  size += addAtom_esds(); // ESDescriptor
+  size += addWord(0x00000000); // ???
+addAtomEnd;
+
 unsigned QuickTimeFileSink::addAtom_rtp() {
   unsigned initFilePosn = ftell(fOutFid);
   unsigned size = addAtomHeader("rtp ");
