@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2002 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // RTCP
 // Implementation
 
@@ -24,16 +24,11 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 ////////// RTCPMemberDatabase //////////
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 class RTCPMemberDatabase {
 public:
-  RTCPMemberDatabase()
-    : fNumMembers(1 /*ourself*/), fTable(HashTable::create(ONE_WORD_HASH_KEYS)) {
+  RTCPMemberDatabase(RTCPInstance& ourRTCPInstance)
+    : fOurRTCPInstance(ourRTCPInstance), fNumMembers(1 /*ourself*/),
+      fTable(HashTable::create(ONE_WORD_HASH_KEYS)) {
   }
 
   virtual ~RTCPMemberDatabase() {
@@ -72,6 +67,7 @@ public:
   void reapOldMembers(unsigned threshold);
 
 private:
+  RTCPInstance& fOurRTCPInstance;
   unsigned fNumMembers;
   HashTable* fTable;
 };
@@ -88,8 +84,8 @@ void RTCPMemberDatabase::reapOldMembers(unsigned threshold) {
     unsigned long timeCount;
     char const* key;
     while ((timeCount = (unsigned long)(iter->next(key))) != 0) {
-#ifdef DEBUG_PRINT
-      fprintf(stderr, "reap: checking SSRC 0x%x: %d (threshold %d)\n", (unsigned long)key, timeCount, threshold);
+#ifdef DEBUG
+      fprintf(stderr, "reap: checking SSRC 0x%lx: %ld (threshold %d)\n", (unsigned long)key, timeCount, threshold);
 #endif
       if (timeCount < (unsigned long)threshold) { // this SSRC is old
         unsigned long ssrc = (unsigned long)key;
@@ -97,12 +93,13 @@ void RTCPMemberDatabase::reapOldMembers(unsigned threshold) {
         foundOldMember = True;
       }
     }
+    delete iter;
 
     if (foundOldMember) {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
         fprintf(stderr, "reap: removing SSRC 0x%x\n", oldSSRC);
 #endif
-      remove(oldSSRC);
+      fOurRTCPInstance.removeSSRC(oldSSRC, True);
     }
   } while (foundOldMember);
 }
@@ -112,7 +109,7 @@ void RTCPMemberDatabase::reapOldMembers(unsigned threshold) {
 
 static double dTimeNow() {
     struct timeval timeNow;
-    gettimeofday(&timeNow, &Idunno);
+    gettimeofday(&timeNow, NULL);
     return (double) (timeNow.tv_sec + timeNow.tv_usec/1000000.0);
 }
 
@@ -123,7 +120,7 @@ static unsigned const preferredPacketSize = 1000; // bytes
 RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
 			   unsigned totSessionBW,
 			   unsigned char const* cname,
-			   RTPSink const* sink, RTPSource const* source,
+			   RTPSink* sink, RTPSource const* source,
 			   Boolean isSSMSource)
   : Medium(env), fRTCPInterface(this, RTCPgs), fTotSessionBW(totSessionBW),
     fSink(sink), fSource(source), fIsSSMSource(isSSMSource),
@@ -132,20 +129,34 @@ RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
     fLastSentSize(0), fLastReceivedSize(0), fLastReceivedSSRC(0),
     fTypeOfEvent(EVENT_UNKNOWN), fTypeOfPacket(PACKET_UNKNOWN_TYPE),
     fHaveJustSentPacket(False), fLastPacketSentSize(0),
-    fByeHandlerTask(NULL), fByeHandlerClientData(NULL) {
-#ifdef DEBUG_PRINT
+    fByeHandlerTask(NULL), fByeHandlerClientData(NULL),
+    fSRHandlerTask(NULL), fSRHandlerClientData(NULL),
+    fRRHandlerTask(NULL), fRRHandlerClientData(NULL),
+    fSpecificRRHandlerTable(NULL) {
+#ifdef DEBUG
   fprintf(stderr, "RTCPInstance[%p]::RTCPInstance()\n", this);
 #endif
+  if (fTotSessionBW == 0) { // not allowed!
+    env << "RTCPInstance::RTCPInstance error: totSessionBW parameter should not be zero!\n";
+    fTotSessionBW = 1;
+  }
+
   if (isSSMSource) RTCPgs->multicastSendOnly(); // don't receive multicast
     
   double timeNow = dTimeNow();
   fPrevReportTime = fNextReportTime = timeNow;
 
-  fKnownMembers = new RTCPMemberDatabase;
+  fKnownMembers = new RTCPMemberDatabase(*this);
   fInBuf = new unsigned char[maxPacketSize];
+  if (fKnownMembers == NULL || fInBuf == NULL) return;
+
+  // A hack to save buffer space, because RTCP packets are always small:
+  unsigned savedMaxSize = OutPacketBuffer::maxSize;
+  OutPacketBuffer::maxSize = maxPacketSize;
   fOutBuf = new OutPacketBuffer(preferredPacketSize, maxPacketSize);
-  if (fKnownMembers == NULL || fOutBuf == NULL) return;
-  
+  OutPacketBuffer::maxSize = savedMaxSize;
+  if (fOutBuf == NULL) return;
+
   // Arrange to handle incoming reports from others:
   TaskScheduler::BackgroundHandlerProc* handler
     = (TaskScheduler::BackgroundHandlerProc*)&incomingReportHandler;
@@ -156,8 +167,13 @@ RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
   onExpire(this);
 }
 
+struct RRHandlerRecord {
+  TaskFunc* rrHandlerTask;
+  void* rrHandlerClientData;
+};
+
 RTCPInstance::~RTCPInstance() {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
   fprintf(stderr, "RTCPInstance[%p]::~RTCPInstance()\n", this);
 #endif
   // Turn off background read handling:
@@ -168,6 +184,15 @@ RTCPInstance::~RTCPInstance() {
   fTypeOfEvent = EVENT_BYE; // not used, but...
   sendBYE();
 
+  if (fSpecificRRHandlerTable != NULL) {
+    AddressPortLookupTable::Iterator iter(*fSpecificRRHandlerTable);
+    RRHandlerRecord* rrHandler;
+    while ((rrHandler = (RRHandlerRecord*)iter.next()) != NULL) {
+      delete rrHandler;
+    }
+    delete fSpecificRRHandlerTable;
+  }
+
   delete fKnownMembers;
   delete fOutBuf;
   delete[] fInBuf;
@@ -176,8 +201,7 @@ RTCPInstance::~RTCPInstance() {
 RTCPInstance* RTCPInstance::createNew(UsageEnvironment& env, Groupsock* RTCPgs,
 				      unsigned totSessionBW,
 				      unsigned char const* cname,
-				      RTPSink const* sink,
-				      RTPSource const* source,
+				      RTPSink* sink, RTPSource const* source,
 				      Boolean isSSMSource) {
   return new RTCPInstance(env, RTCPgs, totSessionBW, cname, sink, source,
 			  isSSMSource);
@@ -210,9 +234,50 @@ unsigned RTCPInstance::numMembers() const {
   return fKnownMembers->numMembers();
 }
 
-void RTCPInstance::setByeHandler(TaskFunc* handlerTask, void* clientData) {
+void RTCPInstance::setByeHandler(TaskFunc* handlerTask, void* clientData,
+				 Boolean handleActiveParticipantsOnly) {
   fByeHandlerTask = handlerTask;
   fByeHandlerClientData = clientData;
+  fByeHandleActiveParticipantsOnly = handleActiveParticipantsOnly;
+}
+
+void RTCPInstance::setSRHandler(TaskFunc* handlerTask, void* clientData) {
+  fSRHandlerTask = handlerTask;
+  fSRHandlerClientData = clientData;
+}
+
+void RTCPInstance::setRRHandler(TaskFunc* handlerTask, void* clientData) {
+  fRRHandlerTask = handlerTask;
+  fRRHandlerClientData = clientData;
+}
+
+void RTCPInstance
+::setSpecificRRHandler(netAddressBits fromAddress, Port fromPort,
+		       TaskFunc* handlerTask, void* clientData) {
+  if (handlerTask == NULL && clientData == NULL) {
+    unsetSpecificRRHandler(fromAddress, fromPort);
+    return;
+  }
+
+  RRHandlerRecord* rrHandler = new RRHandlerRecord;
+  rrHandler->rrHandlerTask = handlerTask;
+  rrHandler->rrHandlerClientData = clientData;
+  if (fSpecificRRHandlerTable == NULL) {
+    fSpecificRRHandlerTable = new AddressPortLookupTable;
+  }
+  fSpecificRRHandlerTable->Add(fromAddress, (~0), fromPort, rrHandler);
+}
+
+void RTCPInstance
+::unsetSpecificRRHandler(netAddressBits fromAddress, Port fromPort) {
+  if (fSpecificRRHandlerTable == NULL) return;
+
+  RRHandlerRecord* rrHandler
+    = (RRHandlerRecord*)(fSpecificRRHandlerTable->Lookup(fromAddress, (~0), fromPort));
+  if (rrHandler != NULL) {
+    fSpecificRRHandlerTable->Remove(fromAddress, (~0), fromPort);
+    delete rrHandler;
+  }
 }
 
 void RTCPInstance::setStreamSocket(int sockNum,
@@ -229,6 +294,17 @@ void RTCPInstance::setStreamSocket(int sockNum,
   fRTCPInterface.startNetworkReading(handler);
 }
 
+void RTCPInstance::addStreamSocket(int sockNum,
+				   unsigned char streamChannelId) {
+  // Add the RTCP-over-TCP interface:
+  fRTCPInterface.setStreamSocket(sockNum, streamChannelId);
+
+  // Turn on background reading for this socket (in case it's not on already):
+  TaskScheduler::BackgroundHandlerProc* handler
+    = (TaskScheduler::BackgroundHandlerProc*)&incomingReportHandler;
+  fRTCPInterface.startNetworkReading(handler);
+}
+
 static unsigned const IP_UDP_HDR_SIZE = 28;
     // overhead (bytes) of IP and UDP hdrs
 
@@ -246,6 +322,8 @@ void RTCPInstance::incomingReportHandler1() {
   int typeOfPacket = PACKET_UNKNOWN_TYPE;
 
   do {
+    int tcpReadStreamSocketNum = fRTCPInterface.nextTCPReadStreamSocketNum();
+    unsigned char tcpReadStreamChannelId = fRTCPInterface.nextTCPReadStreamChannelId();
     if (!fRTCPInterface.handleRead(pkt, maxPacketSize,
 				   packetSize, fromAddress)) {
       break;
@@ -277,7 +355,7 @@ void RTCPInstance::incomingReportHandler1() {
       fLastPacketSentSize = packetSize;
     }
 
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
     fprintf(stderr, "[%p]saw incoming RTCP packet (from address %s, port %d)\n", this, our_inet_ntoa(fromAddress.sin_addr), ntohs(fromAddress.sin_port));
     unsigned char* p = pkt;
     for (unsigned i = 0; i < packetSize; ++i) {
@@ -295,7 +373,7 @@ void RTCPInstance::incomingReportHandler1() {
     if (packetSize < 4) break;
     unsigned rtcpHdr = ntohl(*(unsigned*)pkt);
     if ((rtcpHdr & 0xE0FE0000) != (0x80000000 | (RTCP_PT_SR<<16))) {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
       fprintf(stderr, "rejected bad RTCP packet: header 0x%08x\n", rtcpHdr);
 #endif
       break;
@@ -319,7 +397,7 @@ void RTCPInstance::incomingReportHandler1() {
       Boolean subPacketOK = False;
       switch (pt) {
         case RTCP_PT_SR: {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
 	  fprintf(stderr, "SR\n");
 #endif
 	  if (length < 20) break; length -= 20;
@@ -335,29 +413,91 @@ void RTCPInstance::incomingReportHandler1() {
 					  NTPmsw, NTPlsw, rtpTimestamp);
 	  }
 	  ADVANCE(8); // skip over packet count, octet count
+
+	  // If a 'SR handler' was set, call it now:
+	  if (fSRHandlerTask != NULL) (*fSRHandlerTask)(fSRHandlerClientData);
+
 	  // The rest of the SR is handled like a RR (so, no "break;" here)
 	}
         case RTCP_PT_RR: {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
 	  fprintf(stderr, "RR\n");
 #endif
 	  unsigned reportBlocksSize = rc*(6*4);
-	  if (length < reportBlocksSize) break; length -= reportBlocksSize;
-	  ADVANCE(reportBlocksSize);
-	  // later process these (including perhaps using the bw #s to #####
-	  // give us a more accurate estimate of the session bandwidth) #####
+	  if (length < reportBlocksSize) break;
+	  length -= reportBlocksSize;
+
+          if (fSink != NULL) {
+	    // Use this information to update stats about our transmissions:
+            RTPTransmissionStatsDB& transmissionStats = fSink->transmissionStatsDB();
+            for (unsigned i = 0; i < rc; ++i) {
+              unsigned senderSSRC = ntohl(*(unsigned*)pkt); ADVANCE(4);
+              // We care only about reports about our own transmission, not others'
+              if (senderSSRC == fSink->SSRC()) {
+                unsigned lossStats = ntohl(*(unsigned*)pkt); ADVANCE(4);
+                unsigned highestReceived = ntohl(*(unsigned*)pkt); ADVANCE(4);
+                unsigned jitter = ntohl(*(unsigned*)pkt); ADVANCE(4);
+                unsigned timeLastSR = ntohl(*(unsigned*)pkt); ADVANCE(4);
+                unsigned timeSinceLastSR = ntohl(*(unsigned*)pkt); ADVANCE(4);
+                transmissionStats.noteIncomingRR(reportSenderSSRC, fromAddress,
+						 lossStats,
+						 highestReceived, jitter,
+						 timeLastSR, timeSinceLastSR);
+              } else {
+                ADVANCE(4*5);
+              }
+            }
+          } else {
+            ADVANCE(reportBlocksSize);
+          }
+
+	  if (pt == RTCP_PT_RR) { // i.e., we didn't fall through from 'SR'
+	    // If a 'RR handler' was set, call it now:
+
+	    // Specific RR handler:
+	    if (fSpecificRRHandlerTable != NULL) {
+	      netAddressBits fromAddr;
+	      portNumBits fromPortNum;
+	      if (tcpReadStreamSocketNum < 0) {
+		// Normal case: We read the RTCP packet over UDP
+		fromAddr = fromAddress.sin_addr.s_addr;
+		fromPortNum = ntohs(fromAddress.sin_port);
+	      } else {
+		// Special case: We read the RTCP packet over TCP (interleaved)
+		// Hack: Use the TCP socket and channel id to look up the handler
+		fromAddr = tcpReadStreamSocketNum;
+		fromPortNum = tcpReadStreamChannelId;
+	      }
+	      Port fromPort(fromPortNum); 
+	      RRHandlerRecord* rrHandler
+		= (RRHandlerRecord*)(fSpecificRRHandlerTable->Lookup(fromAddr, (~0), fromPort));
+	      if (rrHandler != NULL) {
+		if (rrHandler->rrHandlerTask != NULL) {
+		  (*(rrHandler->rrHandlerTask))(rrHandler->rrHandlerClientData);
+		}
+	      }
+	    }
+
+	    // General RR handler:
+	    if (fRRHandlerTask != NULL) (*fRRHandlerTask)(fRRHandlerClientData);
+	  }
 
 	  subPacketOK = True;
 	  typeOfPacket = PACKET_RTCP_REPORT;
 	  break;
 	}
         case RTCP_PT_BYE: {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
 	  fprintf(stderr, "BYE\n");
 #endif
 	  // If a 'BYE handler' was set, call it now:
 	  TaskFunc* byeHandler = fByeHandlerTask;
-	  if (byeHandler != NULL) {
+	  if (byeHandler != NULL
+	      && (!fByeHandleActiveParticipantsOnly
+		  || (fSource != NULL
+		      && fSource->receptionStatsDB().lookup(reportSenderSSRC) != NULL)
+		  || (fSink != NULL
+		      && fSink->transmissionStatsDB().lookup(reportSenderSSRC) != NULL))) {
 	    fByeHandlerTask = NULL;
 	        // we call this only once by default 
 	    (*byeHandler)(fByeHandlerClientData);
@@ -371,7 +511,7 @@ void RTCPInstance::incomingReportHandler1() {
 	}
 	// Later handle SDES, APP, and compound RTCP packets #####
         default:
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
 	  fprintf(stderr, "UNSUPPORTED TYPE(0x%x)\n", pt);
 #endif
 	  subPacketOK = True;
@@ -381,7 +521,7 @@ void RTCPInstance::incomingReportHandler1() {
 
       // need to check for (& handle) SSRC collision! #####
 
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
       fprintf(stderr, "validated RTCP subpacket (type %d): %d, %d, %d, 0x%08x\n", typeOfPacket, rc, pt, length, reportSenderSSRC);
 #endif
       
@@ -393,14 +533,14 @@ void RTCPInstance::incomingReportHandler1() {
 	packetOK = True;
 	break;
       } else if (packetSize < 4) {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
 	fprintf(stderr, "extraneous %d bytes at end of RTCP packet!\n", packetSize);
 #endif
 	break;
       }
       rtcpHdr = ntohl(*(unsigned*)pkt);
       if ((rtcpHdr & 0xC0000000) != 0x80000000) {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
 	fprintf(stderr, "bad RTCP subpacket: header 0x%08x\n", rtcpHdr);
 #endif
 	break;
@@ -408,12 +548,12 @@ void RTCPInstance::incomingReportHandler1() {
     }
       
     if (!packetOK) {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
       fprintf(stderr, "rejected bad RTCP subpacket: header 0x%08x\n", rtcpHdr);
 #endif
       break;
     } else {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
       fprintf(stderr, "validated entire RTCP packet\n");
 #endif
     }
@@ -443,7 +583,10 @@ void RTCPInstance::onReceive(int typeOfPacket, int totPacketSize,
 }
 
 void RTCPInstance::sendReport() {
-#ifdef DEBUG_PRINT
+  /// Note: Don't send a SR until at least one RTP packet has been sent. (David Bertrand, 2006.07.18)
+  if (fSink != NULL && !fSink->haveComputedFirstTimestamp()) return;
+
+#ifdef DEBUG
   fprintf(stderr, "sending REPORT\n");
 #endif
   // Begin by including a SR and/or RR report:
@@ -464,7 +607,7 @@ void RTCPInstance::sendReport() {
 }
 
 void RTCPInstance::sendBYE() {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
   fprintf(stderr, "sending BYE\n");
 #endif
   // The packet must begin with a SR and/or RR report:
@@ -475,7 +618,7 @@ void RTCPInstance::sendBYE() {
 }
 
 void RTCPInstance::sendBuiltPacket() {
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
   fprintf(stderr, "sending RTCP packet\n");
   unsigned char* p = fOutBuf->packet();
   for (unsigned i = 0; i < fOutBuf->curPacketSize(); ++i) {
@@ -498,8 +641,18 @@ int RTCPInstance::checkNewSSRC() {
 				       fOutgoingReportCount);
 }
 
-void RTCPInstance::removeSSRC() {
-  fKnownMembers->remove(fLastReceivedSSRC);
+void RTCPInstance::removeLastReceivedSSRC() {
+  removeSSRC(fLastReceivedSSRC, False/*keep stats around*/);
+}
+
+void RTCPInstance::removeSSRC(u_int32_t ssrc, Boolean alsoRemoveStats) {
+  fKnownMembers->remove(ssrc);
+
+  if (alsoRemoveStats) {
+    // Also, remove records of this SSRC from any reception or transmission stats
+    if (fSource != NULL) fSource->receptionStatsDB().removeRecord(ssrc);
+    if (fSink != NULL) fSink->transmissionStatsDB().removeRecord(ssrc);
+  }
 }
 
 void RTCPInstance::onExpire(RTCPInstance* instance) {
@@ -528,7 +681,7 @@ void RTCPInstance::addSR() {
 
   // Insert the NTP and RTP timestamps for the 'wallclock time':
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   fOutBuf->enqueueWord(timeNow.tv_sec + 0x83AA7E80);
       // NTP timestamp most-significant word (1970 epoch -> 1900 epoch)
   double fractionalPart = (timeNow.tv_usec/15625.0)*0x04000000; // 2^32/10^6
@@ -635,7 +788,7 @@ RTCPInstance::enqueueReportBlock(RTPReceptionStats* stats) {
   // Figure out how long has elapsed since the last SR rcvd from this src:
   struct timeval const& LSRtime = stats->lastReceivedSR_time(); // "last SR"
   struct timeval timeNow, timeSinceLSR;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   if (timeNow.tv_usec < LSRtime.tv_usec) {
     timeNow.tv_usec += 1000000;
     timeNow.tv_sec -= 1;
@@ -702,7 +855,7 @@ void RTCPInstance::schedule(double nextTime) {
   fNextReportTime = nextTime;
 
   double secondsToDelay = nextTime - dTimeNow();
-#ifdef DEBUG_PRINT
+#ifdef DEBUG
   fprintf(stderr, "schedule(%f->%f)\n", secondsToDelay, nextTime);
 #endif
   int usToGo = (int)(secondsToDelay * 1000000);
@@ -832,7 +985,7 @@ extern "C" void RemoveMember(packet p) {
   RTCPInstance* instance = (RTCPInstance*)p;
   if (instance == NULL) return;
 
-  instance->removeSSRC();
+  instance->removeLastReceivedSSRC();
 }
 
 extern "C" void RemoveSender(packet /*p*/) {
