@@ -14,14 +14,13 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // RTP Sinks
 // Implementation
 
 #include "RTPSink.hh"
 #include "GroupsockHelper.hh"
 
-//#define DEBUG_RR 1 //#####@@@@@
 ////////// RTPSink //////////
 
 Boolean RTPSink::lookupByName(UsageEnvironment& env, char const* sinkName,
@@ -44,12 +43,6 @@ Boolean RTPSink::isRTPSink() const {
   return True;
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 RTPSink::RTPSink(UsageEnvironment& env,
 		 Groupsock* rtpGS, unsigned char rtpPayloadType,
 		 unsigned rtpTimestampFrequency,
@@ -58,45 +51,56 @@ RTPSink::RTPSink(UsageEnvironment& env,
   : MediaSink(env), fRTPInterface(this, rtpGS),
     fRTPPayloadType(rtpPayloadType),
     fPacketCount(0), fOctetCount(0), fTotalOctetCount(0),
-    fTimestampFrequency(rtpTimestampFrequency),
-    fRTPPayloadFormatName(strDup(rtpPayloadFormatName)),
+    fTimestampFrequency(rtpTimestampFrequency), fHaveComputedFirstTimestamp(False),
     fNumChannels(numChannels) {
-  gettimeofday(&fCreationTime, &Idunno);
+  fRTPPayloadFormatName
+    = strDup(rtpPayloadFormatName == NULL ? "???" : rtpPayloadFormatName);
+  gettimeofday(&fCreationTime, NULL);
   fTotalOctetCountStartTime = fCreationTime;
 
-  fSeqNo = (unsigned short)our_random();
-  fSSRC = (unsigned)our_random();
-  fTimestampBase = (unsigned)our_random();
+  fSeqNo = (u_int16_t)our_random();
+  fSSRC = our_random32();
+  fTimestampBase = our_random32();
   fCurrentTimestamp = fTimestampBase;
 
   fTransmissionStatsDB = new RTPTransmissionStatsDB(*this);
 }
 
 RTPSink::~RTPSink() {
+  delete fTransmissionStatsDB;
   delete[] (char*)fRTPPayloadFormatName;
 }
 
-unsigned RTPSink::convertToRTPTimestamp(struct timeval timestamp) const {
-  unsigned rtpTimestamp = fTimestampBase;
-  rtpTimestamp += (fTimestampFrequency*timestamp.tv_sec);
-  rtpTimestamp += (unsigned)
-    ((2.0*fTimestampFrequency*timestamp.tv_usec + 1000000.0)/2000000);
-       // note: rounding
+u_int32_t RTPSink::convertToRTPTimestamp(struct timeval tv) {
+  u_int32_t rtpTimestampIncrement = timevalToTimestamp(tv);
 
+  if (!fHaveComputedFirstTimestamp) {
+    // Make the first timestamp the same as the current "fTimestampBase", so that
+    // timestamps begin with the value we promised when this "RTPSink" was created:
+    fTimestampBase -= rtpTimestampIncrement;
+    fHaveComputedFirstTimestamp = True;
+  }
+
+  u_int32_t const rtpTimestamp = fTimestampBase + rtpTimestampIncrement;
 #ifdef DEBUG_TIMESTAMPS
-  fprintf(stderr, "timestamp base: %u, presentationTime: %u.%06u,\n\tRTP timestamp: 0x%08x\n",
-	  fTimestampBase, timestamp.tv_sec,
-	  timestamp.tv_usec, rtpTimestamp);
+  fprintf(stderr, "fTimestampBase: 0x%08x, tv: %lu.%06ld\n\t=> RTP timestamp: 0x%08x\n",
+	  fTimestampBase, tv.tv_sec, tv.tv_usec, rtpTimestamp);
   fflush(stderr);
 #endif
 
   return rtpTimestamp;
 }
 
-void RTPSink::getTotalBitrate(unsigned& outNumBytes,
-			      double& outElapsedTime) {
+u_int32_t RTPSink::timevalToTimestamp(struct timeval tv) const {
+  u_int32_t timestamp = (fTimestampFrequency*tv.tv_sec);
+  timestamp += (u_int32_t)((2.0*fTimestampFrequency*tv.tv_usec + 1000000.0)/2000000);
+       // note: rounding
+  return timestamp;
+}
+
+void RTPSink::getTotalBitrate(unsigned& outNumBytes, double& outElapsedTime) {
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
 
   outNumBytes = fTotalOctetCount;
   outElapsedTime = (double)(timeNow.tv_sec-fTotalOctetCountStartTime.tv_sec)
@@ -111,6 +115,32 @@ char const* RTPSink::sdpMediaType() const {
   // default SDP media (m=) type, unless redefined by subclasses
 }
 
+char* RTPSink::rtpmapLine() const {
+  if (rtpPayloadType() >= 96) { // the payload format type is dynamic
+    char* encodingParamsPart;
+    if (numChannels() != 1) {
+      encodingParamsPart = new char[1 + 20 /* max int len */];
+      sprintf(encodingParamsPart, "/%d", numChannels());
+    } else {
+      encodingParamsPart = strDup("");
+    }
+    char const* const rtpmapFmt = "a=rtpmap:%d %s/%d%s\r\n";
+    unsigned rtpmapFmtSize = strlen(rtpmapFmt)
+      + 3 /* max char len */ + strlen(rtpPayloadFormatName())
+      + 20 /* max int len */ + strlen(encodingParamsPart);
+    char* rtpmapLine = new char[rtpmapFmtSize];
+    sprintf(rtpmapLine, rtpmapFmt,
+	    rtpPayloadType(), rtpPayloadFormatName(),
+	    rtpTimestampFrequency(), encodingParamsPart);
+    delete[] encodingParamsPart;
+
+    return rtpmapLine;
+  } else {
+    // The payload format is staic, so there's no "a=rtpmap:" line:
+    return strDup("");
+  }
+}
+
 char const* RTPSink::auxSDPLine() {
   return NULL; // by default
 }
@@ -136,7 +166,7 @@ RTPTransmissionStatsDB::~RTPTransmissionStatsDB() {
 }
 
 void RTPTransmissionStatsDB
-::noteIncomingRR(unsigned SSRC,
+::noteIncomingRR(u_int32_t SSRC, struct sockaddr_in const& lastFromAddress,
                  unsigned lossStats, unsigned lastPacketNumReceived,
                  unsigned jitter, unsigned lastSRTime, unsigned diffSR_RRTime) {
   RTPTransmissionStats* stats = lookup(SSRC);
@@ -151,10 +181,21 @@ void RTPTransmissionStatsDB
 #endif
   }
 
-  stats->noteIncomingRR(lossStats, lastPacketNumReceived, jitter,
+  stats->noteIncomingRR(lastFromAddress,
+			lossStats, lastPacketNumReceived, jitter,
                         lastSRTime, diffSR_RRTime);
 }
 
+void RTPTransmissionStatsDB::removeRecord(u_int32_t SSRC) {
+  RTPTransmissionStats* stats = lookup(SSRC);
+  if (stats != NULL) {
+    long SSRC_long = (long)SSRC;
+    fTable->Remove((char const*)SSRC_long);
+    --fNumReceivers;
+    delete stats;
+  }
+}
+
 RTPTransmissionStatsDB::Iterator
 ::Iterator(RTPTransmissionStatsDB& receptionStatsDB)
   : fIter(HashTable::Iterator::create(*(receptionStatsDB.fTable))) {
@@ -171,12 +212,12 @@ RTPTransmissionStatsDB::Iterator::next() {
   return (RTPTransmissionStats*)(fIter->next(key));
 }
 
-RTPTransmissionStats* RTPTransmissionStatsDB::lookup(unsigned SSRC) const {
+RTPTransmissionStats* RTPTransmissionStatsDB::lookup(u_int32_t SSRC) const {
   long SSRC_long = (long)SSRC;
   return (RTPTransmissionStats*)(fTable->Lookup((char const*)SSRC_long));
 }
 
-void RTPTransmissionStatsDB::add(unsigned SSRC, RTPTransmissionStats* stats) {
+void RTPTransmissionStatsDB::add(u_int32_t SSRC, RTPTransmissionStats* stats) {
   long SSRC_long = (long)SSRC;
   fTable->Add((char const*)SSRC_long, stats);
   ++fNumReceivers;
@@ -185,20 +226,25 @@ void RTPTransmissionStatsDB::add(unsigned SSRC, RTPTransmissionStats* stats) {
 
 ////////// RTPTransmissionStats //////////
 
-RTPTransmissionStats::RTPTransmissionStats(RTPSink& rtpSink, unsigned SSRC)
+RTPTransmissionStats::RTPTransmissionStats(RTPSink& rtpSink, u_int32_t SSRC)
   : fOurRTPSink(rtpSink), fSSRC(SSRC), fLastPacketNumReceived(0),
     fPacketLossRatio(0), fTotNumPacketsLost(0), fJitter(0),
-    fLastSRTime(0), fDiffSR_RRTime(0),
-    fFirstPacket(True) {
+    fLastSRTime(0), fDiffSR_RRTime(0), fFirstPacket(True),
+    fTotalOctetCount_hi(0), fTotalOctetCount_lo(0), 
+    fTotalPacketCount_hi(0), fTotalPacketCount_lo(0) {
+  gettimeofday(&fTimeCreated, NULL);
+
+  fLastOctetCount = rtpSink.octetCount();
+  fLastPacketCount = rtpSink.packetCount();
 }
 
 RTPTransmissionStats::~RTPTransmissionStats() {}
 
-void RTPTransmissionStats::noteIncomingRR(unsigned lossStats,
-					  unsigned lastPacketNumReceived,
-					  unsigned jitter,
-					  unsigned lastSRTime,
-					  unsigned diffSR_RRTime) {
+void RTPTransmissionStats
+::noteIncomingRR(struct sockaddr_in const& lastFromAddress,
+		 unsigned lossStats, unsigned lastPacketNumReceived,
+		 unsigned jitter, unsigned lastSRTime,
+		 unsigned diffSR_RRTime) {
   if (fFirstPacket) {
     fFirstPacket = False;
     fFirstPacketNumReported = lastPacketNumReceived;
@@ -207,8 +253,9 @@ void RTPTransmissionStats::noteIncomingRR(unsigned lossStats,
     fOldLastPacketNumReceived = fLastPacketNumReceived;
     fOldTotNumPacketsLost = fTotNumPacketsLost;
   }
-  gettimeofday(&fTimeReceived, &Idunno);
+  gettimeofday(&fTimeReceived, NULL);
 
+  fLastFromAddress = lastFromAddress;
   fPacketLossRatio = lossStats>>24;
   fTotNumPacketsLost = lossStats&0xFFFFFF;
   fLastPacketNumReceived = lastPacketNumReceived;
@@ -221,6 +268,26 @@ void RTPTransmissionStats::noteIncomingRR(unsigned lossStats,
   unsigned rtd = roundTripDelay(); 
   fprintf(stderr, "=> round-trip delay: 0x%04x (== %f seconds)\n", rtd, rtd/65536.0);
 #endif
+
+  // Update our counts of the total number of octets and packets sent towards
+  // this receiver:
+  u_int32_t newOctetCount = fOurRTPSink.octetCount();
+  u_int32_t octetCountDiff = newOctetCount - fLastOctetCount;
+  fLastOctetCount = newOctetCount;
+  u_int32_t prevTotalOctetCount_lo = fTotalOctetCount_lo;
+  fTotalOctetCount_lo += octetCountDiff;
+  if (fTotalOctetCount_lo < prevTotalOctetCount_lo) { // wrap around
+    ++fTotalOctetCount_hi;
+  }
+
+  u_int32_t newPacketCount = fOurRTPSink.packetCount();
+  u_int32_t packetCountDiff = newPacketCount - fLastPacketCount;
+  fLastPacketCount = newPacketCount;
+  u_int32_t prevTotalPacketCount_lo = fTotalPacketCount_lo;
+  fTotalPacketCount_lo += packetCountDiff;
+  if (fTotalPacketCount_lo < prevTotalPacketCount_lo) { // wrap around
+    ++fTotalPacketCount_hi;
+  }
 }
 
 unsigned RTPTransmissionStats::roundTripDelay() const {
@@ -241,7 +308,23 @@ unsigned RTPTransmissionStats::roundTripDelay() const {
   unsigned lastReceivedTimeNTP
     = (unsigned)((lastReceivedTimeNTP_high<<16) + fractionalPart + 0.5);
 
-  return lastReceivedTimeNTP - fLastSRTime - fDiffSR_RRTime; 
+  int rawResult = lastReceivedTimeNTP - fLastSRTime - fDiffSR_RRTime;
+  if (rawResult < 0) {
+    // This can happen if there's clock drift between the sender and receiver,
+    // and if the round-trip time was very small.
+    rawResult = 0;
+  }
+  return (unsigned)rawResult;
+}
+
+void RTPTransmissionStats::getTotalOctetCount(u_int32_t& hi, u_int32_t& lo) {
+  hi = fTotalOctetCount_hi;
+  lo = fTotalOctetCount_lo;
+}
+
+void RTPTransmissionStats::getTotalPacketCount(u_int32_t& hi, u_int32_t& lo) {
+  hi = fTotalPacketCount_hi;
+  lo = fTotalPacketCount_lo;
 }
 
 unsigned RTPTransmissionStats::packetsReceivedSinceLastRR() const {
