@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A simple RTP sink that packs frames into each outgoing
 //     packet, without any fragmentation or special headers.
 // Implementation
@@ -32,10 +32,11 @@ SimpleRTPSink::SimpleRTPSink(UsageEnvironment& env, Groupsock* RTPgs,
   : MultiFramedRTPSink(env, RTPgs, rtpPayloadFormat,
 		       rtpTimestampFrequency, rtpPayloadFormatName,
 		       numChannels),
-    fSDPMediaTypeString(strDup(sdpMediaTypeString)),
     fAllowMultipleFramesPerPacket(allowMultipleFramesPerPacket) {
+  fSDPMediaTypeString
+    = strDup(sdpMediaTypeString == NULL ? "unknown" : sdpMediaTypeString);
   fSetMBitOnLastFrames
-    = strcmp(sdpMediaTypeString, "video") == 0 && doNormalMBitRule;
+    = strcmp(fSDPMediaTypeString, "video") == 0 && doNormalMBitRule;
 }
 
 SimpleRTPSink::~SimpleRTPSink() {
@@ -79,8 +80,8 @@ void SimpleRTPSink::doSpecialFrameHandling(unsigned fragmentationOffset,
 }
 
 Boolean SimpleRTPSink::
-frameCanAppearAfterPacketStart(unsigned char const* frameStart,
-			       unsigned numBytesInFrame) const {
+frameCanAppearAfterPacketStart(unsigned char const* /*frameStart*/,
+			       unsigned /*numBytesInFrame*/) const {
   return fAllowMultipleFramesPerPacket;
 }
 
