@@ -14,16 +14,12 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
 // A WAV audio file source
 // Implementation
 
-#if defined(__WIN32__) || defined(_WIN32)
-#include <io.h>
-#include <fcntl.h>
-#endif
-
 #include "WAVAudioFileSource.hh"
+#include "InputFile.hh"
 #include "GroupsockHelper.hh"
 
 ////////// WAVAudioFileSource //////////
@@ -31,33 +27,53 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 WAVAudioFileSource*
 WAVAudioFileSource::createNew(UsageEnvironment& env, char const* fileName) {
   do {
-    FILE* fid;
-
-    // Check for a special case file name: "stdin"
-    if (strcmp(fileName, "stdin") == 0) {
-      fid = stdin;
-#if defined(__WIN32__) || defined(_WIN32)
-      _setmode(_fileno(stdin), _O_BINARY); // convert to binary mode
-#endif
-    } else { 
-      fid = fopen(fileName, "rb");
-      if (fid == NULL) {
-	env.setResultMsg("unable to open file \"",fileName, "\"");
-	break;
-      }
-    }
+    FILE* fid = OpenInputFile(env, fileName);
+    if (fid == NULL) break;
 
     WAVAudioFileSource* newSource = new WAVAudioFileSource(env, fid);
     if (newSource != NULL && newSource->bitsPerSample() == 0) {
       // The WAV file header was apparently invalid.
-      delete newSource; newSource = NULL;
+      Medium::close(newSource);
+      break;
     }
+
+    newSource->fFileSize = (unsigned)GetFileSize(fileName, fid);
+
     return newSource;
   } while (0);
 
   return NULL;
 }
 
+unsigned WAVAudioFileSource::numPCMBytes() const {
+  if (fFileSize < fWAVHeaderSize) return 0;
+  return fFileSize - fWAVHeaderSize;
+}
+
+void WAVAudioFileSource::setScaleFactor(int scale) {
+  fScaleFactor = scale;
+
+  if (fScaleFactor < 0 && ftell(fFid) > 0) {
+    // Because we're reading backwards, seek back one sample, to ensure that
+    // (i)  we start reading the last sample before the start point, and
+    // (ii) we don't hit end-of-file on the first read.
+    int const bytesPerSample = (fNumChannels*fBitsPerSample)/8;
+    fseek(fFid, -bytesPerSample, SEEK_CUR);
+  }
+}
+
+void WAVAudioFileSource::seekToPCMByte(unsigned byteNumber) {
+  byteNumber += fWAVHeaderSize;
+  if (byteNumber > fFileSize) byteNumber = fFileSize;
+
+  fseek(fFid, byteNumber, SEEK_SET);
+}
+
+unsigned char WAVAudioFileSource::getAudioFormat() {
+  return fAudioFormat;
+}
+
+
 #define nextc fgetc(fid)
 #define ucEOF ((unsigned char)EOF)
 
@@ -85,7 +101,8 @@ static Boolean skipBytes(FILE* fid, int num) {
 
 WAVAudioFileSource::WAVAudioFileSource(UsageEnvironment& env, FILE* fid)
   : AudioInputDevice(env, 0, 0, 0, 0)/* set the real parameters later */,
-    fFid(fid), fLastPlayTime(0) {
+    fFid(fid), fLastPlayTime(0), fWAVHeaderSize(0), fFileSize(0), fScaleFactor(1),
+    fAudioFormat(WA_UNKNOWN) {
   // Check the WAV file header for validity.
   // Note: The following web pages contain info about the WAV format:
   // http://www.technology.niagarac.on.ca/courses/comp630/WavFileFormat.html
@@ -108,8 +125,11 @@ WAVAudioFileSource::WAVAudioFileSource(UsageEnvironment& env, FILE* fid)
     if (!get4Bytes(fid, formatLength)) break;
     unsigned short audioFormat;
     if (!get2Bytes(fid, audioFormat)) break;
-    if (audioFormat != 1) { // not PCM - we can't handle this
-      env.setResultMsg("Audio format is not PCM");
+
+    fAudioFormat = (unsigned char)audioFormat;
+    if (fAudioFormat != WA_PCM && fAudioFormat != WA_PCMA && fAudioFormat != WA_PCMU) { 
+      // not PCM/PCMU/PCMA - we can't handle this
+      env.setResultMsg("Audio format is not PCM/PCMU/PCMA");
       break;
     }
     unsigned short numChannels;
@@ -151,6 +171,7 @@ WAVAudioFileSource::WAVAudioFileSource(UsageEnvironment& env, FILE* fid)
     if (!skipBytes(fid, 4)) break;
 
     // The header is good; the remaining data are the sample bytes.
+    fWAVHeaderSize = ftell(fid);
     success = True;
   } while (0);
   
@@ -175,15 +196,11 @@ WAVAudioFileSource::WAVAudioFileSource(UsageEnvironment& env, FILE* fid)
 }
 
 WAVAudioFileSource::~WAVAudioFileSource() {
-  fclose(fFid);
+  CloseInputFile(fFid);
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
+// Note: We should change the following to use asynchronous file reading, #####
+// as we now do with ByteStreamFileSource. #####
 void WAVAudioFileSource::doGetNextFrame() {
   if (feof(fFid) || ferror(fFid)) {
     handleClosure(this);
@@ -197,12 +214,28 @@ void WAVAudioFileSource::doGetNextFrame() {
   }
   unsigned const bytesPerSample = (fNumChannels*fBitsPerSample)/8;
   unsigned bytesToRead = fMaxSize - fMaxSize%bytesPerSample;
-  fFrameSize = fread(fTo, 1, bytesToRead, fFid);
+  if (fScaleFactor == 1) {
+    // Common case - read samples in bulk:
+    fFrameSize = fread(fTo, 1, bytesToRead, fFid);
+  } else {
+    // We read every 'fScaleFactor'th sample:
+    fFrameSize = 0; 
+    while (bytesToRead > 0) {
+      size_t bytesRead = fread(fTo, 1, bytesPerSample, fFid);
+      if (bytesRead <= 0) break;
+      fTo += bytesRead;
+      fFrameSize += bytesRead;
+      bytesToRead -= bytesRead;
+
+      // Seek to the appropriate place for the next sample:
+      fseek(fFid, (fScaleFactor-1)*bytesPerSample, SEEK_CUR);
+    }
+  }
 
   // Set the 'presentation time' and 'duration' of this frame:
   if (fPresentationTime.tv_sec == 0 && fPresentationTime.tv_usec == 0) {
     // This is the first frame, so use the current time:
-    gettimeofday(&fPresentationTime, &Idunno);
+    gettimeofday(&fPresentationTime, NULL);
   } else {
     // Increment by the play time of the previous data:
     unsigned uSeconds	= fPresentationTime.tv_usec + fLastPlayTime;
