@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2004, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2006, Live Networks, Inc.  All rights reserved
 // A common framework, used for the "openRTSP" and "playSIP" applications
 // Implementation
 
@@ -21,7 +21,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #include "BasicUsageEnvironment.hh"
 #include "GroupsockHelper.hh"
 #ifdef SUPPORT_REAL_RTSP
-#include "../RealRTSP/include/RealRMFF.hh"
+#include "../RealRTSP/include/RealRTSP.hh"
 #endif
 
 #if defined(__WIN32__) || defined(_WIN32)
@@ -45,7 +45,6 @@ void signalHandlerShutdown(int sig);
 void checkForPacketArrival(void* clientData);
 void checkInterPacketGaps(void* clientData);
 void beginQOSMeasurement();
-Boolean setupDestinationRTSPServer();
 
 char const* progName;
 UsageEnvironment* env;
@@ -57,7 +56,10 @@ TaskToken interPacketGapCheckTimerTask = NULL;
 TaskToken qosMeasurementTimerTask = NULL;
 Boolean createReceivers = True;
 Boolean outputQuickTimeFile = False;
+Boolean generateMP4Format = False;
 QuickTimeFileSink* qtOut = NULL;
+Boolean outputAVIFile = False;
+AVIFileSink* aviOut = NULL;
 Boolean audioOnly = False;
 Boolean videoOnly = False;
 char const* singleMedium = NULL;
@@ -68,44 +70,43 @@ unsigned interPacketGapMaxTime = 0;
 unsigned totNumPacketsReceived = ~0; // used if checking inter-packet gaps
 Boolean playContinuously = False;
 int simpleRTPoffsetArg = -1;
+Boolean sendOptionsRequest = True;
 Boolean sendOptionsRequestOnly = False;
 Boolean oneFilePerFrame = False;
 Boolean notifyOnPacketArrival = False;
 Boolean streamUsingTCP = False;
+portNumBits tunnelOverHTTPPortNum = 0;
 char* username = NULL;
 char* password = NULL;
 char* proxyServerName = NULL;
 unsigned short proxyServerPortNum = 0;
 unsigned char desiredAudioRTPPayloadFormat = 0;
 char* mimeSubtype = NULL;
-unsigned short movieWidth = 240;
-unsigned short movieHeight = 180;
-unsigned movieFPS = 15;
+unsigned short movieWidth = 240; // default
+Boolean movieWidthOptionSet = False;
+unsigned short movieHeight = 180; // default
+Boolean movieHeightOptionSet = False;
+unsigned movieFPS = 15; // default
+Boolean movieFPSOptionSet = False;
 char* fileNamePrefix = "";
 unsigned fileSinkBufferSize = 20000;
 unsigned socketInputBufferSize = 0;
 Boolean packetLossCompensate = False;
 Boolean syncStreams = False;
 Boolean generateHintTracks = False;
-char* destRTSPURL = NULL;
 unsigned qosMeasurementIntervalMS = 0; // 0 means: Don't output QOS data
 unsigned statusCode = 0;
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
 struct timeval startTime;
 
 void usage() {
   *env << "Usage: " << progName
-       << " [-p <startPortNum>] [-r|-q] [-a|-v] [-V] [-e <endTime>] [-E <max-inter-packet-gap-time> [-c] [-s <offset>] [-n]"
-	   << (controlConnectionUsesTCP ? " [-t]" : "")
+       << " [-p <startPortNum>] [-r|-q|-4|-i] [-a|-v] [-V] [-e <endTime>] [-E <max-inter-packet-gap-time> [-c] [-s <offset>] [-n] [-O]"
+	   << (controlConnectionUsesTCP ? " [-t|-T <http-port>]" : "")
        << " [-u <username> <password>"
 	   << (allowProxyServers ? " [<proxy-server> [<proxy-server-port>]]" : "")
        << "]" << (supportCodecSelection ? " [-A <audio-codec-rtp-payload-format-code>|-D <mime-subtype-name>]" : "")
-       << " [-w <width> -h <height>] [-f <frames-per-second>] [-y] [-H] [-Q [<measurement-interval>]] [-F <filename-prefix>] [-b <file-sink-buffer-size>] [-B <input-socket-buffer-size>] [-m] <url> (or " << progName << " -o [-V] <url>)\n";
+       << " [-w <width> -h <height>] [-f <frames-per-second>] [-y] [-H] [-Q [<measurement-interval>]] [-F <filename-prefix>] [-b <file-sink-buffer-size>] [-B <input-socket-buffer-size>] [-I <input-interface-ip-address>] [-m] <url> (or " << progName << " -o [-V] <url>)\n";
   //##### Add "-R <dest-rtsp-url>" #####
   shutdown();
 }
@@ -117,7 +118,7 @@ int main(int argc, char** argv) {
 
   progName = argv[0];
 
-  gettimeofday(&startTime, &Idunno);
+  gettimeofday(&startTime, NULL);
 
 #ifdef USE_SIGNALS
   // Allow ourselves to be shut down gracefully by a SIGHUP or a SIGUSR1:
@@ -158,6 +159,28 @@ int main(int argc, char** argv) {
       break;
     }
 
+    case '4': { // output a 'mp4'-format file (to stdout)
+      outputQuickTimeFile = True;
+      generateMP4Format = True;
+      break;
+    }
+
+    case 'i': { // output an AVI file (to stdout)
+      outputAVIFile = True;
+      break;
+    }
+
+    case 'I': { // specify input interface... 
+      NetAddressList addresses(argv[2]);
+      if (addresses.numAddresses() == 0) {
+	*env << "Failed to find network address for \"" << argv[2] << "\"";
+	break;
+      }
+      ReceivingInterfaceAddr = *(unsigned*)(addresses.firstAddress()->data());
+      ++argv; --argc;
+      break;
+    }
+
     case 'a': { // receive/record an audio stream only
       audioOnly = True;
       singleMedium = "audio";
@@ -217,6 +240,11 @@ int main(int argc, char** argv) {
       break;
     }
 
+    case 'O': { // Don't send an "OPTIONS" request before "DESCRIBE"
+      sendOptionsRequest = False;
+      break;
+    }
+
     case 'o': { // Send only the "OPTIONS" request to the server
       sendOptionsRequestOnly = True;
       break;
@@ -242,6 +270,24 @@ int main(int argc, char** argv) {
       break;
     }
 
+    case 'T': {
+      // stream RTP and RTCP over a HTTP connection
+      if (controlConnectionUsesTCP) {
+	if (argc > 3 && argv[2][0] != '-') {
+	  // The next argument is the HTTP server port number:
+	  if (sscanf(argv[2], "%hu", &tunnelOverHTTPPortNum) == 1
+	      && tunnelOverHTTPPortNum > 0) {
+	    ++argv; --argc;
+	    break;
+	  }
+	}
+      }
+
+      // If we get here, the option was specified incorrectly:
+      usage();
+      break;
+    }
+
     case 'u': { // specify a username and password
       username = argv[2];
       password = argv[3];
@@ -280,26 +326,29 @@ int main(int argc, char** argv) {
       break;
     }
 
-    case 'w': { // specify a width (pixels) for an output QuickTime movie
+    case 'w': { // specify a width (pixels) for an output QuickTime or AVI movie
       if (sscanf(argv[2], "%hu", &movieWidth) != 1) {
 	usage();
       }
+      movieWidthOptionSet = True;
       ++argv; --argc;
       break;
     }
 
-    case 'h': { // specify a height (pixels) for an output QuickTime movie
+    case 'h': { // specify a height (pixels) for an output QuickTime or AVI movie
       if (sscanf(argv[2], "%hu", &movieHeight) != 1) {
 	usage();
       }
+      movieHeightOptionSet = True;
       ++argv; --argc;
       break;
     }
 
-    case 'f': { // specify a frame rate (per second) for an output QT movie
+    case 'f': { // specify a frame rate (per second) for an output QT or AVI movie
       if (sscanf(argv[2], "%u", &movieFPS) != 1) {
 	usage();
       }
+      movieFPSOptionSet = True;
       ++argv; --argc;
       break;
     }
@@ -357,12 +406,6 @@ int main(int argc, char** argv) {
       break;
     }
 
-    case 'R': { // inject received data into a RTSP server
-      destRTSPURL = argv[2];
-      ++argv; --argc;
-      break;
-    }
-
     default: {
       usage();
       break;
@@ -372,18 +415,43 @@ int main(int argc, char** argv) {
     ++argv; --argc;
   }
   if (argc != 2) usage();
-  if (!createReceivers && outputQuickTimeFile) {
-    *env << "The -r and -q flags cannot both be used!\n";
+  if (outputQuickTimeFile && outputAVIFile) {
+    *env << "The -i and -q (or -4) flags cannot both be used!\n";
     usage();
   }
-  if (destRTSPURL != NULL && (!createReceivers || outputQuickTimeFile)) {
-    *env << "The -R flag cannot be used with -r or -q!\n";
+  Boolean outputCompositeFile = outputQuickTimeFile || outputAVIFile;
+  if (!createReceivers && outputCompositeFile) {
+    *env << "The -r and -q (or -4 or -i) flags cannot both be used!\n";
     usage();
   }
+  if (outputCompositeFile && !movieWidthOptionSet) {
+    *env << "Warning: The -q, -4 or -i option was used, but not -w.  Assuming a video width of "
+	 << movieWidth << " pixels\n";
+  }
+  if (outputCompositeFile && !movieHeightOptionSet) {
+    *env << "Warning: The -q, -4 or -i option was used, but not -h.  Assuming a video height of "
+	 << movieHeight << " pixels\n";
+  }
+  if (outputCompositeFile && !movieFPSOptionSet) {
+    *env << "Warning: The -q, -4 or -i option was used, but not -f.  Assuming a video frame rate of "
+	 << movieFPS << " frames-per-second\n";
+  }
   if (audioOnly && videoOnly) {
     *env << "The -a and -v flags cannot both be used!\n";
     usage();
   }
+  if (sendOptionsRequestOnly && !sendOptionsRequest) {
+    *env << "The -o and -O flags cannot both be used!\n";
+    usage();
+  }
+  if (tunnelOverHTTPPortNum > 0) {
+    if (streamUsingTCP) {
+      *env << "The -t and -T flags cannot both be used!\n";
+      usage();
+    } else {
+      streamUsingTCP = True;
+    }
+  }
   if (!createReceivers && notifyOnPacketArrival) {
     *env << "Warning: Because we're not receiving stream data, the -n flag has no effect\n";
   }
@@ -404,18 +472,21 @@ int main(int argc, char** argv) {
     shutdown();
   }
 
-  // Begin by sending an "OPTIONS" command:
-  char* optionsResponse = getOptionsResponse(ourClient, url);
-  if (sendOptionsRequestOnly) {
-    if (optionsResponse == NULL) {
-      *env << clientProtocolName << " \"OPTIONS\" request failed: "
-	   << env->getResultMsg() << "\n";
-    } else {
-      *env << clientProtocolName << " \"OPTIONS\" request returned: "
-	   << optionsResponse << "\n";
-      delete[] optionsResponse;
+  if (sendOptionsRequest) {
+    // Begin by sending an "OPTIONS" command:
+    char* optionsResponse
+      = getOptionsResponse(ourClient, url, username, password);
+    if (sendOptionsRequestOnly) {
+      if (optionsResponse == NULL) {
+	*env << clientProtocolName << " \"OPTIONS\" request failed: "
+	     << env->getResultMsg() << "\n";
+      } else {
+	*env << clientProtocolName << " \"OPTIONS\" request returned: "
+	     << optionsResponse << "\n";
+      }
+      shutdown();
     }
-    shutdown();
+    delete[] optionsResponse;
   }
 
   // Open the URL, to get a SDP description:
@@ -526,28 +597,71 @@ int main(int argc, char** argv) {
     if (outputQuickTimeFile) {
       // Create a "QuickTimeFileSink", to write to 'stdout':
       qtOut = QuickTimeFileSink::createNew(*env, *session, "stdout",
+					   fileSinkBufferSize,
 					   movieWidth, movieHeight,
 					   movieFPS,
 					   packetLossCompensate,
 					   syncStreams,
-					   generateHintTracks);
+					   generateHintTracks,
+					   generateMP4Format);
       if (qtOut == NULL) {
-		*env << "Failed to create QuickTime file sink for stdout: " << env->getResultMsg();
-		shutdown();
+	*env << "Failed to create QuickTime file sink for stdout: " << env->getResultMsg();
+	shutdown();
       }
 
       qtOut->startPlaying(sessionAfterPlaying, NULL);
-    } else if (destRTSPURL != NULL) {
-      // Announce the session into a (separate) RTSP server,
-      // and create one or more "RTPTranslator"s to tie the source
-      // and destination together:
-      if (setupDestinationRTSPServer()) {
-		  *env << "Set up destination RTSP session for \"" << destRTSPURL << "\"\n";
-      } else {
-		  *env << "Failed to set up destination RTSP session for \"" << destRTSPURL
-			  << "\": " << env->getResultMsg() << "\n";
-		shutdown();
+    } else if (outputAVIFile) {
+      // Create an "AVIFileSink", to write to 'stdout':
+      aviOut = AVIFileSink::createNew(*env, *session, "stdout",
+				      fileSinkBufferSize,
+				      movieWidth, movieHeight,
+				      movieFPS,
+				      packetLossCompensate);
+      if (aviOut == NULL) {
+	*env << "Failed to create AVI file sink for stdout: " << env->getResultMsg();
+	shutdown();
       }
+
+      aviOut->startPlaying(sessionAfterPlaying, NULL);
+#ifdef SUPPORT_REAL_RTSP
+    } else if (session->isRealNetworksRDT) {
+      // For RealNetworks' sessions, we create a single output file,
+      // named "output.rm".
+      char outFileName[1000];
+      if (singleMedium == NULL) {
+	snprintf(outFileName, sizeof outFileName, "%soutput.rm", fileNamePrefix);
+      } else {
+	// output to 'stdout' as normal, even though we actually output all media
+	sprintf(outFileName, "stdout");
+      }
+      FileSink* fileSink = FileSink::createNew(*env, outFileName,
+					       fileSinkBufferSize, oneFilePerFrame);
+
+      // The output file needs to begin with a special 'RMFF' header,
+      // in order for it to be usable.  Write this header first:
+      unsigned headerSize;
+      unsigned char* headerData = RealGenerateRMFFHeader(session, headerSize);
+      struct timeval timeNow;
+      gettimeofday(&timeNow, NULL);
+      fileSink->addData(headerData, headerSize, timeNow);
+      delete[] headerData;
+
+      // Start playing the output file from the first subsession.
+      // (Hack: Because all subsessions' data is actually multiplexed on the
+      // single RTSP TCP connection, playing from one subsession is sufficient.)
+      iter.reset();
+      madeProgress = False;
+      while ((subsession = iter.next()) != NULL) {
+	if (subsession->readSource() == NULL) continue; // was not initiated
+
+	  fileSink->startPlaying(*(subsession->readSource()),
+					 subsessionAfterPlaying,
+					 subsession);
+	  madeProgress = True;
+	  break; // play from one subsession only
+      }
+      if (!madeProgress) shutdown();
+#endif
     } else {
       // Create and start "FileSink"s for each subsession:
       madeProgress = False;
@@ -602,23 +716,9 @@ int main(int argc, char** argv) {
 	    unsigned char* configData
 	      = parseGeneralConfigStr(subsession->fmtp_config(), configLen);
 	    struct timeval timeNow;
-	    gettimeofday(&timeNow, &Idunno);
+	    gettimeofday(&timeNow, NULL);
 	    fileSink->addData(configData, configLen, timeNow);
 	    delete[] configData;
-#ifdef SUPPORT_REAL_RTSP
-	  } else if (strcmp(subsession->codecName(), "X-PN-REALAUDIO") == 0 ||
-		     strcmp(subsession->codecName(), "X-PN-REALVIDEO") == 0) {
-	    // For RealNetworks' special streams, the output file needs to begin
-	    // with a special header, in order for it to be usable.
-	    // Write this header first:
-	    unsigned headerSize;
-	    unsigned char* headerData
-	      = RealGenerateRMFFHeader(session, subsession, headerSize);
-	    struct timeval timeNow;
-	    gettimeofday(&timeNow, &Idunno);
-	    fileSink->addData(headerData, headerSize, timeNow);
-	    delete[] headerData;
-#endif
 	  }
 
 	  subsession->sink->startPlaying(*(subsession->readSource()),
@@ -739,6 +839,7 @@ void tearDownStreams() {
 
 void closeMediaSinks() {
   Medium::close(qtOut);
+  Medium::close(aviOut);
 
   if (session == NULL) return;
   MediaSubsessionIterator iter(*session);
@@ -768,7 +869,7 @@ void subsessionAfterPlaying(void* clientData) {
 
 void subsessionByeHandler(void* clientData) {
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   unsigned secsDiff = timeNow.tv_sec - startTime.tv_sec;
 
   MediaSubsession* subsession = (MediaSubsession*)clientData;
@@ -806,6 +907,15 @@ public:
       totNumPacketsReceived(0), totNumPacketsExpected(0) {
     measurementEndTime = measurementStartTime = startTime;
 
+#ifdef SUPPORT_REAL_RTSP
+    if (session->isRealNetworksRDT) { // hack for RealMedia sessions (RDT, not RTP)
+      RealRDTSource* rdt = (RealRDTSource*)src;
+      kBytesTotal = rdt->totNumKBytesReceived();
+      totNumPacketsReceived = rdt->totNumPacketsReceived();
+      totNumPacketsExpected = totNumPacketsReceived; // because we use TCP
+      return;
+    }
+#endif
     RTPReceptionStatsDB::Iterator statsIter(src->receptionStatsDB());
     // Assume that there's only one SSRC source (usually the case):
     RTPReceptionStats* stats = statsIter.next(True);
@@ -840,7 +950,7 @@ static unsigned nextQOSMeasurementUSecs;
 static void scheduleNextQOSMeasurement() {
   nextQOSMeasurementUSecs += qosMeasurementIntervalMS*1000;
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   unsigned timeNowUSecs = timeNow.tv_sec*1000000 + timeNow.tv_usec;
   unsigned usecsToDelay = nextQOSMeasurementUSecs < timeNowUSecs ? 0
     : nextQOSMeasurementUSecs - timeNowUSecs;
@@ -851,7 +961,7 @@ static void scheduleNextQOSMeasurement() {
 
 static void periodicQOSMeasurement(void* /*clientData*/) {
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
 
   for (qosMeasurementRecord* qosRecord = qosRecordHead;
        qosRecord != NULL; qosRecord = qosRecord->fNext) {
@@ -869,6 +979,24 @@ void qosMeasurementRecord
   double timeDiff = secsDiff + usecsDiff/1000000.0;
   measurementEndTime = timeNow;
 
+#ifdef SUPPORT_REAL_RTSP
+  if (session->isRealNetworksRDT) { // hack for RealMedia sessions (RDT, not RTP)
+    RealRDTSource* rdt = (RealRDTSource*)fSource;
+    double kBytesTotalNow = rdt->totNumKBytesReceived();
+    double kBytesDeltaNow = kBytesTotalNow - kBytesTotal;
+    kBytesTotal = kBytesTotalNow;
+
+    double kbpsNow = timeDiff == 0.0 ? 0.0 : 8*kBytesDeltaNow/timeDiff;
+    if (kbpsNow < 0.0) kbpsNow = 0.0; // in case of roundoff error
+    if (kbpsNow < kbits_per_second_min) kbits_per_second_min = kbpsNow;
+    if (kbpsNow > kbits_per_second_max) kbits_per_second_max = kbpsNow;
+
+    totNumPacketsReceived = rdt->totNumPacketsReceived();
+    totNumPacketsExpected = totNumPacketsReceived; // because we use TCP
+    packet_loss_fraction_min = packet_loss_fraction_max = 0.0; // ditto
+    return;
+  }
+#endif
   RTPReceptionStatsDB::Iterator statsIter(fSource->receptionStatsDB());
   // Assume that there's only one SSRC source (usually the case):
   RTPReceptionStats* stats = statsIter.next(True);
@@ -904,13 +1032,16 @@ void qosMeasurementRecord
 void beginQOSMeasurement() {
   // Set up a measurement record for each active subsession:
   struct timeval startTime;
-  gettimeofday(&startTime, &Idunno);
+  gettimeofday(&startTime, NULL);
   nextQOSMeasurementUSecs = startTime.tv_sec*1000000 + startTime.tv_usec;
   qosMeasurementRecord* qosRecordTail = NULL;
   MediaSubsessionIterator iter(*session);
   MediaSubsession* subsession;
   while ((subsession = iter.next()) != NULL) {
     RTPSource* src = subsession->rtpSource();
+#ifdef SUPPORT_REAL_RTSP
+    if (session->isRealNetworksRDT) src = (RTPSource*)(subsession->readSource()); // hack
+#endif
     if (src == NULL) continue;
 
     qosMeasurementRecord* qosRecord
@@ -937,67 +1068,85 @@ void printQOSData(int exitCode) {
     MediaSubsession* subsession;
     while ((subsession = iter.next()) != NULL) {
       RTPSource* src = subsession->rtpSource();
+#ifdef SUPPORT_REAL_RTSP
+      if (session->isRealNetworksRDT) src = (RTPSource*)(subsession->readSource()); // hack
+#endif
       if (src == NULL) continue;
 
       *env << "subsession\t" << subsession->mediumName()
-		<< "/" << subsession->codecName() << "\n";
-
+	   << "/" << subsession->codecName() << "\n";
+      
       unsigned numPacketsReceived = 0, numPacketsExpected = 0;
-
+      
       if (curQOSRecord != NULL) {
-		numPacketsReceived = curQOSRecord->totNumPacketsReceived;
-		numPacketsExpected = curQOSRecord->totNumPacketsExpected;
+	numPacketsReceived = curQOSRecord->totNumPacketsReceived;
+	numPacketsExpected = curQOSRecord->totNumPacketsExpected;
       }
       *env << "num_packets_received\t" << numPacketsReceived << "\n";
       *env << "num_packets_lost\t" << numPacketsExpected - numPacketsReceived << "\n";
       
       if (curQOSRecord != NULL) {
-		unsigned secsDiff = curQOSRecord->measurementEndTime.tv_sec
-		 - curQOSRecord->measurementStartTime.tv_sec;
-		int usecsDiff = curQOSRecord->measurementEndTime.tv_usec
-		 - curQOSRecord->measurementStartTime.tv_usec;
-		double measurementTime = secsDiff + usecsDiff/1000000.0;
-		*env << "elapsed_measurement_time\t" << measurementTime << "\n";
+	unsigned secsDiff = curQOSRecord->measurementEndTime.tv_sec
+	  - curQOSRecord->measurementStartTime.tv_sec;
+	int usecsDiff = curQOSRecord->measurementEndTime.tv_usec
+	  - curQOSRecord->measurementStartTime.tv_usec;
+	double measurementTime = secsDiff + usecsDiff/1000000.0;
+	*env << "elapsed_measurement_time\t" << measurementTime << "\n";
 	
-		*env << "kBytes_received_total\t" << curQOSRecord->kBytesTotal << "\n";
+	*env << "kBytes_received_total\t" << curQOSRecord->kBytesTotal << "\n";
 	
-		*env << "measurement_sampling_interval_ms\t" << qosMeasurementIntervalMS << "\n";
+	*env << "measurement_sampling_interval_ms\t" << qosMeasurementIntervalMS << "\n";
 	
-		if (curQOSRecord->kbits_per_second_max == 0) {
-			// special case: we didn't receive any data:
-			*env <<
-			"kbits_per_second_min\tunavailable\n"
-			"kbits_per_second_ave\tunavailable\n"
-			"kbits_per_second_max\tunavailable\n";
-		} else {
-			*env << "kbits_per_second_min\t" << curQOSRecord->kbits_per_second_min << "\n";
-			*env << "kbits_per_second_ave\t"
-				<< (measurementTime == 0.0 ? 0.0 : 8*curQOSRecord->kBytesTotal/measurementTime) << "\n";
-			*env << "kbits_per_second_max\t" << curQOSRecord->kbits_per_second_max << "\n";
-		}
+	if (curQOSRecord->kbits_per_second_max == 0) {
+	  // special case: we didn't receive any data:
+	  *env <<
+	    "kbits_per_second_min\tunavailable\n"
+	    "kbits_per_second_ave\tunavailable\n"
+	    "kbits_per_second_max\tunavailable\n";
+	} else {
+	  *env << "kbits_per_second_min\t" << curQOSRecord->kbits_per_second_min << "\n";
+	  *env << "kbits_per_second_ave\t"
+	       << (measurementTime == 0.0 ? 0.0 : 8*curQOSRecord->kBytesTotal/measurementTime) << "\n";
+	  *env << "kbits_per_second_max\t" << curQOSRecord->kbits_per_second_max << "\n";
+	}
 	
-		*env << "packet_loss_percentage_min\t" << 100*curQOSRecord->packet_loss_fraction_min << "\n";
-		double packetLossFraction = numPacketsExpected == 0 ? 1.0
-			: 1.0 - numPacketsReceived/(double)numPacketsExpected;
-		if (packetLossFraction < 0.0) packetLossFraction = 0.0;
-		*env << "packet_loss_percentage_ave\t" << 100*packetLossFraction << "\n";
-		*env << "packet_loss_percentage_max\t"
-			<< (packetLossFraction == 1.0 ? 100.0 : 100*curQOSRecord->packet_loss_fraction_max) << "\n";
+	*env << "packet_loss_percentage_min\t" << 100*curQOSRecord->packet_loss_fraction_min << "\n";
+	double packetLossFraction = numPacketsExpected == 0 ? 1.0
+	  : 1.0 - numPacketsReceived/(double)numPacketsExpected;
+	if (packetLossFraction < 0.0) packetLossFraction = 0.0;
+	*env << "packet_loss_percentage_ave\t" << 100*packetLossFraction << "\n";
+	*env << "packet_loss_percentage_max\t"
+	     << (packetLossFraction == 1.0 ? 100.0 : 100*curQOSRecord->packet_loss_fraction_max) << "\n";
 	
-		RTPReceptionStatsDB::Iterator statsIter(src->receptionStatsDB());
-		// Assume that there's only one SSRC source (usually the case):
-		RTPReceptionStats* stats = statsIter.next(True);
-		if (stats != NULL) {
-			*env << "inter_packet_gap_ms_min\t" << stats->minInterPacketGapUS()/1000.0 << "\n";
-			struct timeval totalGaps = stats->totalInterPacketGaps();
-			double totalGapsMS = totalGaps.tv_sec*1000.0 + totalGaps.tv_usec/1000.0;
-			unsigned totNumPacketsReceived = stats->totNumPacketsReceived();
-			*env << "inter_packet_gap_ms_ave\t"
-				<< (totNumPacketsReceived == 0 ? 0.0 : totalGapsMS/totNumPacketsReceived) << "\n";
-			*env << "inter_packet_gap_ms_max\t" << stats->maxInterPacketGapUS()/1000.0 << "\n";
-		}
-
-		curQOSRecord = curQOSRecord->fNext;
+#ifdef SUPPORT_REAL_RTSP
+	if (session->isRealNetworksRDT) {
+	  RealRDTSource* rdt = (RealRDTSource*)src;
+	  *env << "inter_packet_gap_ms_min\t" << rdt->minInterPacketGapUS()/1000.0 << "\n";
+	  struct timeval totalGaps = rdt->totalInterPacketGaps();
+	  double totalGapsMS = totalGaps.tv_sec*1000.0 + totalGaps.tv_usec/1000.0;
+	  unsigned totNumPacketsReceived = rdt->totNumPacketsReceived();
+	  *env << "inter_packet_gap_ms_ave\t"
+	       << (totNumPacketsReceived == 0 ? 0.0 : totalGapsMS/totNumPacketsReceived) << "\n";
+	  *env << "inter_packet_gap_ms_max\t" << rdt->maxInterPacketGapUS()/1000.0 << "\n";
+	} else {
+#endif
+	  RTPReceptionStatsDB::Iterator statsIter(src->receptionStatsDB());
+	  // Assume that there's only one SSRC source (usually the case):
+	  RTPReceptionStats* stats = statsIter.next(True);
+	  if (stats != NULL) {
+	    *env << "inter_packet_gap_ms_min\t" << stats->minInterPacketGapUS()/1000.0 << "\n";
+	    struct timeval totalGaps = stats->totalInterPacketGaps();
+	    double totalGapsMS = totalGaps.tv_sec*1000.0 + totalGaps.tv_usec/1000.0;
+	    unsigned totNumPacketsReceived = stats->totNumPacketsReceived();
+	    *env << "inter_packet_gap_ms_ave\t"
+		 << (totNumPacketsReceived == 0 ? 0.0 : totalGapsMS/totNumPacketsReceived) << "\n";
+	    *env << "inter_packet_gap_ms_max\t" << stats->maxInterPacketGapUS()/1000.0 << "\n";
+	  }
+#ifdef SUPPORT_REAL_RTSP
+	}
+#endif
+	
+	curQOSRecord = curQOSRecord->fNext;
       }
     }
   }    
@@ -1062,10 +1211,12 @@ void checkForPacketArrival(void* /*clientData*/) {
   }
 
   unsigned numSubsessionsToCheck = numSubsessionsChecked;
+  // Special case for "QuickTimeFileSink"s and "AVIFileSink"s:
+  // They might not use all of the input sources:
   if (qtOut != NULL) {
-    // Special case for "QuickTimeFileSink"s: They might not use all of the
-    // input sources:
     numSubsessionsToCheck = qtOut->numActiveSubsessions();
+  } else if (aviOut != NULL) {
+    numSubsessionsToCheck = aviOut->numActiveSubsessions();
   }
 
   Boolean notifyTheUser;
@@ -1078,7 +1229,7 @@ void checkForPacketArrival(void* /*clientData*/) {
   }
   if (notifyTheUser) {
     struct timeval timeNow;
-    gettimeofday(&timeNow, &Idunno);
+    gettimeofday(&timeNow, NULL);
 	char timestampStr[100];
 	sprintf(timestampStr, "%ld%03ld", timeNow.tv_sec, timeNow.tv_usec/1000);
     *env << (syncStreams ? "Synchronized d" : "D")
@@ -1121,194 +1272,3 @@ void checkInterPacketGaps(void* /*clientData*/) {
 				 (TaskFunc*)checkInterPacketGaps, NULL);
   }
 }
-
-// WORK IN PROGRESS #####
-class RTPTranslator: public FramedFilter {
-public:
-  static RTPTranslator* createNew(UsageEnvironment& env,
-				  FramedSource* source);
-
-private:
-  RTPTranslator(UsageEnvironment& env, FramedSource* source);
-  virtual ~RTPTranslator();
-
-  static void afterGettingFrame(void* clientData,
-                                unsigned numBytesRead,
-				unsigned numTruncatedBytes,
-                                struct timeval presentationTime,
-				unsigned durationInMicroseconds);
-  void afterGettingFrame1(unsigned numBytesRead,
-			  unsigned numTruncatedBytes,
-			  struct timeval presentationTime,
-			  unsigned durationInMicroseconds);
-
-private: // redefined virtual function:
-  virtual void doGetNextFrame();
-
-private:
-  //unsigned char fBuffer[50000];//##### Later: parameterize
-};
-
-RTPTranslator* RTPTranslator::createNew(UsageEnvironment& env,
-					FramedSource* source) {
-  // Check whether source is a "RTPSource"??? #####
-  return new RTPTranslator(env, source);
-}
-
-RTPTranslator::RTPTranslator(UsageEnvironment& env, FramedSource* source)
-  : FramedFilter(env, source) {
-}
-
-RTPTranslator::~RTPTranslator() {
-}
-
-void RTPTranslator::doGetNextFrame() {
-  // For now, do a direct relay #####
-  fInputSource->getNextFrame(fTo, fMaxSize,
-			     afterGettingFrame, this,
-			     handleClosure, this);
-}
-
-void RTPTranslator::afterGettingFrame(void* clientData,
-				      unsigned numBytesRead,
-				      unsigned numTruncatedBytes,
-				      struct timeval presentationTime,
-				      unsigned durationInMicroseconds) {
-  RTPTranslator* rtpTranslator = (RTPTranslator*)clientData;
-  rtpTranslator->afterGettingFrame1(numBytesRead, numTruncatedBytes,
-				    presentationTime, durationInMicroseconds);
-}
-
-void RTPTranslator::afterGettingFrame1(unsigned numBytesRead,
-				       unsigned numTruncatedBytes,
-				       struct timeval presentationTime,
-				       unsigned durationInMicroseconds) {
-  fFrameSize = numBytesRead;
-  fPresentationTime = presentationTime;
-  fNumTruncatedBytes = numTruncatedBytes;
-  fDurationInMicroseconds = durationInMicroseconds;
-  afterGetting(this);
-}
-
-//#####
-RTSPClient* rtspClientOutgoing = NULL;
-Boolean setupDestinationRTSPServer() {
-  do {
-    rtspClientOutgoing
-      = RTSPClient::createNew(*env, verbosityLevel, progName);
-    if (rtspClientOutgoing == NULL) break;
-
-    // Construct the SDP description to announce into the RTSP server:
-
-    // First, get our own IP address, and that of the RTSP server:
-    struct in_addr ourIPAddress;
-    ourIPAddress.s_addr = ourSourceAddressForMulticast(*env);
-    char* ourIPAddressStr = strDup(our_inet_ntoa(ourIPAddress));
-
-    NetAddress serverAddress;
-    portNumBits serverPortNum;
-    if (!RTSPClient::parseRTSPURL(*env, destRTSPURL,
-				  serverAddress, serverPortNum)) break;
-    struct in_addr serverIPAddress;
-    serverIPAddress.s_addr = *(unsigned*)(serverAddress.data());
-    char* serverIPAddressStr = strDup(our_inet_ntoa(serverIPAddress));
-
-    char const* destSDPFmt =
-      "v=0\r\n"
-      "o=- %u %u IN IP4 %s\r\n"
-      "s=RTSP session, relayed through \"%s\"\n"
-      "i=relayed RTSP session\n"
-      "t=0 0\n"
-      "c=IN IP4 %s\n"
-      "a=control:*\n"
-      "m=audio 0 RTP/AVP %u\n"
-      "a=control:trackID=0\n";
-    //#####LATER: Support video as well; multiple tracks; other codecs #####
-    unsigned destSDPFmtSize = strlen(destSDPFmt)
-      + 20 /* max int len */ + 20 + strlen(ourIPAddressStr)
-      + strlen(progName)
-      + strlen(serverIPAddressStr)
-      + 3 /* max char len */;
-    char* destSDPDescription = new char[destSDPFmtSize];
-    sprintf(destSDPDescription, destSDPFmt,
-	    our_random(), our_random(), ourIPAddressStr,
-	    progName,
-	    serverIPAddressStr,
-	    desiredAudioRTPPayloadFormat);
-    Boolean announceResult;
-    if (username != NULL) {
-      announceResult
-	= rtspClientOutgoing->announceWithPassword(destRTSPURL,
-						   destSDPDescription,
-						   username, password);
-    } else {
-      announceResult
-	= rtspClientOutgoing->announceSDPDescription(destRTSPURL,
-						     destSDPDescription);
-    }
-    delete[] serverIPAddressStr; delete[] ourIPAddressStr;
-    if (!announceResult) break;
-    
-    // Then, create a "MediaSession" object from this SDP description:
-    MediaSession* destSession
-      = MediaSession::createNew(*env, destSDPDescription);
-    delete[] destSDPDescription;
-    if (destSession == NULL) break;
-
-    // Initiate, setup and play "destSession".
-    // ##### TEMP HACK - take advantage of the fact that we have
-    // ##### a single audio session only.
-    MediaSubsession* destSubsession;
-    PrioritizedRTPStreamSelector* multiSource;
-    int multiSourceSessionId;
-    char const* mimeType
-      = desiredAudioRTPPayloadFormat == 0 ? "audio/PCMU" 
-      : desiredAudioRTPPayloadFormat == 3 ? "audio/GSM"
-      : "audio/???"; //##### FIX
-    if (!destSession->initiateByMediaType(mimeType, destSubsession,
-					  multiSource,
-					  multiSourceSessionId)) break;
-    if (!rtspClientOutgoing->setupMediaSubsession(*destSubsession,
-						  True, True)) break;
-    if (!rtspClientOutgoing->playMediaSubsession(*destSubsession,
-						 0.0, -1.0,
-						 True/*hackForDSS*/)) break;
-
-    // Next, set up "RTPSink"s for the outgoing packets:
-    struct in_addr destAddr; destAddr.s_addr = 0; // because we're using TCP
-    Groupsock* destGS = new Groupsock(*env, destAddr, 0/*aud*/, 255);
-    if (destGS == NULL) break;
-    RTPSink* destRTPSink = NULL;
-    if (desiredAudioRTPPayloadFormat == 0) {
-      destRTPSink = SimpleRTPSink::createNew(*env, destGS, 0, 8000,
-					     "audio", "PCMU");
-    } else if (desiredAudioRTPPayloadFormat == 3) {
-      destRTPSink = GSMAudioRTPSink::createNew(*env, destGS);
-    }
-    if (destRTPSink == NULL) break;
-
-    // Tell the sink to stream using TCP:
-    destRTPSink->setStreamSocket(rtspClientOutgoing->socketNum(), 0/*aud*/);
-    // LATER: set up RTCPInstance also #####
-
-    // Next, set up RTPTranslator(s) between source(s) and destination(s),
-    // and start playing them.
-    MediaSubsessionIterator iter(*session);
-    MediaSubsession *sourceSubsession = NULL;
-    while ((sourceSubsession = iter.next()) != NULL) {
-      if (strcmp(sourceSubsession->mediumName(), "audio") == 0) break;
-    }
-    if (sourceSubsession == NULL) break;
-    RTPTranslator* rtpTranslator
-      = RTPTranslator::createNew(*env, sourceSubsession->readSource());
-    if (rtpTranslator == NULL) break;
-    destRTPSink->startPlaying(*rtpTranslator,
-			      subsessionAfterPlaying, sourceSubsession);
-    
-    // LATER: delete media on close #####
-
-    return True;
-  } while (0);
-
-  return False;
-}
