@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2003, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2006, Live Networks, Inc.  All rights reserved
 // A SIP client test program that opens a SIP URL argument,
 // and extracts the data from each incoming RTP stream.
 
@@ -39,7 +39,8 @@ Medium* createClient(UsageEnvironment& env,
 			      verbosityLevel, applicationName);
 }
 
-char* getOptionsResponse(Medium* client, char const* url) {
+char* getOptionsResponse(Medium* client, char const* url,
+			 char* username, char* password) {
   SIPClient* sipClient = (SIPClient*)client;
   sipClient->envir().setResultMsg("NOT SUPPORTED IN CLIENT");//#####
   return NULL;//#####
