@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2003, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2006, Live Networks, Inc.  All rights reserved
 // A test program that reads a MPEG-4 Video Elementary Stream file,
 // and streams it using RTP
 // main program
@@ -23,7 +23,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #include "GroupsockHelper.hh"
 
 UsageEnvironment* env;
-char const* inputFileName = "test.m4v";
+char const* inputFileName = "test.m4e";
 MPEG4VideoStreamFramer* videoSource;
 RTPSink* videoSink;
 
@@ -57,19 +57,19 @@ int main(int argc, char** argv) {
   videoSink = MPEG4ESVideoRTPSink::createNew(*env, &rtpGroupsock, 96);
 
   // Create (and start) a 'RTCP instance' for this RTP sink:
-  const unsigned totalSessionBandwidth = 500; // in kbps; for RTCP b/w share
+  const unsigned estimatedSessionBandwidth = 500; // in kbps; for RTCP b/w share
   const unsigned maxCNAMElen = 100;
   unsigned char CNAME[maxCNAMElen+1];
   gethostname((char*)CNAME, maxCNAMElen);
   CNAME[maxCNAMElen] = '\0'; // just in case
   RTCPInstance* rtcp
   = RTCPInstance::createNew(*env, &rtcpGroupsock,
-			    totalSessionBandwidth, CNAME,
+			    estimatedSessionBandwidth, CNAME,
 			    videoSink, NULL /* we're a server */,
 			    True /* we're a SSM source */);
   // Note: This starts RTCP running automatically
 
-  RTSPServer* rtspServer = RTSPServer::createNew(*env, 7070);
+  RTSPServer* rtspServer = RTSPServer::createNew(*env, 8554);
   if (rtspServer == NULL) {
     *env << "Failed to create RTSP server: " << env->getResultMsg() << "\n";
     exit(1);
@@ -100,6 +100,7 @@ void afterPlaying(void* /*clientData*/) {
   Medium::close(videoSource);
   // Note that this also closes the input file that this source read from.
 
+  // Start playing once again:
   play();
 }
 
