@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2004, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2006, Live Networks, Inc.  All rights reserved
 // A test program that demonstrates how to stream - via unicast RTP
 // - various kinds of file on demand, using a built-in RTSP server.
 // main program
@@ -23,22 +23,40 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 UsageEnvironment* env;
 
+// To make the second and subsequent client for each stream reuse the same
+// input stream as the first client (rather than playing the file from the
+// start for each client), change the following "False" to "True":
+Boolean reuseFirstSource = False;
+
 // To stream *only* MPEG-1 or 2 video "I" frames
 // (e.g., to reduce network bandwidth),
 // change the following "False" to "True":
 Boolean iFramesOnly = False;
 
+static void announceStream(RTSPServer* rtspServer, ServerMediaSession* sms,
+			   char const* streamName, char const* inputFileName); // fwd
+
 int main(int argc, char** argv) {
   // Begin by setting up our usage environment:
   TaskScheduler* scheduler = BasicTaskScheduler::createNew();
   env = BasicUsageEnvironment::createNew(*scheduler);
 
+  UserAuthenticationDatabase* authDB = NULL;
+#ifdef ACCESS_CONTROL
+  // To implement client access control to the RTSP server, do the following:
+  authDB = new UserAuthenticationDatabase;
+  authDB->addUserRecord("username1", "password1"); // replace these with real strings
+  // Repeat the above with each <username>, <password> that you wish to allow
+  // access to the server.
+#endif
+
   // Create the RTSP server:
-  RTSPServer* rtspServer = RTSPServer::createNew(*env, 7070);
+  RTSPServer* rtspServer = RTSPServer::createNew(*env, 8554, authDB, 45);
   if (rtspServer == NULL) {
     *env << "Failed to create RTSP server: " << env->getResultMsg() << "\n";
     exit(1);
   }
+
   char const* descriptionString
     = "Session streamed by \"testOnDemandRTSPServer\"";
 
@@ -50,19 +68,15 @@ int main(int argc, char** argv) {
   // A MPEG-4 video elementary stream:
   {
     char const* streamName = "mpeg4ESVideoTest";
-    char const* inputFileName = "test.m4v";
+    char const* inputFileName = "test.m4e";
     ServerMediaSession* sms
       = ServerMediaSession::createNew(*env, streamName, streamName,
 				      descriptionString);
     sms->addSubsession(MPEG4VideoFileServerMediaSubsession
-		       ::createNew(*env, inputFileName));
+		       ::createNew(*env, inputFileName, reuseFirstSource));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A MPEG-1 or 2 audio+video program stream:
@@ -74,16 +88,12 @@ int main(int argc, char** argv) {
       = ServerMediaSession::createNew(*env, streamName, streamName,
 				      descriptionString);
     MPEG1or2FileServerDemux* demux
-      = MPEG1or2FileServerDemux::createNew(*env, inputFileName);
+      = MPEG1or2FileServerDemux::createNew(*env, inputFileName, reuseFirstSource);
     sms->addSubsession(demux->newVideoServerMediaSubsession(iFramesOnly));
     sms->addSubsession(demux->newAudioServerMediaSubsession());
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A MPEG-1 or 2 video elementary stream:
@@ -95,14 +105,10 @@ int main(int argc, char** argv) {
       = ServerMediaSession::createNew(*env, streamName, streamName,
 				      descriptionString);
     sms->addSubsession(MPEG1or2VideoFileServerMediaSubsession
-		       ::createNew(*env, inputFileName, iFramesOnly));
+	       ::createNew(*env, inputFileName, reuseFirstSource, iFramesOnly));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A MP3 audio stream (actually, any MPEG-1 or 2 audio file will work):
@@ -111,7 +117,7 @@ int main(int argc, char** argv) {
   // To also reorder ADUs before streaming, uncomment the following:
 //#define INTERLEAVE_ADUS 1
   // (For more information about ADUs and interleaving,
-  //  see <http://www.live.com/rtp-mp3/>)
+  //  see <http://www.live555.com/rtp-mp3/>)
   {
     char const* streamName = "mp3AudioTest";
     char const* inputFileName = "test.mp3";
@@ -130,15 +136,11 @@ int main(int argc, char** argv) {
 #endif
 #endif
     sms->addSubsession(MP3AudioFileServerMediaSubsession
-		       ::createNew(*env, inputFileName,
+		       ::createNew(*env, inputFileName, reuseFirstSource,
 				   useADUs, interleaving));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A WAV audio stream:
@@ -152,14 +154,10 @@ int main(int argc, char** argv) {
     // change the following to True:
     Boolean convertToULaw = False;
     sms->addSubsession(WAVAudioFileServerMediaSubsession
-		       ::createNew(*env, inputFileName, convertToULaw));
+	       ::createNew(*env, inputFileName, reuseFirstSource, convertToULaw));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // An AMR audio stream:
@@ -170,14 +168,10 @@ int main(int argc, char** argv) {
       = ServerMediaSession::createNew(*env, streamName, streamName,
 				      descriptionString);
     sms->addSubsession(AMRAudioFileServerMediaSubsession
-		       ::createNew(*env, inputFileName));
+		       ::createNew(*env, inputFileName, reuseFirstSource));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A 'VOB' file (e.g., from an unencrypted DVD):
@@ -189,19 +183,53 @@ int main(int argc, char** argv) {
 				      descriptionString);
     // Note: VOB files are MPEG-2 Program Stream files, but using AC-3 audio
     MPEG1or2FileServerDemux* demux
-      = MPEG1or2FileServerDemux::createNew(*env, inputFileName);
+      = MPEG1or2FileServerDemux::createNew(*env, inputFileName, reuseFirstSource);
     sms->addSubsession(demux->newVideoServerMediaSubsession(iFramesOnly));
     sms->addSubsession(demux->newAC3AudioServerMediaSubsession());
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
+  }
+
+  // A MPEG-2 Transport Stream:
+  {
+    char const* streamName = "mpeg2TransportStreamTest";
+    char const* inputFileName = "test.ts";
+    ServerMediaSession* sms
+      = ServerMediaSession::createNew(*env, streamName, streamName,
+				      descriptionString);
+    sms->addSubsession(MPEG2TransportFileServerMediaSubsession
+		       ::createNew(*env, inputFileName, reuseFirstSource));
+    rtspServer->addServerMediaSession(sms);
+
+    announceStream(rtspServer, sms, streamName, inputFileName);
+  }
+
+  // An AAC audio stream (ADTS-format file):
+  {
+    char const* streamName = "aacAudioTest";
+    char const* inputFileName = "test.aac";
+    ServerMediaSession* sms
+      = ServerMediaSession::createNew(*env, streamName, streamName,
+				      descriptionString);
+    sms->addSubsession(ADTSAudioFileServerMediaSubsession
+		       ::createNew(*env, inputFileName, reuseFirstSource));
+    rtspServer->addServerMediaSession(sms);
+
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   env->taskScheduler().doEventLoop(); // does not return
 
   return 0; // only to prevent compiler warning
 }
+
+static void announceStream(RTSPServer* rtspServer, ServerMediaSession* sms,
+			   char const* streamName, char const* inputFileName) {
+  char* url = rtspServer->rtspURL(sms);
+  UsageEnvironment& env = rtspServer->envir(); 
+  env << "\n\"" << streamName << "\" stream, from the file \""
+      << inputFileName << "\"\n";
+  env << "Play this stream using the URL \"" << url << "\"\n";
+  delete[] url;
+}
